package de.bsvrz.buv.plugin.bmvew;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse speichert das Element, das via Drag and Drop eingefgt werden soll. <br>
 * Die realisierung erfolgt als Singleton.
 * 
 * @author HofmannJ
 * 
 */
public class SpeicherTransfer
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Speichert die Singleton-Instanz.
   */
  private static SpeicherTransfer _instanz = null;

  /**
   * Speichert das {@link SystemObject} fr den DND-Vorgang
   */
  private Meldung _speicherMeldung = null;

  /**
   * Speichert die altePosition
   */
  private int _altePosition = -1;

  /**
   * Speichert die neue Position
   */
  private int _neuePosition = -1;

  /**
   * Liefert die Singleton-Instanz der Klasse {@link SpeicherTransfer}.
   * 
   * @return Singleton-Instanz der Klasse {@link SpeicherTransfer}.
   */
  public static SpeicherTransfer getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new SpeicherTransfer();
    }

    return _instanz;
  }

  /**
   * Privater Konstruktor
   * 
   */
  private SpeicherTransfer()
  {
  }

  /**
   * Setzt das zu speichernde {@link SystemObject}.
   * 
   * @param systemObject
   *          Zu speicherndes {@link SystemObject}.
   */
  public void setMeldung(Meldung meldung)
  {
    this._speicherMeldung = meldung;
  }

  /**
   * Liefert das gespeicherte {@link SystemObject}.
   * 
   * @return Gespeichertes {@link SystemObject}.
   */
  public Meldung getMeldung()
  {
    return _speicherMeldung;
  }

  /**
   * Liefert die Pid des gespeicherten {@link SystemObject}.
   * 
   * @return Pid des gespeicherten {@link SystemObject}.
   */
  public String getPid()
  {
    if (_speicherMeldung != null)
    {
      return _speicherMeldung.getPid();
    }
    else
    {
      return null;
    }
  }

  /**
   * Setzt die Alte Position
   * 
   * @param altePosition
   */
  public void setAltePosition(int altePosition)
  {
    this._altePosition = altePosition;
    _debug.fine("ALTE POSITION: " + _altePosition);

  }

  public int getAltePosition()
  {
    return _altePosition;
  }

  public void setNeuePosition(int neuePosition)
  {
    this._neuePosition = neuePosition;
  }

  public int getNeuePosition()
  {
    return _neuePosition;
  }

  /**
   * Setzt den Speicher fr das {@link SystemObject} auf <code>null</code> zurck.
   * 
   */
  public void zuruecksetzenSpeicher()
  {
    _speicherMeldung = null;
    _neuePosition = -1;
    _altePosition = -1;
  }

}
