package de.bsvrz.buv.plugin.ereigniskal;

import java.util.List;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Definitionsdialog fr raumliche Gueltigkeiten. Die Klasse liefert ein Eingabefelder, in denen die rauemliche
 * Gltigkeit bestimmt werden kann. <br>
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @version $Revision: 1.4 $ / $Date: 2010/02/04 05:57:26 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class DefinitionRaeumlicheGueltigkeit extends Dialog
{

  private Shell shell = null; // @jve:decl-index=0:visual-constraint="204,120"

  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Button buttonOk = null;

  private Button buttonAbbrechen = null;

  private Label label = null;

  private Combo combo = null;

  private ComboViewer comboViewer;

  private List<SystemObject> _liste;

  /**
   * Speichert die in diesem Dialog erfassten Daten.
   */
  private EintragRaeumlicheGueltigkeit ereigniskalenderDatensatzRaeumlicheGueltigkeit = null;

  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link DefinitionRaeumlicheGueltigkeit}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public DefinitionRaeumlicheGueltigkeit(Shell parent)
  {
    super(parent);

  }

  /**
   * ffnet den Dialog {@link DefinitionRaeumlicheGueltigkeit} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public EintragRaeumlicheGueltigkeit oeffne()
  {
    // shell = new Shell();
    shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.CLOSE);
    shell.setText("Definition rumliche Gltigkeit (Infrastrukturobjekt)");

    shell.setLayout(new GridLayout());
    createCompositeTop();
    createCompositeBottom();
    shell.setSize(new Point(436, 103));
    shell.open();

    holeNetzBestandteile();

    Display display = getParent().getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return ereigniskalenderDatensatzRaeumlicheGueltigkeit;

  }

  /**
   * Generiert die Elemente des Dialogs {@link DefinitionRaeumlicheGueltigkeit}.
   * 
   */
  private void generiereGui()
  {
  }

  /**
   * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
   * 
   */
  private void aktionOk()
  {

    if (ereigniskalenderDatensatzRaeumlicheGueltigkeit != null)
      shell.dispose();
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler");
      mb.setMessage("Es muss ein Netzbestandteil ausgewhlt werden!");
      mb.open();
      return;
    }
  }

  /**
   * Verwirft die nderungen an den raeumlichen Gueltigkeiten und schliet den Dialog.
   * 
   */
  private void aktionAbbrechen()
  {
    ereigniskalenderDatensatzRaeumlicheGueltigkeit = null;
    shell.dispose();
  }

  /**
   * This method initializes compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(shell, SWT.NONE);
    compositeTop.setLayoutData(gridData);
    compositeTop.setLayout(gridLayout);
    label = new Label(compositeTop, SWT.NONE);
    label.setText("Netzbestandteil:");
    createCombo();
  }

  /**
   * This method initializes compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.CENTER;
    gridData1.grabExcessHorizontalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData1);
    compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");
    buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}			
		});
    
    
    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}			
		});
  }

  /**
   * This method initializes combo
   * 
   */
  private void createCombo()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    combo = new Combo(compositeTop, SWT.READ_ONLY);
    combo.setLayoutData(gridData2);
    combo.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {
        IStructuredSelection structuredSelection = (IStructuredSelection)comboViewer.getSelection();

        Object obj = structuredSelection.getFirstElement();

        if (obj instanceof SystemObject)
        {
          SystemObject so = (SystemObject)obj;
          ConfigurationObject co = (ConfigurationObject)so;

          ereigniskalenderDatensatzRaeumlicheGueltigkeit = new EintragRaeumlicheGueltigkeit(co.getName(), co.getInfo()
              .getShortInfo(), so);

        }

      }
    });

    comboViewer = new ComboViewer(combo);
    comboViewer.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        return ((SystemObject)element).getName();
      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
      }

      @Override
      public void dispose()
      {
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {
      }

    });

  }

  /**
   * Holt die Netzbestandteile
   */
  private void holeNetzBestandteile()
  {

    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    _liste = connection.getDataModel().getType("typ.netzBestandTeil").getElements();
    for (SystemObject so : _liste)
    {
      comboViewer.add(so);
    }
  }

}
