package de.bsvrz.buv.plugin.ereigniskal;

/**
 * Die Klasse {@link EintragVerkehrlicheGueltigkeit} reprsentiert die Datenstze zur verkehrlichen Gltigkeit, die im
 * Dialog {@link Definitionsdialog} in der Liste der verkehrlichen Gltigkeiten angezeigt werden. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/04 05:57:26 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class EintragVerkehrlicheGueltigkeit implements Cloneable
{
  /**
   * Enumerator zur Definition des Bezugszeitpunktes "von". Dient der relativen Angabe der zeitlichen Gltigkeit.
   * 
   * @author HofmannJ
   */
  public enum _Bezugszeitpunkt {
    /**
     * Die Relation gibt einen Zeitpunkt vor dem Anfang des Zeitintervalls an.
     */
    vorAnfang,

    /**
     * Die Relation gibt einen Zeitpunkt nach dem Anfang des Zeitintervalls an.
     */
    nachAnfang,

    /**
     * Die Relation gibt einen Zeitpunkt vor dem Ende des Zeitintevalls an.
     */
    vorEnde,

    /**
     * Die Relation gibt einen Zeitpunkt nach dem Ende des Zeitintervalls an.
     */
    nachEnde
  };

  // /**
  // * Enumerator zur Definition des Bezugszeitpunktes "von". Dient der relativen
  // * Angabe der zeitlichen Gltigkeit.
  // *
  // * @author HofmannJ
  // *
  // */
  // public enum BezugszeitpunktVon
  // {
  // /**
  // * Die Relation gibt einen Zeitpunkt vor dem Anfang des Zeitintervalls an.
  // */
  // vorAnfang,
  //		
  // /**
  // * Die Relation gibt einen Zeitpunkt nach dem Anfang des Zeitintervalls an.
  // */
  // nachAnfang
  // };

  // /**
  // * Enumerator zur Definition der Bezugszeitpunktes "nach". Dient der relativen
  // * Angabe der zeitlichen Gltigkeit.
  // *
  // * @author HofmannJ
  // *
  // */
  // public enum BezugszeitpunktBis
  // {
  // /**
  // * Die Relation gibt einen Zeitpunkt vor dem Ende des Zeitintevalls an.
  // */
  // vorEnde,
  //		
  // /**
  // * Die Relation gibt einen Zeitpunkt nach dem Ende des Zeitintervalls an.
  // */
  // nachEnde
  // };

  /**
   * Startzeit der verkehrlichenGltigkeit
   */
  private long _von = 0;

  /**
   * Von-Bezug
   */
  private _Bezugszeitpunkt _enumVonBezugszeitpunkt = null;

  /**
   * Endzeit der verkehrlichenGltigkeit
   */
  private long _bis = 0;

  /**
   * Bis-Bezug
   */
  private _Bezugszeitpunkt _enumBisBezugszeitpunkt = null;

  /**
   * Konstruktor fr einen Datensatz zur verkehrlichen Gltigkeit.
   * 
   * @param von
   *          Startzeitpunkt in Millisekunden seit dem 1.1.1970.
   * @param vonBezug
   *          relative Angabe zur zeitlichen Gltigkeit.
   * @param bis
   *          Endzeitpunkt in Millisekunden sei dem 1.1.1970.
   * @param bisBezug
   *          relative Angabe zur zeitlichen Gltigkeit.
   */
  public EintragVerkehrlicheGueltigkeit(long von, _Bezugszeitpunkt vonBezug, long bis, _Bezugszeitpunkt bisBezug)
  {
    this._von = von;
    this._enumVonBezugszeitpunkt = vonBezug;
    this._bis = bis;
    this._enumBisBezugszeitpunkt = bisBezug;
  }

  /**
   * Liefert den Bezugszeitpunkt von.
   * 
   * @return Bezugszeitpunkt von.
   */
  public long getVon()
  {
    return _von;
  }

  /**
   * Setzt den Bezugszeitpunkt von.
   * 
   * @param _von
   *          Zu setzender Bezugszeitpunkt von.
   */
  public void setVon(long _von)
  {
    this._von = _von;
  }

  /**
   * Liefert die relative Angabe zur zeitlichen Gltigkeit von.
   * 
   * @return Relative Angabe zur zeitlichen Gltigkeit von.
   */
  public _Bezugszeitpunkt getVonBezugszeitpunkt()
  {
    return _enumVonBezugszeitpunkt;
  }

  /**
   * Setzt die relative Angabe zur zeitlichen Gltigkeit von.
   * 
   * @param vonBezugszeitpunkt
   *          Zu setzende relative Angabe zur zeitlichen Gltigkeit von.
   */
  public void setVonBezugszeitpunkt(_Bezugszeitpunkt vonBezugszeitpunkt)
  {
    _enumVonBezugszeitpunkt = vonBezugszeitpunkt;
  }

  /**
   * Liefert den Bezugszeitpunkt bis.
   * 
   * @return Bezugszeitpunkt bis.
   */
  public long getBis()
  {
    return _bis;
  }

  /**
   * Setzt den Bezugszeitpunkt bis.
   * 
   * @param _bis
   *          Zu setzender Bezugszeitpunkt bis.
   */
  public void setBis(long _bis)
  {
    this._bis = _bis;
  }

  /**
   * Liefert die relative Angabe zur zeitlichen Gltigkeit bis.
   * 
   * @return Relative Angabe zur zeitlichen Gltigkeit bis.
   */
  public _Bezugszeitpunkt getBisBezugszeitpunkt()
  {
    return _enumBisBezugszeitpunkt;
  }

  /**
   * Setzt die relative Angabe zur zeitlichen Gltigkeit bis.
   * 
   * @param bisBezugszeitpunkt
   *          Zu setzende relative Angabe zur zeitlichen Gltigkeit bis.
   */
  public void setBisBezugszeitpunkt(_Bezugszeitpunkt bisBezugszeitpunkt)
  {
    _enumBisBezugszeitpunkt = bisBezugszeitpunkt;
  }

  /**
   * Liefert einen Clone der Instanz dieser Klasse.
   * 
   * @return Clone der Instanz dieser Klasse.
   */
  public EintragVerkehrlicheGueltigkeit clone()
  {
    EintragVerkehrlicheGueltigkeit verkehrlicheGueltigkeit = new EintragVerkehrlicheGueltigkeit(_von,
        _enumVonBezugszeitpunkt, _bis, _enumBisBezugszeitpunkt);
    return verkehrlicheGueltigkeit;
  }

  @Override
  public boolean equals(Object obj)
  {
    EintragVerkehrlicheGueltigkeit eintrag = (EintragVerkehrlicheGueltigkeit)obj;

    boolean flagge = false;

    if (_von == eintrag.getVon() && _enumVonBezugszeitpunkt == eintrag.getVonBezugszeitpunkt()
        && _bis == eintrag.getBis() && _enumBisBezugszeitpunkt == eintrag.getBisBezugszeitpunkt())
      flagge = true;

    return flagge;

  }

}
