/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.dav;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.login.LoginDialog;
import de.bsvrz.buv.rw.rw.dav.ArbeitsbereichAktivStatus;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungsAbbauWarten;
import de.bsvrz.buv.rw.rw.dav.UrlasserAngemeldet;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Testet die DaV-Verbindung, zeigt wie Benutzer/Urlasserverbindungen aufgebaut
 * werden knnen und wie ein {@link IDaVVerbindungZuhoerer} zu behandeln ist,
 * mit dessen Hilfe man sich ber die aktuelle DaV-Verbindung informieren lassen
 * kann.
 * 
 * @author HofmannJ
 * 
 */
public class ViewRwDavTest extends ViewPart implements
		IDaVVerbindungsAbbauWarten
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Liefert den DaV-Zugriff
	 */
	private DaVVerbindung _dav = null;

	/**
	 * DaV-Zuhrer
	 */
	private IDaVVerbindungZuhoerer _zuh;

	/**
	 * Dispose-Flag
	 */
	private boolean _disposed = false;

	public static final String ID = "de.bsvrz.buv.plugin.beispiele.dav.ViewRwDavTest";

	private Composite top = null;

	private Group groupDavStatus = null;

	private Label labelBenutzer = null;

	private Label labelIpDav = null;

	private Label labelPort = null;

	private Text textBenutzer = null;

	private Text textIp = null;

	private Text textPort = null;

	private Group groupDavKontrolle = null;

	private Button buttonDav1 = null;

	private Button buttonDav2 = null;

	private Button buttonDavTrennen = null;

	private Group groupDavEvents = null;

	private Text textEreignisse = null;

	private Button buttonDavStatusAktualisieren = null;

	private Button buttonEreignisseLoeschen = null;

	private Button _buttonSetzeAktiv;

	/**
	 * Flag das {@link IDaVVerbindungsAbbauWarten} anzeigt, ob abgebaut werden
	 * darf.
	 */
	private boolean _davAbbauOk = false;

	private Button _abbauTest;

	private Button buttonVerbindungsListenerRegistrieren = null;

	private IDaVVerbindungZuhoerer _verbingungsZuhoerer;

	/**
	 * Speichert den Text fr den Verbindungsabbruch.
	 */
	private String _abbruchTextSpeicher = "";

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGroupDavStatus();
		createGroupDavKontrolle();
		createGroupDavEvents();

		init();
	}

	/**
	 * Initialisierung durchfhren.
	 * 
	 */
	public void init()
	{
		/**
		 * Instanz von DaVVerbindung holen.
		 */
		_dav = DaVVerbindung.getInstanz();

		_zuh = new IDaVVerbindungZuhoerer()
		{
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				if (!_disposed)
				{
					_abbruchTextSpeicher = e.getGrundDaVVerbindunsAbbau();

					PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable()
					{
						@Override
						public void run()
						{
							textEreignisse.setText(_abbruchTextSpeicher);
							aktualisiereStatus();
						}

					});
					// textEreignisse.setText(e.getGrundDaVVerbindunsAbbau());
					// aktualisiereStatus();
				}
			}

			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				if (!_disposed)
				{
					textEreignisse.setText("Verbindung aufgebaut!");
					aktualisiereStatus();
				}
			}
		};
		_dav.hinzufuegenZuhoerer(_zuh);
		aktualisiereStatus();

		/**
		 * Registriert sich bei DaV als Element das erst OK sein muss, bevor die
		 * Verbindung getrennt wird.
		 */
		// _dav.hinzufuegenVerbindungsabbauWarten(this);
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes groupDavStatus
	 * 
	 */
	private void createGroupDavStatus()
	{
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.verticalAlignment = GridData.CENTER;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		groupDavStatus = new Group(top, SWT.NONE);
		groupDavStatus.setLayoutData(gridData);
		groupDavStatus.setLayout(gridLayout);
		groupDavStatus.setText("DaV Status");
		labelBenutzer = new Label(groupDavStatus, SWT.NONE);
		labelBenutzer.setText("Benutzer:");
		textBenutzer = new Text(groupDavStatus, SWT.BORDER);
		textBenutzer.setEditable(false);
		textBenutzer.setLayoutData(gridData1);
		labelIpDav = new Label(groupDavStatus, SWT.NONE);
		labelIpDav.setText("IP:");
		textIp = new Text(groupDavStatus, SWT.BORDER);
		textIp.setEditable(false);
		textIp.setLayoutData(gridData2);
		labelPort = new Label(groupDavStatus, SWT.NONE);
		labelPort.setText("Port:");
		textPort = new Text(groupDavStatus, SWT.BORDER);
		textPort.setEditable(false);
		textPort.setLayoutData(gridData3);
	}

	/**
	 * This method initializes groupDavKontrolle
	 * 
	 */
	private void createGroupDavKontrolle()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.wrap = true;
		rowLayout.fill = false;
		rowLayout.justify = false;
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		groupDavKontrolle = new Group(top, SWT.NONE);
		groupDavKontrolle.setLayoutData(gridData4);
		groupDavKontrolle.setLayout(rowLayout);
		groupDavKontrolle.setText("DaV Kontrolle");
		buttonDav1 = new Button(groupDavKontrolle, SWT.NONE);
		buttonDav1.setText("DaV 1 (Lokal)");

		buttonDav1.setEnabled(false);
		/*
		 * DaV1-Listener
		 */	
		buttonDav1.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("Starte Aufbau der Verbindung 1");

				_dav.setDaVBenutzer("Tester");
				_dav.setDaVPasswort("geheim");
				_dav.setDaVIp("127.0.0.1");
				_dav.setDaVPort(8083);

				if (_dav.baueVerbindungMitDaVAuf())
				{
					_debug.fine("Verbindung OK");
					textEreignisse.setText("Verbindung mit DaV aufgebaut!");
				} else
				{
					_debug.fine("Verbindung Fehler!");
				}

				_debug.fine("Verbindungsaufbau beendet.");
			}			
		});

		buttonDav2 = new Button(groupDavKontrolle, SWT.NONE);
		buttonDav2.setText("DaV 2 (192.0.1.97)");

		buttonDav2.setEnabled(false);
		/*
		 * DaV2-Listener
		 */		
		buttonDav2.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{

				_debug.fine("Starte Aufbau der Verbindung 2");

				_dav.setDaVBenutzer("Tester");
				_dav.setDaVPasswort("geheim");
				_dav.setDaVIp("192.0.1.97");
				_dav.setDaVPort(8085);

				if (_dav.baueVerbindungMitDaVAuf())
				{
					_debug.fine("Verbindung OK");

					textEreignisse.setText("Verbindung mit DaV aufgebaut!");
				} else
				{
					_debug.fine("Verbindung Fehler!");
				}

				_debug.fine("Verbindungsaufbau beendet.");
			}			
		});

		buttonDavTrennen = new Button(groupDavKontrolle, SWT.NONE);
		buttonDavTrennen.setText("DaV Trennen");

		Button buttonDaVUrlasserVerbinden = new Button(groupDavKontrolle, SWT.PUSH);
		buttonDaVUrlasserVerbinden.setText("Urlasser Verbinden");	
		buttonDaVUrlasserVerbinden.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				UrlasserAngemeldet urlasserAngemeldet = UrlasserAngemeldet.getInstanz();
				if ((urlasserAngemeldet.getName() != null)
						&& (urlasserAngemeldet.getPasswort() != null))
				{
					_debug.fine("===> Anmelden als spezieller Urlasser");
					_dav.setUrlasserBenutzer(urlasserAngemeldet.getName());
					_dav.setUrlasserPasswort(urlasserAngemeldet.getPasswort());
				} else
				{
					_debug.fine("===> Anmelden als Std. Urlasser");
					_dav.setUrlasserBenutzer("Tester");
					_dav.setUrlasserPasswort("geheim");
				}

				if (_dav.baueUrlasserVerbindungMitDaVAuf())
				{
					textEreignisse.setText("Urlasserverbindung aufgebaut!");
				}
			}			
		});

		Button buttonDaVUrlasserTrennen = new Button(groupDavKontrolle, SWT.PUSH);
		buttonDaVUrlasserTrennen.setText("Urlasser Trennen");
		buttonDaVUrlasserTrennen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_dav.trenneUrlasserDaVVerbindung();
			}			
		});
		

		buttonDavStatusAktualisieren = new Button(groupDavKontrolle, SWT.NONE);
		buttonDavStatusAktualisieren.setText("DaV Status aktualisieren");
		buttonEreignisseLoeschen = new Button(groupDavKontrolle, SWT.NONE);
		buttonEreignisseLoeschen.setText("Ereignisse Lschen");

		/*
		 * Ereignisse Lschen
		 */
		buttonEreignisseLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				textEreignisse.setText("");
			}			
		});

		/*
		 * Status aktualisieren
		 */
		buttonDavStatusAktualisieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktualisiereStatus();
			}			
		});

		/*
		 * DaV Trennen
		 */
		buttonDavTrennen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_dav.trenneDaVVerbindung();
			}			
		});

		// Benutzerdialog
		Button buttonLoginDialog = new Button(groupDavKontrolle, SWT.PUSH);
		buttonLoginDialog.setText("Login Dialog");
		buttonLoginDialog.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{LoginDialog login = new LoginDialog(Display.getCurrent()
					.getActiveShell());
			boolean ok = login.anzeigen();
			if (ok)
			{
				_debug.fine(getClass().getName() + " => LoginDialog OK!");
				_dav.holeBenutzerDaVVerbindung();
				_dav.baueVerbindungMitDaVAuf();
			}
			}			
		});

		// Urlasserdialog
		Button buttonUrlasserDialog = new Button(groupDavKontrolle, SWT.PUSH);
		buttonUrlasserDialog.setText("Urlasser Dialog");
		buttonUrlasserDialog.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Nicht mehr Verfgbar!");
				mb
						.setMessage("Nicht mehr Verfgbar. Die Speichernfunktion des Meneditiors bringt auch einen Urlasserdialog hoch.");
				mb.open();
			}			
		});

		// Button DaVVerbindung LoginDialog
		Button buttonDaVMitLoginDialogDaten = new Button(groupDavKontrolle,
				SWT.PUSH);
		buttonDaVMitLoginDialogDaten
				.setText("DaV-Verbindung mit LoginDialog-Daten");
		buttonDaVMitLoginDialogDaten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_dav.holeBenutzerDaVVerbindung();
				_dav.baueVerbindungMitDaVAuf();
			}			
		});

		// Button DaVVerbindung UrlasserDialog
		Button buttonDaVMitUrlasserDialogDaten = new Button(groupDavKontrolle,
				SWT.PUSH);
		buttonDaVMitUrlasserDialogDaten
				.setText("DaV-Verbindung mit UrlasserDialog-Daten");
		buttonDaVMitUrlasserDialogDaten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_dav.holeUrlasserDaVVerbindung();
				_dav.baueUrlasserVerbindungMitDaVAuf();
			}			
		});

		_buttonSetzeAktiv = new Button(groupDavKontrolle, SWT.PUSH);
		if (ArbeitsbereichAktivStatus.getInstanz().getArbeitsbereichAktiv())
		{
			_buttonSetzeAktiv.setText("Aktiv");
			_buttonSetzeAktiv.setEnabled(false);
		} else
		{
			_buttonSetzeAktiv.setText("Aktivieren");
		}

		_buttonSetzeAktiv.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_dav.setzeArbeitsbereich(true);
				_buttonSetzeAktiv.setText("Aktiv");
			}			
		});

		_abbauTest = new Button(groupDavKontrolle, SWT.PUSH);
		if (_davAbbauOk)
		{
			_abbauTest.setText("Kann Abgebaut werden");
			_abbauTest.setEnabled(false);

		} else
		{
			_abbauTest.setText("Kann NICHT Abgebaut werden");
		}

		_abbauTest.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (!_davAbbauOk)
				{
					_davAbbauOk = true;
				} else
				{
					_davAbbauOk = false;
				}

				if (_davAbbauOk)
				{
					_abbauTest.setText("Kann Abgebaut werden");
				} else
				{
					_abbauTest.setText("Kann NICHT Abgebaut werden");
				}
			}			
		});
		_abbauTest.setEnabled(false);

		buttonVerbindungsListenerRegistrieren = new Button(groupDavKontrolle,
				SWT.NONE);
		buttonVerbindungsListenerRegistrieren
				.setText("Verbindungslistener Registrieren");
		buttonVerbindungsListenerRegistrieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionRegistriereVerbindungsListener();
			}			
		});

	}

	/**
	 * Diese Methode registriert einen {@link IDaVVerbindungZuhoerer}, mit dessen
	 * Hilfe man sich ber die aktuelle DaV-Verbindung informieren lassen kann.
	 */
	private void aktionRegistriereVerbindungsListener()
	{
		if (_verbingungsZuhoerer == null)
		{
			_verbingungsZuhoerer = new IDaVVerbindungZuhoerer()
			{

				@Override
				public void verbindungAbgebaut(DaVVerbindungEreignis e)
				{
					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
					mb.setText("DaV-Verbindung abgebaut...");
					mb.setMessage("Die DaV-Verbindung wurde getrennt.");
					mb.open();
				}

				@Override
				public void verbindungAufgebaut(DaVVerbindungEreignis e)
				{
					String meldung = "Daten der DaV-Verbindung:\n";
					meldung = meldung + "DaV-Adresse: " + e.getDaVIpAdresse() + ":"
							+ e.getDaVPort() + "\n";
					meldung = meldung + "Benutzer: " + e.getDaVBenutzer() + "\n";

					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
					mb.setText("DaV-Verbindung aufgebaut...");
					mb.setMessage(meldung);
					mb.open();
				}

			};

		}

		DaVVerbindung.getInstanz().entferneZuhoerer(_verbingungsZuhoerer);
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(_verbingungsZuhoerer);

	}

	/**
	 * This method initializes groupDavEvents
	 * 
	 */
	private void createGroupDavEvents()
	{
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = GridData.FILL;
		groupDavEvents = new Group(top, SWT.NONE);
		groupDavEvents.setLayout(new FillLayout());
		groupDavEvents.setLayoutData(gridData5);
		groupDavEvents.setText("Ereignisse");
		textEreignisse = new Text(groupDavEvents, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
	}

	private void aktualisiereStatus()
	{
		if (_dav != null)
		{
			if (_dav.getDaVIp() != null)
			{
				textIp.setText(_dav.getDaVIp());
			} else
			{
				textIp.setText("");
			}
			if (_dav.getDaVPort() != -1)
			{
				String tempText = "" + _dav.getDaVPort();
				textPort.setText(tempText);
			} else
			{
				textPort.setText("");
			}
			if (_dav.getDaVBenutzer() != null)
			{
				textBenutzer.setText(_dav.getDaVBenutzer());
			} else
			{
				textBenutzer.setText("");
			}
		}
	}

	public boolean DaVVerbindungsAbbauOk()
	{
		return _davAbbauOk;
	}

	public void dispose()
	{
		_disposed = true;
		DaVVerbindung.getInstanz().entferneZuhoerer(_verbingungsZuhoerer);
		super.dispose();
	}

} // @jve:decl-index=0:visual-constraint="10,10,312,289"
