/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.mehrfachzwischenablage;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.IMehrfachZwischenablageAusschneiden;
import de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.IMehrfachZwischenablageEinfuegen;
import de.bsvrz.buv.rw.basislib.mehrfachzwischenablage.MehrfachZwischenablageDaten;

public class ZwischenablageTestView extends ViewPart implements
		IMehrfachZwischenablageEinfuegen, IMehrfachZwischenablageAusschneiden
{

	public static final String ID = "de.bsvrz.buv.plugin.pruefrw.mehrfachzwischenablage.ZwischenablageTestView";
	private Composite top = null;
	private Group gruppeTestBereich = null;
	private Composite compositeKontrolle = null;
	private Composite compositeKontrolleRechts = null;
	private Button schaltflaecheOk = null;
	private Table tabelleEinfgen = null;
	private TableViewer tableViewer = null;
	private Button schaltflaecheAbbrechen = null;

	/**
	 * Liste der darzustellenden Daten im Einfgebereich
	 */
	private LinkedList<Object> _datenEinfgen = new LinkedList<Object>(); // @jve:decl-index=0:

	/**
	 * Liste der kopier- und ausschneidbaren Daten
	 */
	private LinkedList<Object> _datenKopierenAusschneiden = new LinkedList<Object>();

	/**
	 * ContenProvider der Tabelle
	 */
	private ContentProvider _contentProvider = null;
	private Group gruppeKopieren = null;
	private Table tabelleAusschneidenEinfgen = null;
	private TableViewer tableViewerKopierenAusschneiden = null;

	/**
	 * ContentProvider fr das Kopieren und Ausschneiden
	 */
	private ContentProvider _contentProviderKopierenAusschneiden = null;

	@Override
	public void createPartControl(Composite parent)
	{
		top = new Composite(parent, SWT.NONE);
		top.setLayout(new GridLayout());
		createGruppeTestBereich();
		createCompositeKontrolle();
		init();
	}

	/**
	 * Durchfhren von Initialisierungen
	 */
	private void init()
	{
		_datenKopierenAusschneiden.add(new Date());
		_datenKopierenAusschneiden.add("Hallo Welt!");
		_datenKopierenAusschneiden.add(new Integer(1234));
		aktionAktualisiereInhaltTabelleKopierenAusschneiden();
		
		MehrfachZwischenablageDaten.getInstanz().hinzufuegenZuhoererAusschneiden(this);
	}

	/**
	 * Aktualisiert die Darstellung der Daten der Tabelle.
	 */
	protected void aktionAktualisiereInhaltTabelleEinfuegen()
	{
		tableViewer.setInput(_datenEinfgen);
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes gruppeTestBereich
	 * 
	 */
	private void createGruppeTestBereich()
	{
		GridData gridDataTabelle = new GridData();
		gridDataTabelle.horizontalAlignment = GridData.FILL;
		gridDataTabelle.grabExcessHorizontalSpace = true;
		gridDataTabelle.grabExcessVerticalSpace = true;
		gridDataTabelle.verticalAlignment = GridData.FILL;
		GridData gridDataTestBereich = new GridData();
		gridDataTestBereich.grabExcessHorizontalSpace = true;
		gridDataTestBereich.verticalAlignment = GridData.FILL;
		gridDataTestBereich.grabExcessVerticalSpace = true;
		gridDataTestBereich.horizontalAlignment = GridData.FILL;
		gruppeTestBereich = new Group(top, SWT.NONE);
		gruppeTestBereich.setLayout(new GridLayout());
		gruppeTestBereich.setLayoutData(gridDataTestBereich);
		gruppeTestBereich.setText("Testbereich zum Einfgen:");
		tabelleEinfgen = new Table(gruppeTestBereich, SWT.NONE);
		tabelleEinfgen.setHeaderVisible(true);
		tabelleEinfgen.setLayoutData(gridDataTabelle);
		tabelleEinfgen.setLinesVisible(true);
		createGruppeKopieren();
		tableViewer = new TableViewer(tabelleEinfgen);

		/*
		 * Tabellen Spalten
		 */
		TableColumn tcClassName = new TableColumn(tableViewer.getTable(), SWT.LEFT);
		tcClassName.setText("Instanz von:");
		tcClassName.setWidth(200);

		TableColumn tcToString = new TableColumn(tableViewer.getTable(), SWT.LEFT);
		tcToString.setText("ToString:");
		tcToString.setWidth(300);

		/*
		 * Tabellenkopf + Linien darstellen
		 */
		tableViewer.getTable().setHeaderVisible(true);
		tableViewer.getTable().setLinesVisible(true);

		/*
		 * Content- + LabelProvider initialisieren.
		 */
		_contentProvider = new ContentProvider();
		tableViewer.setContentProvider(_contentProvider);
		LabelProvider labelProvider = new LabelProvider();
		tableViewer.setLabelProvider(labelProvider);

	}

	/**
	 * This method initializes compositeKontrolle
	 * 
	 */
	private void createCompositeKontrolle()
	{
		GridData gridDataKontrolle = new GridData();
		gridDataKontrolle.grabExcessHorizontalSpace = true;
		gridDataKontrolle.verticalAlignment = GridData.CENTER;
		gridDataKontrolle.horizontalAlignment = GridData.FILL;
		compositeKontrolle = new Composite(top, SWT.NONE);
		compositeKontrolle.setLayout(new GridLayout());
		createCompositeKontrolleRechts();
		compositeKontrolle.setLayoutData(gridDataKontrolle);
	}

	/**
	 * This method initializes compositeKontrolleRechts
	 * 
	 */
	private void createCompositeKontrolleRechts()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridDataKontrolleRechts = new GridData();
		gridDataKontrolleRechts.horizontalAlignment = GridData.END;
		gridDataKontrolleRechts.grabExcessHorizontalSpace = true;
		gridDataKontrolleRechts.verticalAlignment = GridData.CENTER;
		compositeKontrolleRechts = new Composite(compositeKontrolle, SWT.NONE);
		compositeKontrolleRechts.setLayoutData(gridDataKontrolleRechts);
		compositeKontrolleRechts.setLayout(rowLayout);
		schaltflaecheOk = new Button(compositeKontrolleRechts, SWT.NONE);
		schaltflaecheOk.setText("OK");
		schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				//Der Button hat nichts zu tun.
			}			
		});
		
		
		schaltflaecheAbbrechen = new Button(compositeKontrolleRechts, SWT.NONE);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				//Dieser Button hat auch nix zu tun.
			}			
		});
	}

	/**
	 * ContentProvider der Tabelle.<br>
	 * Datenquelle: Liste vom Typ {@link Object} Aufbereitung: 1. Spalte:
	 * Klassenname 2. Spalte: Object to String
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ContentProvider implements IStructuredContentProvider
	{
		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof Collection)
			{
				Collection<Object> liste = (Collection<Object>) inputElement;
				re = new Object[liste.size()];

				int i = 0;
				Iterator<Object> it = liste.iterator();
				while (it.hasNext())
				{
					Object obj = it.next();
					re[i] = obj;
					i++;
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * LabelProvider fr die Tabelle. Spalte 1: {@link Object} Klassenname Spalte
	 * 2: {@link Object} toString
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			switch (columnIndex)
			{
			case 0:
				re = element.getClass().getName();
				break;
			case 1:
				re = element.toString();
				break;
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void einfuegen(List<Object> objekt)
	{
		if ((objekt != null) && (objekt.size() > 0))
		{
			Iterator<Object> it = objekt.iterator();
			while (it.hasNext())
			{
				Object ob = it.next();
				if (!_datenEinfgen.contains(ob))
				{
					_datenEinfgen.add(ob);
				}
			}

			aktionAktualisiereInhaltTabelleEinfuegen();
		}

	}

	protected void aktionAktualisiereInhaltTabelleKopierenAusschneiden()
	{
		tableViewerKopierenAusschneiden.setInput(_datenKopierenAusschneiden);
	}

	/**
	 * This method initializes gruppeKopieren
	 * 
	 */
	private void createGruppeKopieren()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gruppeKopieren = new Group(gruppeTestBereich, SWT.NONE);
		gruppeKopieren.setLayout(new GridLayout());
		gruppeKopieren.setLayoutData(gridData);
		gruppeKopieren.setText("Testbereich zum Kopieren und Ausschneiden:");
		tabelleAusschneidenEinfgen = new Table(gruppeKopieren, SWT.MULTI);
		tabelleAusschneidenEinfgen.setHeaderVisible(true);
		tabelleAusschneidenEinfgen.setLayoutData(gridData1);
		tabelleAusschneidenEinfgen.setLinesVisible(true);
		tableViewerKopierenAusschneiden = new TableViewer(
				tabelleAusschneidenEinfgen);

		/*
		 * Tabellen Spalten
		 */
		TableColumn tcClassName = new TableColumn(tableViewerKopierenAusschneiden
				.getTable(), SWT.LEFT);
		tcClassName.setText("Instanz von:");
		tcClassName.setWidth(200);

		TableColumn tcToString = new TableColumn(tableViewerKopierenAusschneiden
				.getTable(), SWT.LEFT);
		tcToString.setText("ToString:");
		tcToString.setWidth(300);

		/*
		 * Tabellenkopf + Linien darstellen
		 */
		tableViewerKopierenAusschneiden.getTable().setHeaderVisible(true);
		tableViewerKopierenAusschneiden.getTable().setLinesVisible(true);

		_contentProviderKopierenAusschneiden = new ContentProvider();
		tableViewerKopierenAusschneiden.setContentProvider(_contentProviderKopierenAusschneiden);
		LabelProvider labelProviderKopierenAusschneiden = new LabelProvider();
		tableViewerKopierenAusschneiden
				.setLabelProvider(labelProviderKopierenAusschneiden);
		
		
		getSite().setSelectionProvider(tableViewerKopierenAusschneiden);
	}

	@Override
	public void entferneSelektion(Object auszuschneidendeObjekt)
	{
		if(_datenKopierenAusschneiden.contains(auszuschneidendeObjekt))
		{
			_datenKopierenAusschneiden.remove(auszuschneidendeObjekt);
			aktionAktualisiereInhaltTabelleKopierenAusschneiden();
		}
		
	}

	@Override
	public IWorkbenchPart getTeil()
	{
		return this;
	}

}
