/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.start;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert das Interface {@link IStartup}. ber den
 * Erweiterungspunkt <code>org.eclipse.ui.startup</code> wird es registriert.
 * Sobald das Plug-In von Equinox eingebunden wird, wird der Code in dieser
 * Klasse ausgefhrt. <br>
 * <br>
 * 
 * Dies ist der <b><i>ideale</i></b> Punkt, um neue Aktionen zu registrieren.
 * Dies wird hier <b>beispielhaft</b> gezeigt.
 * 
 * @author HofmannJ
 * 
 */
public class Start implements IStartup
{
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger(); 
  
	/**
	 * ID dieses PlugIns
	 */
	private static final String _plugInID = "de.bsvrz.buv.plugin.pruefrw";

	/**
	 * Diese Aufrufe innerhalb dieser Methode werden beim Einbinden des Plug-Ins
	 * automatisch ausgefhrt.
	 */
	@Override
	public void earlyStartup()
	{
		_debug.fine("===> Vor der Registrierung von " + getClass().getName());
		erzeugeActionUndRegistriere();
		_debug.fine("===> Nach der Registrierung von "
				+ getClass().getName());
	}

	/**
	 * Diese Methode erzeugt eine {@link Action}, verpackt diese in einer Instanz
	 * der Klasse {@link DeskriptorAktion}. <br>
	 * <br>
	 * Diese {@link DeskriptorAktion} wird bei der Klasse
	 * {@link ElementRegistrierung} ber deren Methode
	 * {@link ElementRegistrierung#registriereDeskriptorAktion(DeskriptorAktion)}
	 * registriert und steht damit fr die Einbindung in Mens und Symbolleisten
	 * zur Verfgung. <br>
	 * <br>
	 * <u><b>Achtung:</b></u> Den erzeugten Actions muss mittels deren Methode
	 * {@link Action#setId(String)} <b><i><u>zwingend</u></i></b> eine ID
	 * zugewiesen werden.
	 */
	private void erzeugeActionUndRegistriere()
	{
		/*
		 * Erzeugen einer Action.
		 */
		Action action = new Action("Hallo Welt!")
		{
			@Override
			public void run()
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION
						| SWT.APPLICATION_MODAL);
				mb.setText("Information:");
				mb.setMessage("Ein Plugin hat erfolgreich diese "
						+ "Action zum Rahmenwerk hinzugefgt!");
				mb.open();

			}
		};

		/*
		 * Vergeben einer ActionID nicht vergessen!
		 */
		action.setId("Eine ACTION-ID!");

		/*
		 * DeskriptorAktion erzeugen
		 */
		DeskriptorAktion da = new DeskriptorAktion(_plugInID, action, false);
		try
		{
			/*
			 * Und bei der Elementregistrierung registrieren.
			 */
			ElementRegistrierung elementRegistrierung = ElementRegistrierung
					.getInstanz();
			elementRegistrierung.registriereDeskriptorAktion(da);

			/*
			 * Wenn die Registrierung erfolgreich war, msste die ID dieses Plug-Ins
			 * in der ElementRegistrierung auftauchen.
			 */
			List<String> pluginIDs = elementRegistrierung.getPluginIDs();
			Iterator<String> it = pluginIDs.iterator();
			while (it.hasNext())
			{
				String id = it.next();
				_debug.fine("ID: " + id);
			}

		} catch (Error e)
		{
			_debug.fine("E: " + e.toString());
		}

	}

}
