/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.statusleiste;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pruefrw.Aktivator;
import de.bsvrz.buv.rw.basislib.statusleiste.generator.StatusLeisteGenerator;
import de.bsvrz.buv.rw.rw.statusleiste.StatusLeisteZugriff;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert ein Beispiel, wie sich die Statuszeile lschen &
 * befllen lsst.
 * 
 * @author HofmannJ
 * 
 */
public class ViewStatusleisteBeispiel extends ViewPart
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger(); 
  
	/**
	 * ID dieser View
	 */
	public static final String ID = "de.bsvrz.buv.plugin.beispiele.viewStatusLeisteBeispiel";

	/**
	 * ID der Perspektive des Statusleisteneditors
	 */
	public static final String _perspektiveStatusleisteEditorId = "de.bsvrz.buv.rw.basislib.perspectiveStatusLeisteEditor";


	/**
	 * Eltern Composite
	 */
	private Composite _elternComposite = null;

	/**
	 * Gruppe Kontrolle
	 */
	private Group _gruppeKontrolle;

	/**
	 * Gruppe Status
	 */
	private Group _gruppeStatus;

	/**
	 * Ein Textfeld
	 */
	private Text _textFeld = null;

	private static final String _bild1 = "icons/Ort_16x16.png";
	private static final String _bild2 = "icons/Einstellungen_16x16.png";

	public ViewStatusleisteBeispiel()
	{
	}

	@Override
	public void createPartControl(Composite parent)
	{
		this._elternComposite = parent;
		erzeugeDarstellung();
	}

	/**
	 * Erzeugt die Darstellung dieses Composites
	 */
	private void erzeugeDarstellung()
	{
		erzeugeGruppeKontrolle();
		erzeugeGruppeEingabe(_elternComposite);
		erzeugeGruppeStatus();

		_elternComposite.setLayout(new GridLayout());

		GridData gridDataGruppeKontrolle = new GridData();
		gridDataGruppeKontrolle.horizontalAlignment = GridData.FILL;
		gridDataGruppeKontrolle.grabExcessHorizontalSpace = true;
		gridDataGruppeKontrolle.verticalAlignment = GridData.FILL;
		gridDataGruppeKontrolle.grabExcessVerticalSpace = false;
		// _groupKontrolle.setLayout(new GridLayout());
		_gruppeKontrolle.setLayout(new RowLayout());

		_gruppeKontrolle.setLayoutData(gridDataGruppeKontrolle);

		GridData gridDataGruppeStatus = new GridData();
		gridDataGruppeStatus.horizontalAlignment = GridData.FILL;
		gridDataGruppeStatus.grabExcessHorizontalSpace = true;
		gridDataGruppeStatus.verticalAlignment = GridData.FILL;
		gridDataGruppeStatus.grabExcessVerticalSpace = true;

		_gruppeStatus.setLayout(new GridLayout());
		_gruppeStatus.setLayoutData(gridDataGruppeStatus);

	}

	private void aktionEditorOeffnen()
	{
		IPerspectiveDescriptor perspectiveDescriptor = PlatformUI.getWorkbench()
				.getPerspectiveRegistry().findPerspectiveWithId(
						_perspektiveStatusleisteEditorId);

		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.setPerspective(perspectiveDescriptor);
	}

	/**
	 * Diese Methode erzeugt einige {@link StatusLeisteElement}e und registrierte
	 * diese bei der Klasse {@link StatusLeisteVerwaltung}.
	 */
	private void aktionErzeugeStatusZeilenElemente()
	{
		String id = "vtsle.1";
		SlItem sli = new SlItem(id);
		StatusLeisteElement sle1 = new StatusLeisteElement(id, "Testelement#" + id,
				sli);
		StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sle1);
		// StatusLeisteZugriff.getInstanz().hinzufuegenStatusZeilenElement(sli);

		String id2 = "vtsle.2";
		SlItem sli2 = new SlItem(id2);
		StatusLeisteElement sle2 = new StatusLeisteElement(id2, "Testelement#"
				+ id2, sli2);
		StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sle2);
		// StatusLeisteZugriff.getInstanz().hinzufuegenStatusZeilenElement(sli2);

		String id3 = "sl.zeitelement";
		SlZeitItem szi = new SlZeitItem(id);
		StatusLeisteElement sle3 = new StatusLeisteElement(id3, "Zeitelement#"
				+ id3, szi);
		StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sle3);
	}

	private void erzeugeGruppeKontrolle()
	{
		_gruppeKontrolle = new Group(_elternComposite, SWT.NONE);
		_gruppeKontrolle.setText("Kontrolle");

		/*
		 * Button Test. Erzeugt verschiedene StatusLeistenElemente und registriert
		 * diese.
		 */
	
		/*
		 * Entfernt alle Elemente aus der Statusleiste.
		 */
		Button buttonStatuszeileLeeren = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonStatuszeileLeeren.setText("Statuszeile leeren");
		buttonStatuszeileLeeren
				.setToolTipText("Entfernt alle Elemente aus der Statusleiste.");
		buttonStatuszeileLeeren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				StatusLeisteZugriff.getInstanz().entferneAlleStatusZeilenElemente();
			}			
		});

		/*
		 * Erzeugen der Statuszeile.
		 */
		Button buttonAnlegen = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonAnlegen
				.setText("Statusleistegenerator Initialisieren, Zeile erzeugen");
		buttonAnlegen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine("VORHER");
				aktionErzeugeStatusZeilenElemente();
				aktionEditorOeffnen();
				StatusLeisteGenerator.getInstanz().generiereStatusLeiste();
				// aktionEditorOeffnen();
				_debug.fine("NACHHER");
			}			
		});

		/*
		 * Textmeldung setzen
		 */
		Button buttonMeldungText = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonMeldungText.setText("Setze Textmeldung");
		buttonMeldungText
				.setToolTipText("Setzt eine Textmeldung in der Statuszeile.");
		buttonMeldungText.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				StatusLeisteZugriff.getInstanz().setStatusZeilenMeldung(
				"Eine Textmeldung.");
			}			
		});

		/*
		 * Textmeldung mit Bild setzen
		 */
		Button buttonMeldungTextMitBild = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonMeldungTextMitBild.setText("Setze Textmeldung mit Bild");
		buttonMeldungTextMitBild
				.setToolTipText("Setzt eine Textmeldung mit Bild i.d. Statuszeile.");
		buttonMeldungTextMitBild.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				Image meldungsBild = Aktivator.getImageDescriptor(_bild1).createImage();
				StatusLeisteZugriff.getInstanz().setStatusZeilenMeldung(
						"Eine Textmeldung mit Bild.", meldungsBild);
			}			
		});

		/*
		 * Fehlermeldung setzen
		 */
		Button buttonFehlerMeldungText = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonFehlerMeldungText.setText("Setze Fehlermeldung");
		buttonFehlerMeldungText
				.setToolTipText("Setzt eine Fehlermeldung i.d. Statuszeile.");
		buttonFehlerMeldungText.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				StatusLeisteZugriff.getInstanz().setStatusZeilenFehlerMeldung(
				"Eine Fehlermeldung");
			}			
		});

		/*
		 * Fehlermeldung mit Bild setzen
		 */
		Button buttonFehlerMeldungTextMitBild = new Button(_gruppeKontrolle,
				SWT.PUSH);
		buttonFehlerMeldungTextMitBild.setText("Setze Fehlermeldung mit Bild");
		buttonFehlerMeldungTextMitBild
				.setToolTipText("Setzt eine Fehlermeldung mit Bild i.d. Statuszeile.");
		buttonFehlerMeldungTextMitBild.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{	Image fehlerMeldungsBild = Aktivator.getImageDescriptor(_bild2)
				.createImage();
			StatusLeisteZugriff.getInstanz().setStatusZeilenFehlerMeldung(
					"Eine Fehlermeldung mit Bild", fehlerMeldungsBild);
			}			
		});

		/*
		 * Meldungsbereich lschen
		 */
		Button buttonMeldungLoeschen = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonMeldungLoeschen.setText("Meldungsbereich lschen");
		buttonMeldungLoeschen
				.setToolTipText("Lscht den Meldungsbereich der Statuszeile.");
		buttonMeldungLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				StatusLeisteZugriff.getInstanz().loescheStatusZeilenMeldung();
			}			
		});

		/*
		 * Fehlermeldungsbereich
		 */
		Button buttonFehlerMeldungLoeschen = new Button(_gruppeKontrolle, SWT.PUSH);
		buttonFehlerMeldungLoeschen.setText("Fehlermeldungsbereich lschen");
		buttonFehlerMeldungLoeschen
				.setToolTipText("Lscht den Fehlermeldungsbereich der Statuszeile.");
		buttonFehlerMeldungLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				StatusLeisteZugriff.getInstanz().loescheStatusZeilenFehlerMeldung();
			}			
		});

	}

	/**
	 * {@link FileDialog} zum Whlen von Bilddateien.
	 * 
	 * @return Pfad zur Bilddatei oder <code>null</code>, wenn keine Bilddatei
	 *         gewhlt wurde.
	 */
	private String bildDateiWaehlen()
	{
		String pfad = null;

		Shell shell = PlatformUI.getWorkbench().getDisplay().getActiveShell();

		FileDialog dialog = new FileDialog(shell, SWT.OPEN);
		dialog.setFilterNames(new String[] { "PNG", "JPEG", "JPG" });
		dialog.setFilterExtensions(new String[] { "*.png", "*.jpeg", "*.jpg" });
		pfad = dialog.open();

		return pfad;
	}

	/**
	 * Entfernt alles aus dem Meldungsbereich der Statusleiste.
	 */
	private void meldungsbereichZuruecksetzen()
	{
		StatusLeisteZugriff slz = StatusLeisteZugriff.getInstanz();
		slz.loescheStatusZeilenFehlerMeldung();
		slz.loescheStatusZeilenMeldung();
	}

	/**
	 * Erzeugt eine {@link Group} zur Eingabe von Statusmeldungen.
	 */
	private void erzeugeGruppeEingabe(Composite elternComposite)
	{
		// Layout fr Eingabefelder
		GridData gridDataEingabe = new GridData();
		gridDataEingabe.grabExcessHorizontalSpace = true;
		gridDataEingabe.verticalAlignment = GridData.CENTER;
		gridDataEingabe.horizontalAlignment = GridData.FILL;

		// Layout fr Buttons
		GridData gridDataButton = new GridData();
		gridDataButton.grabExcessHorizontalSpace = true;
		// gridDataButton.verticalAlignment = GridData.LEFT;
		gridDataButton.horizontalAlignment = GridData.FILL;

		// Gruppe Eingabe erzeugen
		Group gruppeEingabe = new Group(elternComposite, SWT.NONE);
		gruppeEingabe.setText("Eingabe:");

		/*
		 * Textmeldung
		 */
		Label labelMeldung = new Label(gruppeEingabe, SWT.NONE);
		labelMeldung.setText("Meldungseingabe:");

		final Text textMeldung = new Text(gruppeEingabe, SWT.BORDER);
		textMeldung.setLayoutData(gridDataEingabe);

		Button buttonTextMeldung = new Button(gruppeEingabe, SWT.PUSH);
		buttonTextMeldung.setText("Textmeldung setzen");
		buttonTextMeldung.setLayoutData(gridDataButton);
		buttonTextMeldung.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				String text = textMeldung.getText();
				if ((text != null) && (text.length() > 0))
				{
					meldungsbereichZuruecksetzen();
					StatusLeisteZugriff.getInstanz().setStatusZeilenMeldung(text);
				}
			}			
		});

		/*
		 * Textmeldung mit Bild
		 */
		Label labelMeldungMitBild = new Label(gruppeEingabe, SWT.NONE);
		labelMeldungMitBild.setText("Bild zur Meldung whlen:");

		final Text textMeldungBildPfad = new Text(gruppeEingabe, SWT.BORDER);
		textMeldungBildPfad.setEditable(false);
		textMeldungBildPfad.setLayoutData(gridDataEingabe);

		Button buttonTextMeldungMitBild = new Button(gruppeEingabe, SWT.PUSH);
		buttonTextMeldungMitBild.setText("Bild setzen");
		buttonTextMeldungMitBild.setLayoutData(gridDataButton);	
		buttonTextMeldungMitBild.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				String pfad = bildDateiWaehlen();
				if (pfad != null)
				{
					textMeldungBildPfad.setText(pfad);

					Image img = new Image(PlatformUI.getWorkbench().getDisplay(), pfad);

					if ((img.getBounds().width > 16) || (img.getBounds().height > 16))
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
						mb.setText("Proportionen des Bildes sind zu gro!");
						mb
								.setMessage("Hinweis: Ein Bild fr die Statuszeile darf eine maximale Seitenlnge von 16 Pixeln aufweisen.");
						mb.open();
					} else
					{
						String text = textMeldung.getText();
						if ((text != null) && (text.length() > 0))
						{
							meldungsbereichZuruecksetzen();
							StatusLeisteZugriff.getInstanz()
									.setStatusZeilenMeldung(text, img);
						} else
						{
							meldungsbereichZuruecksetzen();
							StatusLeisteZugriff.getInstanz().setStatusZeilenMeldung("", img);
						}
					}
				}
			}			
		});

		/*
		 * Fehlermeldung
		 */
		Label labelFehlerMeldung = new Label(gruppeEingabe, SWT.NONE);
		labelFehlerMeldung.setText("Fehlermeldungseingabe:");

		final Text textFehlerMeldung = new Text(gruppeEingabe, SWT.BORDER);
		textFehlerMeldung.setLayoutData(gridDataEingabe);

		Button buttonTextFehlerMeldung = new Button(gruppeEingabe, SWT.PUSH);
		buttonTextFehlerMeldung.setText("Fehlermeldung setzen");
		buttonTextFehlerMeldung.setLayoutData(gridDataButton);
		buttonTextFehlerMeldung.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				String text = textFehlerMeldung.getText();
				if ((text != null) && (text.length() > 0))
				{
					meldungsbereichZuruecksetzen();
					StatusLeisteZugriff.getInstanz().setStatusZeilenFehlerMeldung(text);
				}
			}			
		});

		/*
		 * Fehlermeldung mit Bild
		 */
		Label labelFehlerMeldungMitBild = new Label(gruppeEingabe, SWT.NONE);
		labelFehlerMeldungMitBild.setText("Fehlermeldung mit Bild:");

		final Text textFehlerMeldungMitBild = new Text(gruppeEingabe, SWT.BORDER);
		textFehlerMeldungMitBild.setLayoutData(gridDataEingabe);
		textFehlerMeldungMitBild.setEditable(false);

		Button buttonTextFehlerMeldungMitbild = new Button(gruppeEingabe, SWT.PUSH);
		buttonTextFehlerMeldungMitbild.setText("Fehlermeldungsbild setzen");
		buttonTextFehlerMeldungMitbild.setLayoutData(gridDataButton);		
		buttonTextFehlerMeldungMitbild.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				String pfad = bildDateiWaehlen();
				if (pfad != null)
				{
					textFehlerMeldungMitBild.setText(pfad);

					Image img = new Image(PlatformUI.getWorkbench().getDisplay(), pfad);
					if ((img.getBounds().width > 16) || (img.getBounds().height > 16))
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
						mb.setText("Proportionen des Bildes sind zu gro!");
						mb
								.setMessage("Hinweis: Ein Bild fr die Statuszeile darf eine maximale Seitenlnge von 16 Pixeln aufweisen.");
						mb.open();
					} else
					{
						String text = textFehlerMeldung.getText();

						if ((text != null) && (text.length() > 0))
						{
							meldungsbereichZuruecksetzen();
							StatusLeisteZugriff.getInstanz().setStatusZeilenFehlerMeldung(
									text, img);
						} else
						{
							meldungsbereichZuruecksetzen();
							StatusLeisteZugriff.getInstanz().setStatusZeilenFehlerMeldung("",
									img);
						}
					}
				}
			}			
		});

		/*
		 * Layout der Gruppe
		 */
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;

		GridData gridDataGruppe = new GridData();
		gridDataGruppe.horizontalAlignment = GridData.FILL;
		gridDataGruppe.grabExcessHorizontalSpace = true;
		gridDataGruppe.grabExcessVerticalSpace = true;
		gridDataGruppe.verticalAlignment = GridData.FILL;

		gruppeEingabe.setLayoutData(gridDataGruppe);
		gruppeEingabe.setLayout(gridLayout);

	}

	private void erzeugeGruppeStatus()
	{
		_gruppeStatus = new Group(_elternComposite, SWT.NONE);
		_gruppeStatus.setText("Status");

		_textFeld = new Text(_gruppeStatus, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		_textFeld.setEditable(true);

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		_textFeld.setLayoutData(gridData);

	}

	/**
	 * Diese Klasse liefert ein {@link ContributionItem}, welches eine Uhr
	 * beinhaltet und in der Statusleiste positioniert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SlZeitItem extends ContributionItem
	{
		private Label _zeitLabel = null;

		private Text _textZeitfeld = null;

		private boolean _laufe = true;

		private int i = 0;

		private Thread thTimer = null;

		public SlZeitItem(String id)
		{
			super(id);
		}

		public void fill(Composite parent)
		{
			_zeitLabel = new Label(parent, SWT.NONE);
			_zeitLabel.setText(super.getId() + " Zeit:");
			_textZeitfeld = new Text(parent, SWT.BORDER);
			_textZeitfeld.setEditable(false);
			thTimer = new Thread(new Timer());
			thTimer.start();

		}

		public void dispose()
		{
			thTimer.interrupt();
			_laufe = false;

		}

		private class Timer implements Runnable
		{

			public void run()
			{
				while (_laufe)
				{
					Display.getDefault().syncExec(new Runnable()
					{
						public void run()
						{
							i++;
							// _textZeitfeld.setText("I: " + i);

							// DateFormat df =
							// DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
							// DateFormat.MEDIUM);
							DateFormat df = new SimpleDateFormat("HH:mm:ss");
							if (_laufe)
							{
								_textZeitfeld.setText(df.format(new Date()));
							}

						}
					});
					try
					{
						Thread.sleep(100);
					} catch (InterruptedException ie)
					{
							_debug.fine("Timerthread gestoppt => " + ie.toString());
							_laufe = false;

					}
				}

			}
		}

	}

	/**
	 * Diese Klasse liefert ein {@link ContributionItem}, welches in der
	 * Statusleiste positioniert werden kann.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class SlItem extends ContributionItem
	{
		private Label _infoLabel = null;

		private Text _infoText = null;

		private Button _buttonWithImage = null;

		private Text _externerText = null;

		public SlItem(String id)
		{
			super(id);
		}

		public void fill(Composite parent)
		{
			_infoLabel = new Label(parent, SWT.NONE);
			_infoLabel.setText("Info:");
			_infoText = new Text(parent, SWT.BORDER);
			_infoText.setText("Ein Infotext in der " + getId() + "-Version.");
			_infoText.setEditable(false);

			ImageDescriptor descriptor = Aktivator.getImageDescriptor(_bild1);

			_buttonWithImage = new Button(parent, SWT.FLAT);
			_buttonWithImage.setImage(descriptor.createImage());
		
			_buttonWithImage.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					if (_externerText != null)
					{
						_externerText.setText("Gelscht durch ID " + getId());
						_externerText.pack();
					} else
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
						mb.setText("Achtung!");
						mb
								.setMessage("Hinweis: Es wurde keine Referenz auf ein TextFeld bergeben!");
						mb.open();
					}
				}			
			});

		}
	}

	@Override
	public void setFocus()
	{
	}

}
