/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.pruefrw.zeitspezifikationselemente;

import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.kalender.CompositeZeitBereich;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IEintragBereich;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.IFelderFuellen;
import de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender.SchaltflaechenFabrikEreignisSystemKalenderDialog;

public class ViewEreignisSystemKalenderTest extends ViewPart
{

	public static final String ID = "de.bsvrz.buv.plugin.pruefrw.zeitspezifikationselemente.ViewEreignisSystemKalenderTest"; 
	private Composite top = null;
	private Group gruppeZeitgruppen = null;
	private Group gruppeSteuerung = null;
//	private Button buttonAktion1 = null;
	private ScrolledComposite scrolledComposite = null;
	private Composite _compositeInhalt;

	private int counter = 0;

	@Override
	public void createPartControl(Composite parent)
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		top = new Composite(parent, SWT.NONE);
		createGruppeZeitgruppen();
		top.setLayout(gridLayout);
		createGruppeSteuerung();
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes gruppeZeitgruppen
	 * 
	 */
	private void createGruppeZeitgruppen()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gruppeZeitgruppen = new Group(top, SWT.NONE);
		gruppeZeitgruppen.setLayout(new FillLayout());
		gruppeZeitgruppen.setLayoutData(gridData);
		createScrolledComposite();
		gruppeZeitgruppen.setText("Zeitgruppen:");
	}

	/**
	 * This method initializes gruppeSteuerung
	 * 
	 */
	private void createGruppeSteuerung()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		rowLayout.justify = false;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		gruppeSteuerung = new Group(top, SWT.NONE);
		gruppeSteuerung.setText("Kontrolle:");
		gruppeSteuerung.setLayout(rowLayout);
		gruppeSteuerung.setLayoutData(gridData1);


		IFelderFuellen felderFuellen = new IFelderFuellen()
		{

			@Override
			public void fuellen(LinkedList<IEintragBereich> bereiche)
			{
				// System.out.println("VON: " + bereich.getVonLesbar() + " BIS: " +
				// bereich.getBisLesbar());

				if (bereiche != null)
				{
					System.out.println("ANZ:= " + bereiche.size());
					if (bereiche.size() > 0)
					{
						aktionErzeugen(bereiche);
					}
				}

			}

		};
		SchaltflaechenFabrikEreignisSystemKalenderDialog.erzeugeButton(
				gruppeSteuerung, felderFuellen, null, null);

	}

	/**
	 * This method initializes scrolledComposite
	 * 
	 */
	private void createScrolledComposite()
	{
		scrolledComposite = new ScrolledComposite(gruppeZeitgruppen, SWT.BORDER
				| SWT.H_SCROLL | SWT.V_SCROLL);
		
		scrolledComposite.getVerticalBar().setIncrement(20);

		_compositeInhalt = new Composite(scrolledComposite, SWT.NONE);
		// _compositeInhalt.setSize(500, 9000);
		GridLayout gl = new GridLayout();
		gl.numColumns = 1;
		_compositeInhalt.setLayout(gl);

		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		// scrolledComposite.setMinSize(500, 500);

		scrolledComposite.setContent(_compositeInhalt);

//		new Button(_compositeInhalt, SWT.PUSH).setText("One");
//		new Button(_compositeInhalt, SWT.PUSH).setText("Two");
		scrolledComposite.setMinSize(_compositeInhalt.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
	}

	protected void aktionErzeugen(LinkedList<IEintragBereich> bereiche)
	{
		Iterator<IEintragBereich> it = bereiche.iterator();
		while (it.hasNext())
		{
			IEintragBereich eintragBereich = it.next();
			CompositeZeitBereich compositeZeitBereich = new CompositeZeitBereich(
					_compositeInhalt, SWT.NONE);
			compositeZeitBereich
					.setzeStartZeitPunkt(new Date(eintragBereich.getVon()));
			compositeZeitBereich.setzeEndZeitPunkt(new Date(eintragBereich.getBis()));
			scrolledComposite.setMinSize(_compositeInhalt.computeSize(SWT.DEFAULT,
					SWT.DEFAULT));
			_compositeInhalt.layout();
		}
	}

	protected void aktion1()
	{
		System.out.println("Einfgen...");
		Label label = new Label(_compositeInhalt, SWT.NONE);
		label.setText("Eingefgt: " + counter);
		counter++;

		scrolledComposite.setMinSize(_compositeInhalt.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));

		_compositeInhalt.layout();

	}

}
