package de.bsvrz.buv.plugin.sim;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.Data.TimeValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Simulationsverwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Das Verbindungsobjekt
   */
  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;
  
  /**
   * SynchronisationsObjekt fr den Singleton
   */
  private static Object _syncObject = new Object();

  /**
   * Liste der {@link ISimulationListener}, die ber einen neue Simulation informiert werden mchten.
   */
  private LinkedList<ISimulationListener> _simulationListener = new LinkedList<ISimulationListener>();

  /**
   * Liste der Simulationen
   */
  private List<Simulation> _eintraege = new LinkedList<Simulation>();

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Anmelden, wenn die Verbindung vorhanden ist.
      // Ansonsten wird dies dem Listener berlassen.
      ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

      MutableSet ms = konfigurationsObjekt.getMutableSet("Simulationen");

      if (ms != null)
      {
        ms.addChangeListener(this);
        _debug.fine("Listener Menge Simulationen angemeldet");
      }
      else{
        _debug.error("Menge Simulationen ist null");
        
        // Message Sender initialisieren
        ClientApplication app = _connection.getLocalApplicationObject();

        SystemObject user = (SystemObject)_connection.getLocalUser();
        MessageCauser causer = new MessageCauser(user, "Fehler: Anmelden auf Menge", app.getName());
        MessageSender instance = MessageSender.getInstance();
       
        instance.setApplicationLabel(app.getName());

        Long id = app.getId();        
        
        instance.sendMessage(id.toString(), 
            MessageType.SYSTEM_DOMAIN, 
            "", 
            MessageGrade.ERROR,
            (SystemObject)_connection.getLocalConfigurationAuthority(), 
            MessageState.NEW_MESSAGE, 
            causer,
            "Menge Simulationen ist null!");         
      }
      

    }
    else
      _debug.error("Plugin Simulation : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @return ClientDavInterface
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Fgt einen Listener zur Liste der SimulationListener hinzu.
   * 
   * @param listener
   */
  public void hinzufuegenSimulationListener(ISimulationListener listener)
  {
    if (!_simulationListener.contains(listener))
    {
      _simulationListener.add(listener);
    }
    else
    {
      _debug.error("Der SimulationZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }

  /**
   * Entfernt einen {@link ISimulationListener} von der Liste der Zuhrer.
   * 
   * @param ISimulationListener
   *          Zu entfernender {@link ISimulationListener}.
   */
  public void entferneSimulationListener(ISimulationListener listener)
  {
    if (_simulationListener.contains(listener))
    {
      _simulationListener.remove(listener);
    }
    else
    {
      _debug.error("Der SimulationZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link ISimulationListener} darber, dass eine Simulation gendert hat.
   * 
   */
  public void benachrichtigeSimulationListener(List<Simulation> eintraege)
  {
    SimulationEreignis ereignis = new SimulationEreignis(this, eintraege);

    Iterator<ISimulationListener> it = _simulationListener.iterator();
    while (it.hasNext())
    {
      ISimulationListener listener = it.next();
      listener.simulationEingang(ereignis);
    }
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    List<Simulation> listLoesch = new ArrayList<Simulation>();
    List<Simulation> listHinzu = new ArrayList<Simulation>();

    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();
      final DataDescription dd = result.getDataDescription();
      final SystemObject object = result.getObject();

      AttributeGroup atgZeit = _connection.getDataModel().getAttributeGroup("atg.simulationsZeitBereich");

      if (data != null)
      {

        if (atgZeit.equals(dd.getAttributeGroup()))
        {

          for (Simulation s : _eintraege)
          {
            if (s.getPid().equals(object.getPid()))
            {

              TimeValue tv1 = data.getItem("SimulationsBeginn").asTimeValue();
              TimeValue tv2 = data.getItem("SimulationsEnde").asTimeValue();

              s.setSimulationsstart(tv1.getMillis());
              s.setSimulationsende(tv2.getMillis());
            }

          }

        }
        else
        {

          Simulation simulation = new Simulation(object.getName());
          simulation.setPid(object.getPid());

          DynamicObject dynObj = (DynamicObject)object;

          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.simulationsEigenschaften");
          Data dataConf = dynObj.getConfigurationData(atg);
          SystemObject strecke = dataConf.getItem("SimulationsStreckenReferenz").asReferenceValue().getSystemObject();
          String variante = dataConf.getItem("SimulationsVariante").asScaledValue().getText();
          simulation.setSimulationsStrecke(strecke);
          simulation.setVariante(variante);

          if (result.getDataDescription().getAttributeGroup().getPid().equals("atg.simulationsSteuerungOnline"))
          {
            simulation.setGewaehlteSimulationsArt(Simulation.SimulationsArt.Online);
          }
          else if (result.getDataDescription().getAttributeGroup().getPid().equals("atg.simulationsSteuerungOffline"))
          {
            simulation.setGewaehlteSimulationsArt(Simulation.SimulationsArt.Offline);

          }

          String zustand = data.getItem("SimulationsZustand").asTextValue().getText();

          if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.neu.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.neu);
          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Vorstart.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.Vorstart);
          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Start.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.Start);
          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Pause.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.Pause);
          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Stop.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.Stop);
          }
          else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.gelscht.toString()))
          {
            simulation.getSimulationssteuerungDaten().setAktuellerZustand(
                SimulationssteuerungDaten.SimulationsZustand.gelscht);
          }

          String vorhanden = null;

          for (Simulation s : _eintraege)
          {
            if (s.getPid().equals(simulation.getPid()))
              vorhanden = s.getPid();
          }

          if (vorhanden == null)
            _eintraege.add(simulation);
          else
          {
            for (Simulation s : _eintraege)
            {
              if (s.getPid().equals(simulation.getPid()))
              {

                listLoesch.add(s);
                listHinzu.add(simulation);

              }
            }
          }
        }
      }
    }

    _eintraege.removeAll(listLoesch);
    _eintraege.addAll(listHinzu);

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        benachrichtigeSimulationListener(_eintraege);
      }
    });

  }

  /**
   * Holt die Liste der Simulationen
   * 
   * @return simulationen
   */
  public List<Simulation> getSimulationen()
  {

    return _eintraege;

  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    _debug.fine("update Menge: " + set.getName());

    if (set.getName().equals("Simulationen"))
    {

      List<Simulation> listAdd = new ArrayList<Simulation>();

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {

          SystemObject so = addedObjects[i];

          _debug.fine("added - " + addedObjects[i]);

          if (so.isOfType("typ.onlineSimulation"))
          {
            anmeldeLeseDaV(so.getPid(), "atg.simulationsSteuerungOnline", "asp.zustand", ReceiverRole.receiver());
          }
          else if (so.isOfType("typ.offlineSimulation"))
          {
            anmeldeLeseDaV(so.getPid(), "atg.simulationsSteuerungOffline", "asp.zustand", ReceiverRole.receiver());
            anmeldeLeseDaV(so.getPid(), "atg.simulationsZeitBereich", "asp.zustand", ReceiverRole.receiver());
          }

        }

        _eintraege.addAll(listAdd);

      }

      List<Simulation> listRem = new ArrayList<Simulation>();

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];

          _debug.fine("removed - " + removedObjects[i]);

          for (Simulation s : _eintraege)
          {
            if (s.getPid().equals(so.getPid()))
            {

              listRem.add(s);
              break;
            }
          }

        }
        _eintraege.removeAll(listRem);
      }

      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          benachrichtigeSimulationListener(_eintraege);
        }
      });

    }

  }

  /**
   * Fhrt das Anmelden auf die Simulationen durch
   */
  public void anmelden()
  {

    if (_connection != null)
    {

      ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

      ObjectSet objekte = konfigurationsObjekt.getObjectSet("Simulationen");

      List<SystemObject> list = objekte.getElements();

      
      List<SystemObject> listTmp = new ArrayList<SystemObject>();      
      for (SystemObject systemObject : list)
      {
        if (systemObject.isValid())
        {
          listTmp.add(systemObject);

        }
      }
      
     
      for (SystemObject systemObject : listTmp)
      {

        if (systemObject.isOfType("typ.onlineSimulation"))
        {
          anmeldeLeseDaV(systemObject.getPid(), "atg.simulationsSteuerungOnline", "asp.zustand", ReceiverRole
              .receiver());

        }
        else if (systemObject.isOfType("typ.offlineSimulation"))
        {
          anmeldeLeseDaV(systemObject.getPid(), "atg.simulationsSteuerungOffline", "asp.zustand", ReceiverRole
              .receiver());
          anmeldeLeseDaV(systemObject.getPid(), "atg.simulationsZeitBereich", "asp.zustand", ReceiverRole.receiver());

        }
      }
    }
  }

  /**
   * Bestimmt den Default-Bereich des DaV
   * 
   * @param connection
   * @return ConfigurationArea
   */
  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }
}
