package de.bsvrz.buv.plugin.sim;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * ber den Definitionsdialog kann ein Simulationseintrag neu angelegt werden.
 * 
 * <br>
 * 
 * {@link Definitionsdialog} wird aktiv, wenn in {@link Verwaltungsdialog} der Button "neu" gedrckt wurde.
 * 
 * <br>
 * <br>
 * 
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-2</li>
 * </ul>
 * 
 * @version $Revision: 1.1 $ / $Date: 2010/02/02 15:22:01 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class Definitionsdialog extends Dialog
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Datensatz hinter diesem Dialog.
   */
  private Simulation _simulationDatensatz = null;

  /**
   * Konstruktor fr neue Instanzen der Klasse {@link Definitionsdialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public Definitionsdialog(Shell parent)
  {
    super(parent);
  }

  /**
   * ffnet den Dialog {@link Definitionsdialog}.
   * 
   * @return Liefert den {@link Simulation}, wenn ein neuer SimulationsEintrag generiert wurde, sonst <code>null</code>.
   */
  public Simulation oeffne()
  {
    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    return _simulationDatensatz;
  }

  /**
   * ffnet den Dialog {@link Definitionsdialog} zum Bearbeiten eines Simulationseintrages.
   * 
   * @param simulationDatensatz
   *          SimulationsDatensatz, der bearbeitet werden soll.
   * 
   * @return Liefert <code>true</code>, wenn ein Simulationseintrag bearbeitet wurde, sonst <code>false</code>.
   *         <br>
   *         Wenn <code>true</code> geliefert wurde, kann dies die Klasse {@link Verwaltungsdialog} auswerten und die
   *         Listendarstellung der Simulationen aktualisieren.
   * 
   */
  public boolean oeffne(Simulation simulationDatensatz)
  {
    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    return false;
  }

  /**
   * Generiert die Elemente des Dialogs {@link Definitionsdialog}.
   * 
   */
  private void generiereGui()
  {
  }

  /**
   * Aktion hinter dem Button ok. bernimmt die aktuellen Einstellungen. Folgende Schritte werden durchgefhrt:
   * <ul>
   * <li>Prfung ob eine Simulationsvariante frei ist, diese wird dem neu erstellten Objekt zugeordnet.</li>
   * <li>Vergabe der PID des Objektes nach dem Schema sim.001 ... sim.999.</li>
   * <li>Zuordnung des neu erstellten Objekts zur entsprechenden Simulations-Verwaltung. </li>
   * <li>Ist keine Variante mehr frei ist, wird dies in einem Dialog mitgeteilet.</li>
   * <li>Prfung, ob der eingegebene Zeitbereich gltig ist. </li>
   * <li>Prfung, ob ein Name eingegeben wurde </li>
   * </ul>
   * 
   */
  private void aktionOk()
  {

  }

  /**
   * Aktion hinter dem Button Abbrechen. Dialog {@link Definitionsdialog} wird ohne bernahme der getroffenen
   * Einstellungen geschlossen.
   * 
   */
  private void aktionAbbrechen()
  {

  }

  /**
   * ffnet die Hilfe zum Dialog {@link Definitionsdialog}.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Aktivator.PLUGIN_ID + ".simulation";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Setzt die Art der Simulation (Online Simulation oder Offline Simulation)
   * 
   * @param onlineSimulation
   *          Wenn <code>true</code>, dann handelt es sich um eine Online Simulation. Bei <code>false</code>
   *          handels es sich um eine Offline Simulation.
   */
  private void setSimulationsArt(boolean onlineSimulation)
  {

  }

}
