package de.bsvrz.buv.plugin.sim;

import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.plugin.sim.SimulationssteuerungDaten.SimulationsZustand;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Bei dem Dialog zur Simulationssteuerung erscheint in der Titelzeile der Name der Simulation. ber die Schaltflchen
 * lassen sich die aktuelle Simulation starten (Start), anhalten (Pause, Fortsetzung mit Start), stoppen (Stop) und bei
 * Offline-Simulationen um einen Schritt weierspringen (Einzelschritt). ber den Schieberegler lsst sich bei
 * Offline-Simulationen die Triggergeschwindigkeit einstel-len. Der Schieberegler, die aktuelle Simulationszeit und die
 * Schaltflchen Einzelschritt und Pause sind bei der Online-Simulation nicht darzustellen.
 * 
 * <br>
 * 
 * Wird die Simulationssteuerung geschlossen, so wird die Simulation dadurch nicht beendet oder angehalten. Das
 * Simulationssteuerungsfenster ist dann ber den Dialog Verwaltung Simulation wieder darstellbar.
 * 
 * <br>
 * <br>
 * 
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-3</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:03:56 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class SimulationSteuerungsdialog extends Dialog implements ISimulationListener, ClientReceiverInterface
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Shell shell = null; // @jve:decl-index=0:visual-constraint="204,32"

  private Composite compositeBottom = null;

  private Button buttonStart = null;

  private Button buttonStop = null;

  private Group groupZeitpunkt = null;

  private Group groupGeschwindigkeit = null;

  private Scale scale = null;

  private DatumZeit zeitPunkt = null;
  
  //private Text text2 = null;

  //private Text text11 = null;

  private Button buttonPause = null;

  private Button buttonEinzelschritt = null;

  private Composite compositeScale = null;

  private Label label0 = null;

  private Label label05 = null;

  private Label label1 = null;

  private Label label2 = null;

  private Label label4 = null;

  private Simulation _eintrag = null;

  private DaVKommunikation _daVKommunikation;

  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link SimulationSteuerungsdialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public SimulationSteuerungsdialog(Shell parent)
  {
    super(parent);
    _daVKommunikation = DaVKommunikation.getInstanz();
    _daVKommunikation.hinzufuegenSimulationListener(this);
  }

  /**
   * ffnet den Dialog {@link Simulationssteuerung}.
   * 
   * @param simulationssteuerungDatensatz
   *          Simulation, die gesteuert werden soll.
   * 
   */
  public Simulation oeffne(Simulation eintrag)
  {

    _eintrag = eintrag;

     shell = new Shell();
    //shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.CLOSE);
    shell.setText("Steuerung Simulation '" + eintrag.getName() + "'");

    shell.setLayout(new GridLayout());
    createGroupGeschwindigkeit();
    createGroupZeitpunkt();
    createCompositeBottom();

    aktiviereGuiElemente();

    if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();
      AttributeGroup atg = dataModel.getAttributeGroup("atg.simulationsZeit");
      Aspect asp = dataModel.getAspect("asp.standard");
      DataDescription dd = new DataDescription(atg, asp);

      _daVKommunikation.holeDaVVerbindung().subscribeReceiver(this, dataModel.getObject(_eintrag.getPid()), dd,
          ReceiveOptions.normal(), ReceiverRole.receiver());

    }

    shell.setSize(new Point(349, 230));
    shell.open();

    Display display = getParent().getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();
    
    return _eintrag;

  }

  private void aktiviereGuiElemente()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();
    
    if (!getShell().isDisposed()){

      if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.neu))
      {
        buttonStart.setEnabled(false);
        buttonStop.setEnabled(false);
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(false);
      }
      else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Vorstart))
      {
        buttonStart.setEnabled(true);
        buttonStop.setEnabled(false);
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(false);
      }
      else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Start))
      {
        buttonStart.setEnabled(false);
        buttonStop.setEnabled(true);
        if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
        {
          buttonPause.setEnabled(true);
          buttonEinzelschritt.setEnabled(false);
        }
      }
      else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Pause))
      {
        buttonStart.setEnabled(true);
        buttonStop.setEnabled(true);
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(true);
      }
      else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Stop))
      {
        buttonStart.setEnabled(true);
        buttonStop.setEnabled(false);
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(false);
      }
      else if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.gelscht))
      {
        buttonStart.setEnabled(false);
        buttonStop.setEnabled(false);
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(false);
      }
  
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {    
        scale.setEnabled(false);
  //      text11.setEnabled(false);
  //      text2.setEnabled(false);
        zeitPunkt.setEnabled(false);
        
        buttonPause.setEnabled(false);
        buttonEinzelschritt.setEnabled(false);
        
      }
      
    }

  }

  /**
   * Generiert die Elemente des Dialogs {@link SimulationSteuerungsdialog}.
   * 
   */
  private void generiereGui()
  {
  }

  /**
   * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
   * 
   */
  private void aktionStart()
  {

    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");
      }

      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();
      if (zustand.equals(SimulationssteuerungDaten.SimulationsZustand.Stop))
        data.getItem("SimulationsZustand").asTextValue().setText("Vorstart");
      else
        data.getItem("SimulationsZustand").asTextValue().setText("Start");

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asScaledValue().set(holeWertSchieber());
      }

      Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
      sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
          SenderRole.sender());

      // shell.dispose();

    }

  }

  private Float holeWertSchieber()
  {
    Float wert = new Float(0);

    if (scale != null)
    {
      switch (scale.getSelection())
      {
        case 0:
          wert = 0.0f;
          break;
        case 1:
          wert = 0.25f;
          break;
        case 2:
          wert = 0.5f;
          break;
        case 3:
          wert = 0.75f;
          break;
        case 4:
          wert = 1.0f;
          break;
        case 5:
          wert = 1.5f;
          break;
        case 6:
          wert = 2.0f;
          break;
        case 7:
          wert = 3.0f;
          break;
        case 8:
          wert = 4.0f;
          break;
        default:
          wert = 0.0f;
      }
    }
    return wert;
  }

  /**
   * Verwirft die nderungen am regulren Ausdruck und schliet den Dialog.
   * 
   */
  private void aktionStop()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");

      }
      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();

      data.getItem("SimulationsZustand").asTextValue().setText("Stop");
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asScaledValue().set(holeWertSchieber());
      }

      Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
      sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
          SenderRole.sender());

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {

        atg = dataModel.getAttributeGroup("atg.simulationsZeit");
        asp = dataModel.getAspect("asp.standard");
        DataDescription dd = new DataDescription(atg, asp);

        _daVKommunikation.holeDaVVerbindung().subscribeReceiver(this, dataModel.getObject(_eintrag.getPid()), dd,
            ReceiveOptions.normal(), ReceiverRole.receiver());

      }
    }
    //shell.dispose();

  }

  /**
   * Verwirft die nderungen am regulren Ausdruck und schliet den Dialog.
   * 
   */
  private void aktionPause()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");

      }
      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();

      data.getItem("SimulationsZustand").asTextValue().setText("Pause");
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asScaledValue().set(holeWertSchieber());
      }

      Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
      sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
          SenderRole.sender());

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {

        atg = dataModel.getAttributeGroup("atg.simulationsZeit");
        asp = dataModel.getAspect("asp.standard");
        DataDescription dd = new DataDescription(atg, asp);

        _daVKommunikation.holeDaVVerbindung().subscribeReceiver(this, dataModel.getObject(_eintrag.getPid()), dd,
            ReceiveOptions.normal(), ReceiverRole.receiver());

      }
      // shell.dispose();

    }
  }
  
  
  private void aktionEinzelschritt()
  {
    SimulationsZustand zustand = _eintrag.getSimulationssteuerungDaten().getAktuellerZustand();

    if (_eintrag != null)
    {

      DataModel dataModel = _daVKommunikation.holeDaVVerbindung().getDataModel();

      AttributeGroup atg = null;

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Online))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOnline");
      }
      else if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        atg = dataModel.getAttributeGroup("atg.simulationsSteuerungOffline");

      }
      Aspect asp = dataModel.getAspect("asp.vorgabe");
      Data data = _daVKommunikation.holeDaVVerbindung().createData(atg);

      SystemObject so = (SystemObject)_daVKommunikation.holeDaVVerbindung().getLocalApplicationObject();

      data.getItem("SimulationsZustand").asTextValue().setText("Pause");
      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {
        data.getItem("Simulationsgeschwindigkeit").asScaledValue().set(holeWertSchieber());
      }

      Sender sender = new Sender(_daVKommunikation.holeDaVVerbindung());
      sender.sende(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(_eintrag.getPid()), atg, asp, data,
          SenderRole.sender());

      if (_eintrag.getGewaehlteSimulationsArt().equals(Simulation.SimulationsArt.Offline))
      {

        atg = dataModel.getAttributeGroup("atg.simulationsZeit");
        asp = dataModel.getAspect("asp.standard");
        DataDescription dd = new DataDescription(atg, asp);

        _daVKommunikation.holeDaVVerbindung().subscribeReceiver(this, dataModel.getObject(_eintrag.getPid()), dd,
            ReceiveOptions.normal(), ReceiverRole.receiver());

      }
      // shell.dispose();

    }

  }

  /**
   * This method initializes compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = true;
    rowLayout.wrap = false;
    rowLayout.fill = false;
    rowLayout.pack = false;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData1);
    compositeBottom.setLayout(rowLayout);
    buttonStart = new Button(compositeBottom, SWT.NONE);
    buttonStart.setText("Start");
    buttonStart.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionStart();
			}			
		});
    
    
    buttonPause = new Button(compositeBottom, SWT.NONE);
    buttonPause.setText("Pause");
    buttonPause.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionPause();
			}			
		});

    buttonStop = new Button(compositeBottom, SWT.NONE);
    buttonStop.setText("Stop");   
    buttonStop.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionStop();
			}			
		});
    
    
    buttonEinzelschritt = new Button(compositeBottom, SWT.NONE);
    buttonEinzelschritt.setText("Einzelschritt");    
    buttonEinzelschritt.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionEinzelschritt();
			}			
		});

  }

  /**
   * This method initializes groupZeitpunkt
   * 
   */
  private void createGroupZeitpunkt()
  {
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 2;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = false;
    gridData3.verticalAlignment = GridData.FILL;
    groupZeitpunkt = new Group(shell, SWT.NONE);
    groupZeitpunkt.setText("Simulationszeitpunkt");
    groupZeitpunkt.setLayoutData(gridData3);
    groupZeitpunkt.setLayout(gridLayout1);
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.BEGINNING;
    gridData10.grabExcessHorizontalSpace = false;
    gridData10.verticalAlignment = GridData.CENTER;
    zeitPunkt = new DatumZeit(groupZeitpunkt, SWT.NONE, Eingabetyp.datumuhrEinfach, false, false);
    zeitPunkt.setLayoutData(gridData10);
//    text2 = new Text(groupZeitpunkt, SWT.BORDER);
//    text11 = new Text(groupZeitpunkt, SWT.BORDER);
  }

  /**
   * This method initializes groupGeschwindigkeit
   * 
   */
  private void createGroupGeschwindigkeit()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.FILL;
    gridData4.grabExcessHorizontalSpace = true;
    gridData4.grabExcessVerticalSpace = false;
    gridData4.verticalAlignment = GridData.CENTER;
    groupGeschwindigkeit = new Group(shell, SWT.NONE);
    groupGeschwindigkeit.setText("Steuerung Ablaufgeschwindigkeit");
    groupGeschwindigkeit.setLayout(gridLayout);
    groupGeschwindigkeit.setLayoutData(gridData4);
    scale = new Scale(groupGeschwindigkeit, SWT.NONE);
    scale.setLayoutData(gridData5);
    scale.setMinimum(0);
    scale.setMaximum(8);
    scale.setIncrement(2);
    scale.setPageIncrement(1);
    scale.setSelection(4);
    scale.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        super.widgetSelected(e);
      }
    });

    createCompositeScale();
  }

  /**
   * This method initializes compositeScale
   * 
   */
  private void createCompositeScale()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.widthHint = -1;
    gridData.horizontalIndent = 0;
    gridData.verticalAlignment = GridData.CENTER;
    compositeScale = new Composite(groupGeschwindigkeit, SWT.NONE);
    compositeScale.setLayoutData(gridData);
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 5;
    gridLayout.marginWidth = 11;
    gridLayout.makeColumnsEqualWidth = true;
    gridLayout.horizontalSpacing = 56;
    compositeScale.setLayout(gridLayout);
    label0 = new Label(compositeScale, SWT.NONE);
    label0.setText("0");
    label05 = new Label(compositeScale, SWT.NONE);
    label05.setText("0,5");
    label1 = new Label(compositeScale, SWT.NONE);
    label1.setText("1");
    label2 = new Label(compositeScale, SWT.NONE);
    label2.setText("2");
    label4 = new Label(compositeScale, SWT.NONE);
    label4.setText("4");
  }

  @Override
  public void simulationEingang(SimulationEreignis e)
  {
    for (Simulation s : e.getSimulationen())
    {
      if (_eintrag != null && s.getPid().equals(_eintrag.getPid()))
      {
        _eintrag = s;
        break;
      }
    }

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        aktiviereGuiElemente();

      }
    });

  }

  @Override
  public void update(ResultData[] results)
  {
    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();
      final DataDescription dd = result.getDataDescription();
      final SystemObject object = result.getObject();

      if (data != null)
      {

        long l = data.getItem("Zeit").asTimeValue().getMillis();
        final Date d = new Date();
        d.setTime(l);

        Display.getDefault().asyncExec(new Runnable() {
          public void run()
          {
            if (!getShell().isDisposed()) 
              zeitPunkt.setDatum(d);

          }
        });
      }
    }
  }

  public Shell getShell()
  {
    return shell;
  }

}
