package de.bsvrz.buv.plugin.syskal;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderArbeiter;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der SystemkalenderVerwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface, MutableSetChangeListener
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;
  
  /**
   * SynchronisationsObjekt fr den Singleton
   */
  private static Object _syncObject = new Object();

  /**
   * Liste der {@link IBenutzerZuhoerer}, die ber einen neue angemeldeten Benuter informiert werden mchten.
   */
  private LinkedList<ISystemKalenderListener> _systemKalenderListener = new LinkedList<ISystemKalenderListener>();

  private List<Eintrag> _eintraege = new LinkedList<Eintrag>();

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
//    if (_instanz == null) { 
//      synchronized(_syncObject) { 
//          if (_instanz == null) { 
//              _instanz = new DaVKommunikation();
//          }
//      }
//    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Anmelden, wenn die Verbindung vorhanden ist.
      // Ansonsten wird dies dem Listener berlassen.
      SystemkalenderArbeiter arbeiter = SystemkalenderArbeiter.getInstance(_connection, _connection
          .getLocalConfigurationAuthority().getPid());
      arbeiter.starteSystemKalenderArbeiter();

//      SystemkalenderArbeiter arbeiter = SystemkalenderArbeiter.getInstanz();
//      arbeiter.starteSystemKalenderArbeiter(_connection, _connection
//        .getLocalConfigurationAuthority().getPid());
    }
    else
      _debug.error("Plugin Systemkalendereintrag : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Fgt einen Listener zur Liste der SystemkalenderListener hinzu.
   * 
   * @param listener
   */
  public void hinzufuegenSystemKalenderListener(ISystemKalenderListener listener)
  {
    if (!_systemKalenderListener.contains(listener))
    {
      _systemKalenderListener.add(listener);
    }
    else
    {
      _debug
          .error("Der SystemKalenderZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt.");
    }
  }

  /**
   * Entfernt einen {@link ISystemKalenderListener} von der Liste der Zuhrer.
   * 
   * @param benutzerZuhoerer
   *          Zu entfernender {@link ISystemKalenderListener}.
   */
  public void entferneSystemKalenderListener(ISystemKalenderListener listener)
  {
    if (_systemKalenderListener.contains(listener))
    {
      _systemKalenderListener.remove(listener);
    }
    else
    {
      _debug.error("Der SystemKalenderZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }

  /**
   * Benachrichtigt alle registrierten {@link ISystemKalenderListener} darber, dass ein Eintrag verndert wurde.
   * 
   */
  public void benachrichtigeSystemKalenderListener(List<Eintrag> eintraege)
  {
    SystemKalenderEreignis ereignis = new SystemKalenderEreignis(this, eintraege);

    Iterator<ISystemKalenderListener> it = _systemKalenderListener.iterator();
    while (it.hasNext())
    {
      ISystemKalenderListener listener = it.next();
      listener.systemKalenderEingang(ereignis);
    }
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

    ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

    MutableSet ms = konfigurationsObjekt.getMutableSet("SystemKalenderEintrge");

    if (ms != null)
    {
      ms.addChangeListener(this);
      _debug.fine("Listener Menge SystemKalenderEintrge angemeldet");
      
    }
    else{
      _debug.error("Menge SystemKalenderEintrge ist null!");
      
      ClientApplication app = _connection.getLocalApplicationObject();

      SystemObject user = (SystemObject)_connection.getLocalUser();
      MessageCauser causer = new MessageCauser(user, "Fehler: Anmelden auf Menge", app.getName());
      MessageSender instance = MessageSender.getInstance();
      instance.setApplicationLabel(app.getName());

      Long id = app.getId();        
      
      instance.sendMessage(id.toString(), 
          MessageType.SYSTEM_DOMAIN, 
          "", 
          MessageGrade.ERROR,
          (SystemObject)_connection.getLocalConfigurationAuthority(), 
          MessageState.NEW_MESSAGE, 
          causer,
          "Menge SystemKalenderEintrge ist null!");         
    }

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    List<Eintrag> listLoesch = new ArrayList<Eintrag>();
    List<Eintrag> listHinzu = new ArrayList<Eintrag>();

    for (int i = 0; i < results.length; i++)
    {

      final ResultData result = results[i];
      final Data data = result.getData();

      if (data != null)
      {

        Eintrag eintrag = new Eintrag();

        eintrag.setPid(result.getObject().getPid());
        
        String definition = data.getItem("Definition").asTextValue().getText();
        
        if (definition.contains(":="))
        {

          String[] split = definition.split(":=");

          if (split.length == 2)
          {

            eintrag.setName(split[0]);

            if (split[1].contains("<") && split[1].contains(">"))
            {

              eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.Zeitbereichsangabe);

              String repl = split[1].replaceAll("[<>]", "");

              if (repl.contains("(") && repl.contains(")"))
              {

                int start = repl.indexOf("(");
                int ende = repl.indexOf(")");

                String dat = repl.substring(0, start);
                String[] splitDat = dat.split("-");

                String sub = repl.substring(start + 1, ende);

                int[] a = new int[10];
                int[] e = new int[10];
                int cnta = 0;
                int cnte = 0;

                for (int loc = 0; loc < sub.length(); loc++)
                {
                  char current = sub.charAt(loc);
                  if (current == '{')
                    a[cnta++] = loc + 1;
                  if (current == '}')
                    e[cnte++] = loc;
                }

                if (cnta == cnte)
                {

                  String[] sub2 = new String[cnta];

                  for (int j = 0; j < cnta; j++)
                  {

                    sub2[j] = sub.substring(a[j], e[j]);                    
                    split = sub2[j].split("-");
                    
                    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
                    Date d1 = null;
                    Date d2 = null;
                    try
                    {
                      d1 = sdf.parse(splitDat[0] + " " + split[0]);
                      d2 = sdf.parse(splitDat[1] + " " + split[1]);
                    }
                    catch (ParseException e1)
                    {
                      e1.printStackTrace();
                    }
                    eintrag.setAnfangsZeitpunkt(d1.getTime());
                    eintrag.setEndZeitpunkt(d2.getTime());
                    
//                    eintrag.setAnfangsZeitpunkt(splitDat[0] + " " + split[0].substring(0, 5));
//                    eintrag.setEndZeitpunkt(splitDat[1] + " " + split[1].substring(0, 5));

                  }
                }
              }
              else
              {

                String[] split2 = repl.split("-");

                if (split.length == 2)
                {
                  SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
                  Date d1 = null;
                  Date d2 = null;
                  try
                  {
                    d1 = sdf.parse(split2[0]);
                    d2 = sdf.parse(split2[1]);
                  }
                  catch (ParseException e1)
                  {
                    e1.printStackTrace();
                  }
                  eintrag.setAnfangsZeitpunkt(d1.getTime());
                  eintrag.setEndZeitpunkt(d2.getTime());
                  
//                  String regex = "[. :,]";
//
//                  String[] split3 = split2[0].split(regex);
//                  String[] split4 = split2[1].split(regex);
//
//                  eintrag.setAnfangsZeitpunkt(split3[0] + "." + split3[1] + "." + split3[2] + " " + split3[3] + ":"
//                      + split3[4]);
//                  eintrag.setEndZeitpunkt(split4[0] + "." + split4[1] + "." + split4[2] + " " + split4[3] + ":"
//                      + split4[4]);
                }
                else
                  _debug.fine("Fehler Definition: " + definition);
              }

            }
            else if (split[1].contains(".") && split[1].contains(","))
            {

              eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.Datumseingabe);
              eintrag.setDatum(split[1]);

            }
            else
            {

              eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.FreieDefinition);
                            
              eintrag.setFreieDefinition(split[1]);

            }
          }
          else
            _debug.fine("Fehler Definition: " + definition);
        }
        else
        {
          eintrag.setArtDerZeitspezifikation(Eintrag._artDerZeitspezifikation.VordefinierterEintrag);
          eintrag.setName(definition);
          eintrag.setVordefinierterEintrag(definition);
        }

        String vorhanden = null;

        for (Eintrag e : _eintraege)
        {
          if (e.getPid().equals(eintrag.getPid()))
            vorhanden = e.getPid();
        }

        if (vorhanden == null)
          _eintraege.add(eintrag);
        else
        {
          for (Eintrag e : _eintraege)
          {
            if (e.getPid().equals(eintrag.getPid()))
            {

              listLoesch.add(e);
              listHinzu.add(eintrag);

            }

          }
        }
      }
    }

    _eintraege.removeAll(listLoesch);
    _eintraege.addAll(listHinzu);

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        benachrichtigeSystemKalenderListener(_eintraege);
      }
    });

  }

  /**
   * Holt die Liste der SystemkalenderEintraege
   * 
   * @return
   */
  public List<Eintrag> getSystemKalenderEintraege()
  {

    return _eintraege;

  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    _debug.fine("update Menge: " + set.getName());

    if (set.getName().equals("SystemKalenderEintrge"))
    {

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {

          SystemObject so = addedObjects[i];

          _debug.fine("added - " + addedObjects[i]);

          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
          Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
          DataDescription dd = new DataDescription(atg, asp);

          _connection.subscribeReceiver(this, so, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

        }

      }

      List<Eintrag> list = new ArrayList<Eintrag>();

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];

          _debug.fine("removed - " + removedObjects[i]);

          for (Eintrag e : _eintraege)
          {
            if (e.getPid().equals(so.getPid()))
            {

              list.add(e);
              break;
            }
          }

        }

      }

      _eintraege.removeAll(list);

      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          benachrichtigeSystemKalenderListener(_eintraege);
        }
      });

    }

  }

  /**
   * Erzeugt einen Systemkalendereintrag anhand der bergebenen Pid und des Namen
   * 
   * @param pid
   * @param name
   */
  public void erzeugeSystemKalenderEintrag(String pid, String name)
  {

    DataModel dataModel = _connection.getDataModel();

    // ConfigurationArea configArea = dataModel.getConfigurationArea("kb.default.testKonfiguration");
    // ConfigurationObject configObj = (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    ConfigurationAuthority configObj = _connection.getLocalConfigurationAuthority();

    DynamicObjectType dynObjTyp = (DynamicObjectType)dataModel.getType("typ.systemKalenderEintrag");

    AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
    VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjTyp, configObj,
        attributgruppe, aspekt);
    Data data = _connection.createData(attributgruppe);
    Data[] datas = new Data[1];
    datas[0] = data;

//    if (dataModel.getObject(pid) == null)
    {

      vewEigenschaften.erzeuge(pid, name, "SystemKalenderEintrge", null);

    }

  }

  /**
   * Lscht einen Eintrag anhand der bergegeben Pid
   * 
   * @param pid
   */
  public void loescheSystemKalenderEintrag(String pid)
  {

    DataModel dataModel = _connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup("atg.systemKalenderEintrag");
    Aspect asp = dataModel.getAspect("asp.parameterSoll");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, dataModel.getObject(pid), dd);

    // ConfigurationArea configArea = dataModel.getConfigurationArea("kb.default.testKonfiguration");
    // ConfigurationObject confObject = (ConfigurationObject)dataModel.getObject("kv.testKonfiguration");

    ConfigurationArea configArea = bestimmeDefaultBereich(_connection);
    ConfigurationAuthority confObject = _connection.getLocalConfigurationAuthority();

    DynamicObjectType dynObjType = (DynamicObjectType)dataModel.getType("typ.systemKalenderEintrag");

    AttributeGroup attributgruppe = dataModel.getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspekt = dataModel.getAspect("asp.eigenschaften");
    VerwaltungDynObj vewEigenschaften = new VerwaltungDynObj(_connection, dataModel, configArea, dynObjType,
        confObject, attributgruppe, aspekt);
    Data data = _connection.createData(attributgruppe);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewEigenschaften.setDynamicObject((DynamicObject)dataModel.getObject(pid));
    vewEigenschaften.loesche(pid, "SystemKalenderEintrge");

  }

  /**
   * Fhrt die Anmeldung durch
   */
  public void anmeldenNeu()
  {

    if (_connection != null)
    {

      ConfigurationObject konfigurationsObjekt = (ConfigurationObject)_connection.getLocalConfigurationAuthority();

      ObjectSet objekte = konfigurationsObjekt.getObjectSet("SystemKalenderEintrge");

      List<SystemObject> list = objekte.getElements();

      List<SystemObject> listTmp = new ArrayList<SystemObject>();
      for (SystemObject systemObject : list)
      {
        if (systemObject.isValid())
        {

          listTmp.add(systemObject);
        }
      }


      anmeldeLeseDaV(listTmp, "atg.systemKalenderEintrag", "asp.parameterSoll", ReceiverRole.receiver());

    }

  }

  /**
   * Bestimmt des Default-Bereich des DaV
   * 
   * @param connection
   * @return
   */
  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }
  
  
  public boolean isObjektVorhanden(String pid){
    
    SystemObject object = _connection.getDataModel().getObject(pid);
    
    if (object != null)
      return true;
    
    return false;
  }
  

}
