/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * bernimmt die Kommunikation mit dem DaV und enthlt Funktionen zum Abfragen
 * der im DaV hinterlegten Berechtigungen.
 * 
 * Diese Klasse implementiert das Singleton-Entwurfsmuster.
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:12:25 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class SystemObjekteSystem
{
	/**
	 * Liefert die einzigste Instanz der Klasse {@link SystemObjekteSystem}.
	 */
	private static SystemObjekteSystem _instanz = null;

	/**
	 * Verbindung zum DaV.
	 */
	private static ClientDavInterface _connection = null;

	/**
	 * Die Liste mit allen Systemobjekten
	 */
	private List<SystemObject> _systemObjekte = new ArrayList<SystemObject>();

	/**
	 * Die Liste mit den dazugehrigen Pid's
	 */
	private List<String> _systemObjektePid = new ArrayList<String>();

	/**
	 * Liste der {@link IOberflaechenBerechtigungZuhoerer}, die ber die
	 * Berechtigung informiert werden mchten.
	 */
	private LinkedList<ISystemObjekteZuhoerer> _systemObjekteZuhoerer = new LinkedList<ISystemObjekteZuhoerer>();

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link SystemObjekteSystem}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link SystemObjekteSystem}.
	 */
	public static SystemObjekteSystem getInstanz()
	{
		if (_instanz == null)
		{
			_instanz = new SystemObjekteSystem();
		}

		return _instanz;
	}

	/**
	 * Privater Konstruktor des Singletons.
	 * 
	 */
	private SystemObjekteSystem()
	{
		SystemObjekteSystem._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

		/*
		 * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt
		 * werden.
		 */
		IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer()
		{
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				_connection = null;
			}

			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				_connection = e.getDaVVerbindung();
			}

		};

		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(zuhoerer);

		// Display.getDefault().asyncExec(new Runnable(){
		//
		// @Override
		// public void run()
		// {
		holeSystemObjekte();
		// }
		//      
		// });

	}

	/**
	 * Fgt einen Listener zur Liste der SystemObjekteListener hinzu.
	 * 
	 * @param listener
	 */
	public void hinzufuegenSystemObjekteZuhoerer(ISystemObjekteZuhoerer listener)
	{
		if (!_systemObjekteZuhoerer.contains(listener))
		{
			_systemObjekteZuhoerer.add(listener);
		}

	}

	/**
	 * Entfernt einen {@link ISystemObjekteZuhoerer} von der Liste der Zuhrer.
	 * 
	 * @param systemObjekteZuhoerer
	 *          Zu entfernender {@link ISystemObjekteZuhoerer}.
	 */
	public void entferneSystemObjekteZuhoerer(ISystemObjekteZuhoerer listener)
	{
		if (_systemObjekteZuhoerer.contains(listener))
		{
			_systemObjekteZuhoerer.remove(listener);
		}

	}

	/**
	 * Benachrichtigt alle registrierten {@link ISystemObjekteZuhoerer} darber,
	 * dass ein neuer Benutzer sich angemeldet hat.
	 * 
	 */
	public void benachrichtigeSystemObjekteZuhoerer(SystemObjekteEreignis ereignis)
	{

		Iterator<ISystemObjekteZuhoerer> it = _systemObjekteZuhoerer.iterator();
		while (it.hasNext())
		{
			ISystemObjekteZuhoerer listener = it.next();

			if (ereignis.getAenderung())
				listener.aenderung(ereignis);

		}
	}

	/**
	 * Holt das Verbindungsobjekt
	 * 
	 * @return
	 */
	public ClientDavInterface getConnection()
	{
		return _connection;
	}

	/**
	 * Holt alle SystemObjekte aus der Konfiguration und fgt sie der Liste der
	 * SystemObjekte hinzu. Zustzlich wird berwacht, ob dynamische Objekte der
	 * Konfiguration hinzugefgt bzw. aus der Konfiguration entfernt werden. Ist
	 * dies der Fall, werden alle Zuhoerer ber die neuen bzw. veralteten Objekte
	 * informiert.
	 */
	private void holeSystemObjekte()
	{

		if (_connection != null)
		{
			DataModel dataModel = _connection.getDataModel();

			try
			{
				SystemObjectType type = dataModel.getType("typ.konfigurationsBereich");
				// Liste aller Systemobjekte
				List<SystemObject> objects = type.getObjects();

				for (SystemObject so1 : objects)
				{
					ConfigurationArea element = (ConfigurationArea) so1;
					Collection<SystemObject> currentObjects = element.getCurrentObjects();

					for (SystemObject so2 : currentObjects)
					{
						_systemObjekte.add(so2);
						_systemObjektePid.add(so2.getPid());
					}
				}

				List<SystemObjectType> baseTypes = _connection.getDataModel()
						.getBaseTypes();
				for (SystemObjectType sot : baseTypes)
				{

					if (sot.isOfType("typ.dynamischesObjekt"))
					{

						DynamicObjectType dot = (DynamicObjectType) sot;

						dot
								.addObjectCreationListener(new DynamicObjectType.DynamicObjectCreatedListener()
								{

									@Override
									public void objectCreated(DynamicObject createdObject)
									{
										_systemObjekte.add(createdObject);
										_systemObjektePid.add(createdObject.getPid());

										SystemObjekteEreignis ereignins = new SystemObjekteEreignis(
												this, true, createdObject, null);

										benachrichtigeSystemObjekteZuhoerer(ereignins);
									}
								});

						dot.addInvalidationListener(new InvalidationListener()
						{

							@Override
							public void invalidObject(DynamicObject dynamicObject)
							{
								_systemObjekte.remove(dynamicObject);
								_systemObjektePid.remove(dynamicObject.getPid());

								SystemObjekteEreignis ereignins = new SystemObjekteEreignis(
										this, false, null, dynamicObject);

								benachrichtigeSystemObjekteZuhoerer(ereignins);

							}
						});
					}
				}
			}
			catch (RuntimeException e)
			{
				e.printStackTrace();
			}
		}
	}

	/**
	 * Holt die Liste der SystemObjekte
	 * 
	 * @return Liste mit SystemObjekten
	 */
	public List<SystemObject> getSystemObjekte()
	{
		return _systemObjekte;
	}

	/**
	 * Holt die Liste der Pids der SystemObjekte
	 * 
	 * @return Liste mit Pids
	 */
	public List<String> getSystemObjektePid()
	{
		return _systemObjektePid;
	}

	public static void setConnection(ClientDavInterface connection)
	{
		_connection = connection;
	}

}
