/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen.dialog;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.EinstellungenBenutzerSpezifischNetzweit;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class DialogEinstellungenWaehler extends Dialog
{
	private Debug _debug = Debug.getLogger();

	protected static final String IStructuredSelection = null;

	private Shell _shell = null;

	private Composite _compositeOben = null;

	private Composite _compositeUnten = null;

	private Composite _compositeUntenRechts = null;

	private Button _schaltflaecheOk = null;

	private Button _schaltflaecheAbbrechen = null;

	private Group _gruppeBenutzer = null;

	private Group _gruppeParameter = null;

	private TableViewer _tvBenutzer = null;

	private TableViewer _tvParameter = null;

	/**
	 * Liefert die Verbindung zum DaV
	 */
	private DaVVerbindung _dav = DaVVerbindung.getInstanz();

	/**
	 * Benutzerspezifische, netzweite Einstellungen. Initialisierung erfolgt i.d.
	 * Methode initialisiere.
	 */
	private EinstellungenBenutzerSpezifischNetzweit _ebsn = null;

	/**
	 * Speichert den selektierten Parameter
	 */
	private String[] _selektierterParameter = null;

	public static void main(String[] args)
	{
		final Display display = new Display();
		final Shell parent = new Shell(display, SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL);

		DialogEinstellungenWaehler thisClass = new DialogEinstellungenWaehler(
				parent);
		thisClass.oeffne();
	}

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 */
	public DialogEinstellungenWaehler(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * ffnet den Dialog und liefert den selektierten Parameter zurck.
	 * 
	 * @return Liefert den selektierten Parameter als Array vom Typ String (erster
	 *         Wert == Parametername, zweiter Wert == Parameterwert) oder liefert
	 *         <code>null</code> wenn kein Parameter selektiert oder abgebrochen
	 *         wurde.
	 */
	public String[] oeffne()
	{
		erzeugeShell();
		initialisiereBenutzerliste();

		_shell.open();
		_shell.setSize(_shell.getBounds().width + 1, _shell.getBounds().height);

		Display display = getParent().getDisplay();

		// Positionieren des Dialogs bei der Position des Mauszeigers
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _selektierterParameter;
	}

	private void initialisiereBenutzerliste()
	{
		ClientDavInterface daVVerbindung = _dav.getDaVVerbindung();
		if (daVVerbindung != null)
		{
			if (_ebsn == null)
			{
				_ebsn = new EinstellungenBenutzerSpezifischNetzweit(
						(ClientDavConnection) daVVerbindung);
			} else
			{
				_ebsn.setDaVVerbindung((ClientDavConnection) daVVerbindung);
			}

			List<SystemObject> bestimmeAlleBenutzerSO = _ebsn
					.bestimmeAlleBenutzerSO();

			if ((bestimmeAlleBenutzerSO != null)
					&& (bestimmeAlleBenutzerSO.size() > 0))
			{
				LinkedList<String> benutzer = new LinkedList<String>();
				Iterator<SystemObject> it = bestimmeAlleBenutzerSO.iterator();
				while (it.hasNext())
				{
					SystemObject systemObject = it.next();
					benutzer.add(systemObject.getName());
				}

				_tvBenutzer.setInput(benutzer);
			}
		}
	}

	/**
	 * This method initializes sShell
	 */
	private void erzeugeShell()
	{
		_shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		_shell.setText("Einstellungenwhler");
		erzeugeCompositeOben();
		erzeugeCompositeUnten();
		_shell.setSize(new Point(600, 330));
		_shell.setLayout(new GridLayout());
	}

	/**
	 * This method initializes compositeOben
	 * 
	 */
	private void erzeugeCompositeOben()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_compositeOben = new Composite(_shell, SWT.NONE);
		erzeugeGroupBenutzer();
		_compositeOben.setLayout(gridLayout);
		erzeugeGroupParameter();
		_compositeOben.setLayoutData(gridData);
	}

	/**
	 * This method initializes compositeUnten
	 * 
	 */
	private void erzeugeCompositeUnten()
	{
		GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.verticalAlignment = GridData.CENTER;
		_compositeUnten = new Composite(_shell, SWT.NONE);
		_compositeUnten.setLayout(new GridLayout());
		erzeugeCompositeUntenRechts();
		_compositeUnten.setLayoutData(gridDataUnten);
	}

	/**
	 * Schliet den Dialog
	 */
	private void schliesseDialog()
	{
		_shell.dispose();
	}

	/**
	 * Schliet den Dialog, der selektierte Parameter wird zurckgeliefert.
	 */
	private void aktionOk()
	{
		schliesseDialog();
	}

	/**
	 * Schliet den Dialog, er wird kein Parameter zurckgeliefert.
	 */
	private void aktionSchliessen()
	{
		_selektierterParameter = null;
		schliesseDialog();
	}

	/**
	 * This method initializes compositeUntenRechts
	 * 
	 */
	private void erzeugeCompositeUntenRechts()
	{
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		rowLayout.justify = false;
		GridData gridDataUntenRechts = new GridData();
		gridDataUntenRechts.horizontalAlignment = GridData.END;
		gridDataUntenRechts.grabExcessHorizontalSpace = true;
		gridDataUntenRechts.verticalAlignment = GridData.CENTER;
		_compositeUntenRechts = new Composite(_compositeUnten, SWT.NONE);
		_compositeUntenRechts.setLayoutData(gridDataUntenRechts);
		_compositeUntenRechts.setLayout(rowLayout);
		_schaltflaecheOk = new Button(_compositeUntenRechts, SWT.NONE);
		_schaltflaecheOk.setText("OK");
		_schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}
		});

		_schaltflaecheAbbrechen = new Button(_compositeUntenRechts, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}
		});
	}

	/**
	 * This method initializes groupBenutzer
	 * 
	 */
	private void erzeugeGroupBenutzer()
	{
		GridData gridDataTVBenutzer = new GridData();
		gridDataTVBenutzer.horizontalAlignment = GridData.FILL;
		gridDataTVBenutzer.grabExcessHorizontalSpace = true;
		gridDataTVBenutzer.grabExcessVerticalSpace = true;
		gridDataTVBenutzer.verticalAlignment = GridData.FILL;
		GridData gridDataBenutzer = new GridData();
		gridDataBenutzer.horizontalAlignment = GridData.FILL;
		gridDataBenutzer.grabExcessHorizontalSpace = true;
		gridDataBenutzer.grabExcessVerticalSpace = true;
		gridDataBenutzer.verticalAlignment = GridData.FILL;
		_gruppeBenutzer = new Group(_compositeOben, SWT.NONE);
		_gruppeBenutzer.setLayout(new GridLayout());
		_gruppeBenutzer.setLayoutData(gridDataBenutzer);
		_gruppeBenutzer.setText("Benutzer:");

		_tvBenutzer = new TableViewer(_gruppeBenutzer, SWT.FULL_SELECTION
				| SWT.MULTI);
		_tvBenutzer.getTable().setLayoutData(gridDataTVBenutzer);

		TableColumn tcBenutzer = new TableColumn(_tvBenutzer.getTable(), SWT.LEFT);
		tcBenutzer.setText("Benutzer:");
		tcBenutzer.setWidth(150);

		_tvBenutzer.getTable().setHeaderVisible(true);
		_tvBenutzer.getTable().setLinesVisible(true);

		_tvBenutzer.setContentProvider(new BenutzerContentProvider());
		_tvBenutzer.setLabelProvider(new BenutzerLabelProvider());

		_tvBenutzer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Object firstElement = selection.getFirstElement();

				if (firstElement instanceof String)
				{
					aktionBenutzerSelektiert((String) firstElement);
				}
			}
		});

		// LinkedList<String> inhalt = new LinkedList<String>();
		// for (int i = 0; i < 100; i++)
		// {
		// inhalt.add("Benutzer #" + i);
		// }
		//
		// _tvBenutzer.setInput(inhalt);

	}

	/**
	 * This method initializes groupParameter
	 * 
	 */
	private void erzeugeGroupParameter()
	{
		GridData gridDataTVGruppe = new GridData();
		gridDataTVGruppe.horizontalAlignment = GridData.FILL;
		gridDataTVGruppe.grabExcessHorizontalSpace = true;
		gridDataTVGruppe.grabExcessVerticalSpace = true;
		gridDataTVGruppe.verticalAlignment = GridData.FILL;
		GridData gridDataParameter = new GridData();
		gridDataParameter.horizontalAlignment = GridData.FILL;
		gridDataParameter.grabExcessHorizontalSpace = true;
		gridDataParameter.grabExcessVerticalSpace = true;
		gridDataParameter.verticalAlignment = GridData.FILL;
		_gruppeParameter = new Group(_compositeOben, SWT.NONE);
		_gruppeParameter.setLayout(new GridLayout());
		_gruppeParameter.setLayoutData(gridDataParameter);
		_gruppeParameter.setText("Parameter:");

		_tvParameter = new TableViewer(_gruppeParameter, SWT.FULL_SELECTION
				| SWT.MULTI);
		_tvParameter.getTable().setLayoutData(gridDataTVGruppe);

		TableColumn tc1 = new TableColumn(_tvParameter.getTable(), SWT.LEFT);
		tc1.setText("Parameter:");
		tc1.setWidth(100);

		TableColumn tc2 = new TableColumn(_tvParameter.getTable(), SWT.LEFT);
		tc2.setText("Wert:");
		tc2.setWidth(200);

		_tvParameter.getTable().setHeaderVisible(true);
		_tvParameter.getTable().setLinesVisible(true);

		_tvParameter.setContentProvider(new ParameterContentProvider());
		_tvParameter.setLabelProvider(new ParameterLabelProvider());

		_tvParameter.addSelectionChangedListener(new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Object firstElement = selection.getFirstElement();
				if (firstElement instanceof String[])
				{
					String[] wert = (String[]) firstElement;
					aktionParameterSelektiert(wert);
				}

			}

		});

		// LinkedList<String[]> liste = new LinkedList<String[]>();
		// for (int i = 0; i < 50; i++)
		// {
		// String[] wert = new String[] { "Parameter " + i, "Wert " + i };
		// liste.add(wert);
		// }
		//
		// _tvGruppe.setInput(liste);
		// System.out.println("LISTE: " + liste.toString());
	}

	/**
	 * Aktion die ausgefhrt wird, wenn ein Benutzer selektiert wird.
	 */
	private void aktionBenutzerSelektiert(String benutzerName)
	{
		_debug.fine("aktionBenutzerSelektiert:: " + benutzerName);

		if (_ebsn != null)
		{
			String xmlStruktur = _ebsn.getEintrag(benutzerName);
			_debug.fine("XML:\n " + xmlStruktur);

			EinstellungenSpeicher es = new EinstellungenSpeicher();
			es.einlesenXmlAusString(xmlStruktur);

			HashMap<String, String> hmParams = es.getAlleParameter();
			if (hmParams.size() > 0)
			{
				LinkedList<String[]> parameter = new LinkedList<String[]>();
				Iterator<Entry<String, String>> iterator = hmParams.entrySet()
						.iterator();
				while (iterator.hasNext())
				{
					Entry<String, String> entry = iterator.next();
					String[] wert = new String[] { entry.getKey(), entry.getValue() };
					parameter.add(wert);
				}

				_tvParameter.setInput(parameter);
			}
		}

	}

	/**
	 * Aktion die ausgefhrt wird, wenn ein Parameter selektiert wird.
	 */
	private void aktionParameterSelektiert(String[] wert)
	{
		if (wert.length == 2)
		{
			_debug.fine("Parameter: " + wert[0] + " Wert: " + wert[1]);
			this._selektierterParameter = wert;
		}
	}

	/**
	 * ContentProvider zur Darstellung der Benutzer, bernimmt als Parameter eine
	 * Liste von Strings.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class BenutzerContentProvider implements IStructuredContentProvider
	{

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{

			Object[] re = null;
			if (inputElement instanceof LinkedList)
			{
				LinkedList liste = (LinkedList) inputElement;
				if (liste.size() > 0)
				{

					re = new Object[liste.size()];

					int i = 0;

					Iterator it = liste.iterator();
					while (it.hasNext())
					{
						Object object = it.next();
						String einfuegen = "leer";

						if (object instanceof String)
						{
							einfuegen = (String) object;
						}

						re[i] = einfuegen;

						i++;
					}
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	private class BenutzerLabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "Nicht bekannter Datentyp";

			if (element instanceof String)
			{
				re = (String) element;
				_debug.fine("HIER");
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Als Eingangswert wird eine Liste von Arrays (vom Typ String der Lnge 2)
	 * erwartet.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ParameterContentProvider implements IStructuredContentProvider
	{

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement != null)
			{
				if (inputElement instanceof LinkedList)
				{
					LinkedList liste = (LinkedList) inputElement;

					if (liste.size() > 0)
					{
						re = new Object[liste.size()];

						Iterator it = liste.iterator();
						int i = 0;
						while (it.hasNext())
						{
							Object object = it.next();
							if (object instanceof String[])
							{
								re[i] = object;
							}
							i++;
						}
					}

				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	/**
	 * {@link LabelProvider} der {@link TableViewer}s _tvGruppe.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ParameterLabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "Objekt nicht definiert!";

			if (element instanceof String[])
			{
				String[] inhalt = (String[]) element;

				if ((inhalt != null) && (inhalt.length == 2))
				{
					switch (columnIndex)
					{
					case 0:
						re = inhalt[0];
						break;
					case 1:
						re = inhalt[1];
						break;
					}
				}
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

}
