/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class EinstellungenDaVSchreibenTest implements ClientSenderInterface
{
  /**
   * Verbindung zum DaV
   */
  private ClientDavConnection _verbindung = null;

  /**
   * IP
   */
  private static final String _ip = "localhost";

  /**
   * Port
   */
  private static final int _port = 8083;

  /**
   * Login
   */
  private static final String _benutzer = "Tester";

  /**
   * Passwort
   */
  private static final String _passwort = "geheim";

  /**
   * Datenmodell des DaV
   */
  private DataModel _dataModell = null;

  /**
   * SystemObjekt
   */
  private SystemObject _systemObject = null;

  /**
   * Bezeichnung der Attributgruppe
   */
  private static final String _atgBezeichnung = "atg.globaleEinstellungenOberflcheNetzweit";

  /**
   * Attributgruppe
   */
  private AttributeGroup _atg = null;

  /**
   * Aspekt
   */
  private Aspect _asp = null;

  /**
   * Datenbeschreibung
   */
  private DataDescription _dataDescription = null;

  /**
   * @param args
   */
  public static void main(String[] args)
  {
    @SuppressWarnings("unused")
		EinstellungenDaVSchreibenTest te = new EinstellungenDaVSchreibenTest();
  }

  public EinstellungenDaVSchreibenTest()
  {
    baueDaVVerbindungAuf();
    anmeldenSenden();

    EinstellungenSpeicher es = new EinstellungenSpeicher();

    String w = "";
    for (int j = 0; j < 160; j++)
    {
      w = w + "#";
    }

    for (int i = 1; i < 10; i++)
    {
      // System.out.println("I: " + i);
      es.hinzufuegenParameter("ID." + i, "Eintrag #" + i
          + "=>Hier sollte schon eine ganze Menge von Zeichen reinpassen: " + w);
    }

    System.out.println("Vor Formatierter String erzeugung");
    String xmlString = es.ausgabeXmlFormatiert(true);
    System.out.println("Nach Formatierter String erzeugung");
    es.schreibeString("C:\\1.xml", xmlString, true);

    String xml = xmlString;

    System.out.println("Anzahl Zeichen Wert: " + xml.length());

    System.out.println("Vor Split");

    String[] zeilen = xml.split("\n");

    System.out.println("Nach Split");

    System.out.println("Anzahl Zeilen: " + zeilen.length);

    sende(zeilen);

    abmeldenSenden();
    schliesseDaVVerbindung();
  }

  /**
   * Anmelden zum Senden von Daten
   */
  public void anmeldenSenden()
  {
    _dataModell = _verbindung.getDataModel();

    List<SystemObject> objekte = bestimmeObjekte("typ.autarkeOrganisationsEinheit");
    if (objekte.size() == 1)
    {
      _systemObject = objekte.get(0);
      _atg = _dataModell.getAttributeGroup(_atgBezeichnung);

      _asp = _dataModell.getAspect("asp.parameterVorgabe");
      _dataDescription = new DataDescription(_atg, _asp);

      SenderRole senderRole = SenderRole.sender();

      try
      {
        _verbindung.subscribeSender(this, _systemObject, _dataDescription, senderRole);
      }
      catch (OneSubscriptionPerSendData e)
      {
        e.printStackTrace();
      }

    }

  }

  /**
   * Sendet die Werte aus einem String-Array.
   * 
   * @param wert
   *          String-Array mit den Werten als String.
   */
  public void sende(String[] wert)
  {
    _atg = _dataModell.getAttributeGroup(_atgBezeichnung);
    Data data = _verbindung.createData(_atg);
    Data item = data.getItem("BenutzereinstellungenGlobal");
    TextArray textArray = item.getTextArray("Voreinstellungen");
    textArray.setLength(wert.length);

    for (int i = 0; i < wert.length; i++)
    {
      textArray.getTextValue(i).setText(wert[i]);
    }

    ResultData datenTelegramm = new ResultData(_systemObject, _dataDescription, new Date().getTime(), data);

    try
    {
      _verbindung.sendData(datenTelegramm);
    }
    catch (DataNotSubscribedException e)
    {
      System.err.println(getClass().getName() + "::");
      e.printStackTrace();
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Abmeldung
   */
  public void abmeldenSenden()
  {
    _verbindung.unsubscribeSender(this, _systemObject, _dataDescription);

  }

  /**
   * Baut die Verbindung zum DaV auf.
   */
  private void baueDaVVerbindungAuf()
  {
    try
    {
      _verbindung = new ClientDavConnection();

      // IP setzen
      _verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

      // Port setzen
      _verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

      // Pfad zum Zwischenspeichern setzen
      _verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

      try
      {
        _verbindung.connect();
        System.out.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

        try
        {
          _verbindung.login(_benutzer, _passwort);
          System.out.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");
        }
        catch (InconsistentLoginException e)
        {
          // e.printStackTrace();
          System.err.println("Verbindungsaufbau Fehler: Konnte Benutzer nicht einloggen.");
          _verbindung = null;
        }

      }
      catch (CommunicationError e)
      {
        // e.printStackTrace();
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }
      catch (ConnectionException e)
      {
        // e.printStackTrace();
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }

    }
    catch (MissingParameterException e)
    {
      // e.printStackTrace();
      System.err.println("Verbindungsaufbau Fehler: " + e.toString());
      _verbindung = null;
    }
  }

  /**
   * Schliet die Verbindung zum DaV
   */
  private void schliesseDaVVerbindung()
  {
    if (_verbindung != null)
    {
      _verbindung.disconnect(false, "Kein Fehler.");
      System.out.println("\nVerbindungsabbau: Verbindung wurde abgebaut.");
    }
  }

  /**
   * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren (z.B. typ.de)
   * 
   * @param objPid
   *          Pid der Objekttypen
   * @return Liste mit den Systemobjekten
   */
  private List<SystemObject> bestimmeObjekte(String objPid)
  {
    System.out.println("bestimmeObjekte --> " + objPid);

    List<SystemObject> objekte = new ArrayList<SystemObject>();

    // Systemobjekt erzeugen

    SystemObjectType typeSysObj = null;

    typeSysObj = _verbindung.getDataModel().getType(objPid);

    if (typeSysObj == null)
      return objekte;

    if (typeSysObj.isConfigurating())
    {
      typeSysObj = (ConfigurationObjectType)typeSysObj;
      System.out.println("isConfigurating");
    }
    else
    {
      typeSysObj = (DynamicObjectType)typeSysObj;
      System.out.println("Dynamic");
    }

    // Elemente dieses Systemobjekts einlesen

    List<SystemObject> typobjekte = typeSysObj.getElements();

    Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

    while (iterator_Typen.hasNext())
    {
      SystemObject sysObj = (SystemObject)iterator_Typen.next();

      objekte.add(sysObj);

      // System.out.println(sysObj.getPid());
    }

    return objekte;
  }

  @Override
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
  }

  @Override
  public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
  {
    return false;
  }

  /**
   * Liest den Inhalt einer Textdatei ein und liefert diese als String zurck.
   * 
   * @param dateiPfad
   *          Pfad zur Datei.
   * @return Inhalt der Datei als String.
   */
  public String dateiEinlesen(String dateiPfad)
  {
    String re = null;

    File file = new File(dateiPfad);
    if ((file.exists()) && (file.isFile()))
    {

      StringBuffer buf = new StringBuffer();
      try
      {

        FileReader fr = new FileReader(dateiPfad);

        try
        {
          for (int c; (c = fr.read()) != -1;)
          {
            buf.append((char)c);

          }

          re = buf.toString();
        }
        catch (IOException e)
        {
          e.printStackTrace();
        }

      }
      catch (FileNotFoundException e)
      {
        e.printStackTrace();
      }
    }

    return re;
  }
}
