/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.ereignisSystemKalender;

import java.util.LinkedList;

/**
 * Kapsel fr Ereigniskalendereintrge
 * 
 * @author HofmannJ
 * 
 */
public class ErkaKapsel implements IEintragKapsel
{
	/**
	 * Arbeiter zur Bearbeitung von Anfragen an den Systemkalender
	 */
	private SysKalAnfrageArbeiter _sysKalAnfrageArbeiter = null;

	/**
	 * Verweis auf einen {@link EintragEreigniskalender}
	 */
	private EintragEreigniskalender _eintragEreigniskalender = null;

	/**
	 * Speichert die Liste der {@link IEintragBereich}e.
	 */
	private LinkedList<IEintragBereich> _bereiche = new LinkedList<IEintragBereich>();

	/**
	 * Konstruktor. bernimmt einen {@link EintragEreigniskalender}.
	 * 
	 * @param eintrag
	 *          {@link EintragEreigniskalender}
	 */
	public ErkaKapsel(EintragEreigniskalender eintrag,
			SysKalAnfrageArbeiter sysKalAnfrageArbeiter, Long startAuswahlDialog, Long endeAuswahlDialog)
	{
		this._eintragEreigniskalender = eintrag;
		this._sysKalAnfrageArbeiter = sysKalAnfrageArbeiter;

		Long anfang = _eintragEreigniskalender.getAnfangsZeitpunkt();
		Long ende = _eintragEreigniskalender.getEndZeitpunkt();

		if (!anfang.equals(ende))
		{
			ErkaBereich bereich = new ErkaBereich(anfang, ende);
			_bereiche.add(bereich);
		} else
		{
			if (_eintragEreigniskalender.getSystemKalenderEintrag() != null)
			{
//				IEintragKapsel kapsel = _sysKalAnfrageArbeiter.berechneGueltigVonBisS(
//						_eintragEreigniskalender.getSystemKalenderEintrag().getPid(),
//						anfang, ende);
				
				IEintragKapsel kapsel = _sysKalAnfrageArbeiter.berechneGueltigVonBisS(
						_eintragEreigniskalender.getSystemKalenderEintrag().getPid(),
						startAuswahlDialog, endeAuswahlDialog);
							
				_bereiche.addAll(kapsel.getBereiche());			
			} else
			{
				ErkaBereich bereich = new ErkaBereich(anfang, ende);
				_bereiche.add(bereich);
			}
		}

	}

	@Override
	public void addBereich(Long von, Long bis)
	{
	}

	@Override
	public void addBereich(IEintragBereich bereich)
	{
		this._bereiche.add(bereich);
	}

	@Override
	public int compareTo(IEintragKapsel o)
	{
		return this.getPid().compareTo(o.getPid());
	}

	@Override
	public LinkedList<IEintragBereich> getBereiche()
	{
		return _bereiche;
	}

	@Override
	public int getBereicheAnzahl()
	{
		int re = 0;

		if (_bereiche != null)
		{
			re = _bereiche.size();
		}

		return re;
	}

	@Override
	public String getPid()
	{
		String re = null;
		if (_eintragEreigniskalender != null)
		{
			re = _eintragEreigniskalender.getPid();
		}
		return re;
	}

	public String toString()
	{
		String re = _eintragEreigniskalender.getName() + " (Pid: "
				+ _eintragEreigniskalender.getPid() + ")";
		return re;
	}

	@Override
	public String toStringLong()
	{
		return null;
	}

}
