/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kontextmenu;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert eine Beispiel-View ({@link ViewPart}) die zeigt, wie mit Hilfe der Klasse {@link KontextMenu}
 * ein Kontextmen mit Selektionsmechanismus fr die vorhandenen Eintrge realisiert wird. <br>
 * <br>
 * Es wird eine Tabelle dargestellt, an die ein Kontextmen angebunden wird. Ein Doppelklick auf eine Zelle lst die im
 * Kontextmen selektierte {@link Action} aus, es erscheint eine {@link MessageBox} die sich auf Zelleninhalt und
 * selektierte {@link Action} bezieht.<br>
 * <br>
 * Ist <b><i>keine</i></b> {@link Action} selektiert, erscheint eine {@link MessageBox} mit einem entsprechenden
 * Warnhinweis.
 * 
 * @author HofmannJ
 * 
 */
public class ViewMitKontextMenu extends ViewPart
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * ID des {@link ViewPart}
   */
  public static final String ID = "de.bsvrz.buv.rw.basislib.kontextmenu.ViewMitKontextMenu";

  /**
   * Basis Composite
   */
  private Composite top = null;

  /**
   * Gruppe Oben
   */
  private Group groupOben = null;

  /**
   * Unteres Composite
   */
  private Composite compositeUnten = null;

  /**
   * Composite unten rechts
   */
  private Composite compositeUntenRechts = null;

  /**
   * Button OK
   */
  private Button buttonOk = null;

  /**
   * Button Abbruch
   */
  private Button buttonAbbruch = null;

  /**
   * Tabelle
   */
  private Table tableTest = null;

  /**
   * {@link TableViewer} zur Tabelle
   */
  private TableViewer tableViewer = null;

  @Override
  public void createPartControl(Composite parent)
  {
    top = new Composite(parent, SWT.NONE);
    top.setLayout(new GridLayout());
    createGroupOben();
    createCompositeUnten();
  }

  @Override
  public void setFocus()
  {
    _debug.fine(getClass().getName() + " hat jetzt den Focus. Aber wem nutzt das schon?");
  }

  /**
   * Erzeugt die obere Gruppe mit einer {@link Table} und einem angeschlossenen {@link TableViewer}.<br>
   * <br>
   * Auf die Tabelle wird ein Kontextmen mit drei {@link Action}s angemeldet.
   * 
   */
  private void createGroupOben()
  {
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    groupOben = new Group(top, SWT.NONE);
    groupOben.setLayout(new GridLayout());
    groupOben.setLayoutData(gridData);
    groupOben.setText("bersicht:");
    tableTest = new Table(groupOben, SWT.FULL_SELECTION);
    tableTest.setLayoutData(gridDataTabelle);

    /*
     * Den TableViewer erzeugen
     */
    tableViewer = new TableViewer(tableTest);

    /*
     * Erste Spalte der Tabelle
     */
    TableColumn tc1 = new TableColumn(tableViewer.getTable(), SWT.CENTER);
    tc1.setText("Spalte 1:");
    tc1.setWidth(200);

    /*
     * Zweite Spalte der Tabelle
     */
    TableColumn tc2 = new TableColumn(tableViewer.getTable(), SWT.LEFT);
    tc2.setText("Spalte 2:");
    tc2.setWidth(300);

    /*
     * Kopfzeilen und Linien darstellen
     */
    tableViewer.getTable().setHeaderVisible(true);
    tableViewer.getTable().setLinesVisible(true);

    /*
     * ContentProvider und LabelProvider setzen
     */
    tableViewer.setContentProvider(new ContentProvider());
    tableViewer.setLabelProvider(new LabelProvider());

    /*
     * Eintrge fr die Tabelle erzeugen
     */
    LinkedList<String> l = new LinkedList<String>();
    l.add("Hallo");
    l.add("Welt!");
    l.add("Testeintrag");

    /*
     * Die Eintrge dem TableViewer als Input bergeben
     */
    tableViewer.setInput(l);

    /*
     * Erzeugen des Kontextmens
     */
    final KontextMenu kontextMenu = new KontextMenu();

    /*
     * Erzeugen und Registrieren der ersten Action fr das Kontextmen
     */
    Action aktionEins = new Action() {
      public void run()
      {
        _debug.fine("=> Erste Aktion ist da!");
        
      }
    };
    aktionEins.setText("Erste Aktion");
    aktionEins.setImageDescriptor(Activator.getImageDescriptor("icons/actions/cancel16x16.png"));
    kontextMenu.hinzufuegenAktion(aktionEins);

    /*
     * Erzeugen und Registrieren der ersten Action fr das Kontextmen
     */
    Action aktionZwei = new Action() {
      public void run()
      {
        _debug.fine("=> Zweite Aktion ist da!");
      }
    };
    aktionZwei.setText("Zweite Aktion");
    aktionZwei.setImageDescriptor(Activator.getImageDescriptor("icons/actions/editdelete16x16.png"));
    kontextMenu.hinzufuegenAktion(aktionZwei);

    /*
     * Erzeugen und Registrieren der ersten Action fr das Kontextmen
     */
    Action aktionDrei = new Action() {
      public void run()
      {
        _debug.fine("=> Dritte Aktion ist da!");
      }
    };
    aktionDrei.setText("Dritte Aktion");
    aktionDrei.setImageDescriptor(Activator.getImageDescriptor("icons/menubar/laden_16x16.png"));
    kontextMenu.hinzufuegenAktion(aktionDrei);

    /*
     * Registrierung des von der Klasse Kontextmen gelieferten MenuManagers auf dem TableVier unter der aktuellen
     * Seite.
     */
    MenuManager mm = kontextMenu.getKontextMenu(tableViewer.getControl());
    this.getSite().registerContextMenu(mm, tableViewer);

    /*
     * Hinzufgen eines SelektionListeners zur Tabelle, der auf Doppelklicks reagiert. Es wird das selektierte Objekt
     * der entsprechenden Zelle als String zurckgeliefert und die selektierte Action des KontextMenu ausgefhrt. Ist im
     * KontextMenu keine Action selektiert, erscheint ein entsprechender Hinweis.
     */
    tableTest.addSelectionListener(new SelectionAdapter() {
      public void widgetDefaultSelected(SelectionEvent e)
      {

        String text = "";

        IStructuredSelection is = (IStructuredSelection)tableViewer.getSelection();
        Object erstesElement = is.getFirstElement();

        if (erstesElement instanceof String)
        {
          text = (String)erstesElement;
        }

        if (kontextMenu.selektierteAktionAusfuehren())
        {
          MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.APPLICATION_MODAL
              | SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Information:");
          mb.setMessage("Es wurde die Aktion \"" + kontextMenu.getSelektierteAktion().getText()
              + "\" selektiert und per Doppelklick auf dem Eintrag \"" + text + "\" ausgefhrt.");

          mb.open();

        }
        else
        {
          MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.APPLICATION_MODAL
              | SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler:");
          mb.setMessage("Es ist zur Zeit keie Aktion selektiert, " + "die per Doppelklick ausgefhrt werden kann!");
          mb.open();

        }

      }
    });

  }

  /**
   * Erzeugt den unteren Bereich.
   * 
   */
  private void createCompositeUnten()
  {
    GridData gridDataUnten = new GridData();
    gridDataUnten.horizontalAlignment = GridData.FILL;
    gridDataUnten.grabExcessHorizontalSpace = true;
    gridDataUnten.verticalAlignment = GridData.CENTER;
    compositeUnten = new Composite(top, SWT.NONE);
    compositeUnten.setLayout(new GridLayout());
    createCompositeUntenRechts();
    compositeUnten.setLayoutData(gridDataUnten);
  }

  /**
   * Erzeugt den unteren, rechten Bereich.
   * 
   */
  private void createCompositeUntenRechts()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridDataUntenRechts = new GridData();
    gridDataUntenRechts.grabExcessHorizontalSpace = true;
    gridDataUntenRechts.verticalAlignment = GridData.CENTER;
    gridDataUntenRechts.horizontalAlignment = GridData.END;
    compositeUntenRechts = new Composite(compositeUnten, SWT.NONE);
    compositeUntenRechts.setLayoutData(gridDataUntenRechts);
    compositeUntenRechts.setLayout(rowLayout);

    buttonOk = new Button(compositeUntenRechts, SWT.NONE);
    buttonOk.setText("OK");
    buttonOk.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine(buttonOk.getText());
			}			
		});

    buttonAbbruch = new Button(compositeUntenRechts, SWT.NONE);
    buttonAbbruch.setText("Abbruch");
    buttonAbbruch.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_debug.fine(buttonAbbruch.getText());
			}			
		});
  }

  /**
   * Liefert einen {@link IStructuredContentProvider} fr den {@link TableViewer} der Tabelle.
   * 
   * @author HofmannJ
   * 
   */
  private class ContentProvider implements IStructuredContentProvider
  {
    @SuppressWarnings("unchecked")
	@Override
    public Object[] getElements(Object inputElement)
    {
      Object[] re = null;

      if (inputElement instanceof LinkedList)
      {
        LinkedList<String> liste = (LinkedList<String>)inputElement;
        re = new Object[liste.size()];

        Iterator<String> it = liste.iterator();
        int i = 0;
        while (it.hasNext())
        {
          String eintrag = it.next();
          re[i] = eintrag;
          i++;
        }

      }

      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  /**
   * Liefert den {@link ITableLabelProvider} fr den {@link TableViewer} der Tabelle.
   * 
   * @author HofmannJ
   * 
   */
  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String re = element.toString();
      return re;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

}
