/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.legende;

import java.util.Date;

/**
 * Diese Klasse kennt den speichert Zeitpunkt, der in der Legende anzuzeigen ist. ber einen Listener wird die Legende
 * von der anzuzeigenden Zeit informiert, mit Methoden kann die Zeit auf den aktuellen Zeitpunkt oder einen bestimmten
 * Zeitpunkt eingestellt werden.
 * 
 * @author HofmannJ
 * 
 */
public class LegendeDarzustellendeZeit
{
  /**
   * Zeitpunkt
   */
  private long _zeitPunkt = -1;

  /**
   * ILegendeZeitpunkt Listener
   */
  private ILegendenZeitpunkt _zeitpunktListener = null;

  /**
   * Privater Konstruktor
   */
  private LegendeDarzustellendeZeit()
  {
  }

  /**
   * Speichert die Singleton-Instanz der Klasse {@link LegendeDarzustellendeZeit}.
   * 
   * @author HofmannJ
   * 
   */
  private static class Innen
  {
    private static LegendeDarzustellendeZeit INSTANZ = new LegendeDarzustellendeZeit();
  }

  /**
   * Liefert die Singleton-Instanz der Klasse {@link LegendeDarzustellendeZeit}.
   * 
   * @return Singleton-Instanz der Klasse {@link LegendeDarzustellendeZeit}
   */
  public static LegendeDarzustellendeZeit getInstanz()
  {
    return Innen.INSTANZ;
  }

  /**
   * Setzt die aktuelle Zeit als Zeit, die in der Legende angezeigt werden soll.
   */
  public void setZeitpunktJetzt()
  {
    _zeitPunkt = new Date().getTime();
    benachrichtigeZeitPunktListener();
  }

  /**
   * Setzt Zeitpunkt, der i.d. Legende als Zeit angezeigt werden soll.
   * 
   * @param zeitpunkt
   *          Zeit in Millisekunden seit dem 1.1.1970, 0 Uhr.
   */
  public void setZeitpunkt(long zeitpunkt)
  {
    _zeitPunkt = zeitpunkt;
    benachrichtigeZeitPunktListener();
  }

  /**
   * Liefert den Zeitpunkt, der in der Legende angezeigt werden soll.
   * 
   * @return Zeit in Millisekunden seit dem 1.1.1970, 0 Uhr.
   */
  public long getZeitpunkt()
  {
    return _zeitPunkt;

  }

  /**
   * Fgt einen Zeitpunktlistener(Typ: {@link ILegendenZeitpunkt}) hinzu.
   * 
   * @param zeitpunktListener
   *          Hinzuzufgender {@link ILegendenZeitpunkt} Listener.
   */
  public void addZeitpunktListener(ILegendenZeitpunkt zeitpunktListener)
  {
    this._zeitpunktListener = zeitpunktListener;
    if (_zeitpunktListener != null)
    {
      benachrichtigeZeitPunktListener();
    }
  }

  /**
   * Entfernt einen ZeitpunktListener (Typ: {@link ILegendenZeitpunkt}).
   * 
   * @param zeitpunktListener
   *          Zu entfernender {@link ILegendenZeitpunkt} Listener.
   */
  public void removeZeitpunktListener(ILegendenAbschnitt zeitpunktListener)
  {
    if (_zeitpunktListener.equals(zeitpunktListener))
    {
      _zeitpunktListener = null;
    }
  }

  /**
   * Benachrichtigt den Zeitpunktlistener
   */
  private void benachrichtigeZeitPunktListener()
  {
    if (_zeitpunktListener != null)
    {
      _zeitpunktListener.getZeit(_zeitPunkt);
    }
  }

}
