/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IMarkSelection;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Speichert die Daten der Mehrfachzwischenablage.
 * 
 * @author HofmannJ
 * 
 */
public class MehrfachZwischenablageDaten
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();


	/**
	 * Speichert die Eintrge der Mehrfachzwischenablage
	 */
	private static List<ZwischenablageDatensatz> _eintraege = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Liste der Zuhrer.
	 */
	private static List<IMehrfachZwischenablageDatenZuhoerer> _zuhoerer = new LinkedList<IMehrfachZwischenablageDatenZuhoerer>();

	/**
	 * Speichert die letzte Selektion in einem externen Viewer/Editor
	 */
	// private static Object _selektionsSpeicherExtern = null;
	private static List<ZwischenablageDatensatz> _selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Speichert die aktuelle Selektion der Mehrfachzwischenablage.
	 */
	private static List<ZwischenablageDatensatz> _selektionsSpeicherMehrfachZwischenablage = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Liste der Ausschneidezuhrer
	 */
	private static List<IMehrfachZwischenablageAusschneiden> _zuhoererAusschneiden = new LinkedList<IMehrfachZwischenablageAusschneiden>();

	/**
	 * Merkt sich den {@link IWorkbenchPart} der letzten Selektion. Damit kann
	 * z.B. das lschen initialisiert werden!
	 */
	@SuppressWarnings("unused")
	private static IWorkbenchPart _teilLetzterSelektion = null;

	private static class Innen
	{
		private static MehrfachZwischenablageDaten _instanz = new MehrfachZwischenablageDaten();
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static MehrfachZwischenablageDaten getInstanz()
	{
		// if (_instanz == null)
		// {
		// _instanz = new MehrfachZwischenablageDaten();
		// }
		// return _instanz;

		return Innen._instanz;
	}

	/**
	 * Fgt einen {@link Object} am Anfang der Liste ein.
	 * 
	 * @param datensatz
	 *          Einzufgendes {@link Object}.
	 */
	protected void hinzufuegenEintrag(ZwischenablageDatensatz datensatz)
	{
		if (datensatz != null)
		{
			synchronized (_eintraege)
			{

				if (!pruefeObObjektInListe(datensatz, _eintraege))
				{
					_eintraege.add(0, datensatz);
					benachrichtigeZuhoerer();
					_debug.fine(getClass().getName()
							+ "::hinzufuegenEintrag \n\tEs wurde ein Eintrag hinzugefgt.\n");
				} else
				{
					_debug
							.error(getClass().getName()
									+ "::hinzufuegenEintraege \n\t=>Das Objekt ist bereits in der Liste vorhanden und wird nicht erneut hinzugefgt.\t");
				}
			}

		} else
		{
			_debug
					.error(getClass().getName()
							+ "::hinzufuegenEintrag \n\t=>Es kann kein NULL Objekt akzeptiert werden!\n");
		}

	}

	/**
	 * Fgt eine Liste vom Typ {@link Object} am Anfang der Liste ein
	 * 
	 * @param datenSatz
	 *          Liste der einzufgenden {@link Object}e.
	 */
	protected void hinzufuegenEintraege(List<ZwischenablageDatensatz> datenSatz)
	{
		int counter = 0;
		Iterator<ZwischenablageDatensatz> it = datenSatz.iterator();
		List<ZwischenablageDatensatz> tempObjekte = new LinkedList<ZwischenablageDatensatz>();
		while (it.hasNext())
		{
			ZwischenablageDatensatz object = it.next();
			if (object != null)
			{
				synchronized (_eintraege)
				{
					if (!pruefeObObjektInListe(object, _eintraege))
					{
						tempObjekte.add(object);
						// _eintraege.add(object);
						counter++;
					} else
					{
						_debug
								.error(getClass().getName()
										+ "::hinzufuegenEintraege \n\t=>Das Objekt ist bereits in der Liste vorhanden und wird nicht erneut hinzugefgt.\t");
					}
				}

			} else
			{
				_debug
						.error(getClass().getName()
								+ "::hinzufuegenEintraege \n\t=>Es kann kein NULL Objekt akzeptiert werden!\n");
			}
		}

		synchronized (_eintraege)
		{
			_eintraege.addAll(0, tempObjekte);
		}

		benachrichtigeZuhoerer();

		_debug.fine(getClass().getName() + "::hinzufuegenEintrag \n\tEs wurden "
				+ counter + " Eintrge hinzugefgt.\n");

	}

	/**
	 * Prft, ob ein Objekt in einer Liste von Objeken schon vorhanden ist.
	 * 
	 * @param objekt
	 *          Das Objekt, auf dessen vorgkommen geprft werden soll.
	 * @param liste
	 *          Die zu prfende Liste.
	 * @return Liefert <code>true</code>, wenn das Objekt schon in der Liste
	 *         vorkommer, sonst <code>false</code>.
	 */
	private boolean pruefeObObjektInListe(ZwischenablageDatensatz objekt,
			List<ZwischenablageDatensatz> liste)
	{
		boolean returnWert = false;
		synchronized (liste)
		{
			Iterator<ZwischenablageDatensatz> it = liste.iterator();
			while (it.hasNext())
			{
				if (it.next().equals(objekt))
				{
					returnWert = true;
					break;
				}
			}
		}
		return returnWert;
	}

	/**
	 * Entfernt ein {@link Object} aus der Liste.
	 * 
	 * @param object
	 *          Zu entfernendes {@link Object}.
	 */
	protected void entferneEintrag(Object object)
	{
		synchronized (_eintraege)
		{
			_eintraege.remove(object);
			benachrichtigeZuhoerer();
		}

		_debug.fine(getClass().getName()
				+ "::hinzufuegenEintrag \n\tEs wurde ein Eintrag entfernt.\n");

	}

	protected void entferneSelektierteEintraege()
	{
		synchronized (_selektionsSpeicherMehrfachZwischenablage)
		{
			Iterator<ZwischenablageDatensatz> it = _selektionsSpeicherMehrfachZwischenablage
					.iterator();
			while (it.hasNext())
			{
				Object loeschObjekt = it.next();
				synchronized (_eintraege)
				{
					_eintraege.remove(loeschObjekt);
				}
			}
			benachrichtigeZuhoerer();
		}
	}

	/**
	 * Fgt einen {@link IMehrfachZwischenablageDatenZuhoerer} zur Liste der
	 * Zuhrer hinzu.
	 * 
	 * @param zuhoerer
	 *          Hinzuzufgender {@link IMehrfachZwischenablageDatenZuhoerer}.
	 */
	public void hinzufuegenZuhoerer(IMehrfachZwischenablageDatenZuhoerer zuhoerer)
	{
		_debug.fine(getClass().getName() + " => Zuhrer registriert.");
		synchronized (_zuhoerer)
		{
			_zuhoerer.add(zuhoerer);
		}
	}

	/**
	 * Entfernt einen {@link IMehrfachZwischenablageDatenZuhoerer} von der List
	 * der Zuhrer.
	 * 
	 * @param zuhoerer
	 *          Zu entferndender {@link IMehrfachZwischenablageDatenZuhoerer}.
	 */
	public void entferneZuhoerer(IMehrfachZwischenablageDatenZuhoerer zuhoerer)
	{
		_debug.fine(getClass().getName() + " => Zuhrer entfernt");
		synchronized (_zuhoerer)
		{
			_zuhoerer.remove(zuhoerer);
		}
	}

	/**
	 * Benachrichtigt alle Zuhrer darber, dass sich etwas a.d. Datenstruktur
	 * gendert hat.
	 * 
	 */
	private void benachrichtigeZuhoerer()
	{
		int i = -1;

		_debug.fine(getClass().getName()
				+ "::benachrichtigeZuhoerer \n\t=> Anzahl Zuhrer: " + _zuhoerer.size()
				+ "\n");

		synchronized (_zuhoerer)
		{

			Iterator<IMehrfachZwischenablageDatenZuhoerer> it = _zuhoerer.iterator();

			while (it.hasNext())
			{
				synchronized (_eintraege)
				{
					i++;
					_debug.fine(getClass().getName()
							+ "::benachrichtigeZuhoerer \n\t => Benachrichtige Zuhrer #" + i
							+ "\n");

					IMehrfachZwischenablageDatenZuhoerer zuhoerer = it.next();
					zuhoerer.EintraegeListeAktualisiert(_eintraege);
				}

			}
		}
	}

	/**
	 * Liefert die Liste aller Eintrge (als {@link Object}) i.d.
	 * Mehrfachzwischenablage.
	 * 
	 * @return Liste vom Typ {@link Object}.
	 */
	public List<ZwischenablageDatensatz> getListe()
	{
		return _eintraege;
	}

	/**
	 * Privater Konstruktor
	 * 
	 */
	private MehrfachZwischenablageDaten()
	{
		_debug.fine(getClass().getName()
				+ "::MehrfachZwischenablageDaten \n\t=> erzeuge SelectionListener\n");

		ISelectionListener listener = new ISelectionListener()
		{			
			public void selectionChanged(IWorkbenchPart part, ISelection selection)
			{
				/*
				 * Merken, welcher IWorkbenchPart die letzte Selektion durchgefhrt hat.
				 */
				_teilLetzterSelektion = part;

				/*
				 * IStructuredSelection
				 */
				if (selection instanceof IStructuredSelection)
				{
					IStructuredSelection sel = (IStructuredSelection) selection;
					// Object firstElement = sel.getFirstElement();
					// _selektionsSpeicherExtern = firstElement;

					_selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();

					@SuppressWarnings("unchecked")
					Iterator it = sel.iterator();
					while (it.hasNext())
					{
						Object object = it.next();

						_debug.fine("SELEKTION: " + object.getClass().getName());
						_selektionsSpeicherExtern.add(new ZwischenablageDatensatz(object,
								part));
					}
				}
				/*
				 * ITextSelection
				 */
				else if (selection instanceof ITextSelection)
				{
					_selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();
					ITextSelection ts = (ITextSelection) selection;
					String text = ts.getText();
					_selektionsSpeicherExtern
							.add(new ZwischenablageDatensatz(text, part));
				}
				/*
				 * IMarkSelection
				 */
				else
				{
					IMarkSelection ms = (IMarkSelection) selection;
					try
					{
						String text = ms.getDocument().get(ms.getOffset(), ms.getLength());
						_selektionsSpeicherExtern.add(new ZwischenablageDatensatz(text,
								part));
					} catch (BadLocationException ble)
					{
					}
				}

			}

		};

		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService()
				.addSelectionListener(listener);

		_debug
				.fine("MehrfachZwischenablageDaten => SelectionListener registriert.");

	}

	/**
	 * Fgt die Selektion aus dem Selektionsspeicher hinzu.
	 * 
	 */
	public void selektionHinzufuegen()
	{
		aktionLoeschenBereitsEingefuegterObjekte();

		// _eintraege.add(_selektionsSpeicherExtern);

		/*
		 * Entfernen von Elementen, die bereits i.d. Liste der Eintrge vorhanden
		 * sind.
		 */
		List<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();

		synchronized (_selektionsSpeicherExtern)
		{

			Iterator<ZwischenablageDatensatz> it = _selektionsSpeicherExtern
					.iterator();

			while (it.hasNext())
			{
				ZwischenablageDatensatz object = it.next();
				if (pruefeObObjektInListe(object, _eintraege))
				{
					System.err
							.println(getClass().getName()
									+ "::selektionHinzufuegen \n\t=> Das selektierte Objekt ist bereits in der Zwischenablage vorhanden und wird nicht erneut hinzugefgt.");
				} else
				{
					tempListe.add(object);
				}
			}
		}

		synchronized (_eintraege)
		{
			// _eintraege.addAll(0, _selektionsSpeicherExtern);
			_eintraege.addAll(0, tempListe);
		}

		benachrichtigeZuhoerer();
	}

	/**
	 * Fgt die Selektion aus dem Selektionsspeicher hinzu.
	 * 
	 */
	public void selektionHinzufuegenAusschneiden()
	{
		aktionLoeschenBereitsEingefuegterObjekte();

		/*
		 * Die Methode selektionHinzufuegenAusschneiden wird nur dann aktiv, wenn
		 * der aktive ViewPart bzw. EditPart das Interface
		 * IMehrfachZwischenablageAusschneiden implementiert.
		 */

		/*
		 * Entfernen von Elementen, die bereits i.d. Liste der Eintrge vorhanden
		 * sind.
		 */
		List<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();
		synchronized (_selektionsSpeicherExtern)
		{

			Iterator<ZwischenablageDatensatz> it = _selektionsSpeicherExtern
					.iterator();

			while (it.hasNext())
			{
				ZwischenablageDatensatz datenSatz = it.next();
				if (pruefeObObjektInListe(datenSatz, _eintraege))
				{
					System.err
							.println(getClass().getName()
									+ "::selektionHinzufuegen \n\t=> Das selektierte Objekt ist bereits in der Zwischenablage vorhanden und wird nicht erneut hinzugefgt.");
				} else
				{
					datenSatz.setAuschgeschnitten(true);
					tempListe.add(datenSatz);
				}
			}
		}

		synchronized (_eintraege)
		{
			// _eintraege.addAll(0, _selektionsSpeicherExtern);
			_eintraege.addAll(0, tempListe);
		}

		benachrichtigeZuhoerer();
		// benachrichtigeZuhoererAusschneiden();
	}

	/**
	 * Lscht bereits eingefgte Elemente aus der Zwischenablage.
	 */
	private void aktionLoeschenBereitsEingefuegterObjekte()
	{
		LinkedList<ZwischenablageDatensatz> loeschListe = new LinkedList<ZwischenablageDatensatz>();

		synchronized (_eintraege)
		{

			/*
			 * Elmente der Zwischenablage, die bereits eingefgt wurden, in die
			 * Lschliste aufnehmen.
			 */
			Iterator<ZwischenablageDatensatz> itElementeZwischenablage = _eintraege
					.iterator();
			while (itElementeZwischenablage.hasNext())
			{
				ZwischenablageDatensatz datensatz = itElementeZwischenablage.next();
				if (datensatz.isBereitsEingefuegt())
				{
					loeschListe.add(datensatz);
				}
			}

			/*
			 * Lschliste durchlaufen, gefundene Elemente aus Zwischenablage
			 * entfernen.
			 */
			if (loeschListe.size() > 0)
			{
				Iterator<ZwischenablageDatensatz> it = loeschListe.iterator();
				while (it.hasNext())
				{
					ZwischenablageDatensatz datensatz = it.next();
					if (_eintraege.contains(datensatz))
					{
						_eintraege.remove(datensatz);
					}
				}
			}
		}
	}

	/**
	 * Setzt das aktuell selektierte Objekt der
	 * {@link SichtMehrfachZwischenablage}.
	 * 
	 * @param objekt
	 *          Ein Objekt.
	 */
	protected void setSelektionMehrfachZwischenablageIntern(
			List<ZwischenablageDatensatz> objekte)
	{
		synchronized (_selektionsSpeicherMehrfachZwischenablage)
		{
			_selektionsSpeicherMehrfachZwischenablage = objekte;
		}
	}

	/**
	 * Liefert das aktuell selektierte Objekt der
	 * {@link SichtMehrfachZwischenablage}.
	 * 
	 * @return Ein Objekt.
	 */
	protected List<ZwischenablageDatensatz> getSelektionMehrfachZwischenablageIntern()
	{
		LinkedList<ZwischenablageDatensatz> re = new LinkedList<ZwischenablageDatensatz>();

		if (_selektionsSpeicherMehrfachZwischenablage.size() == 0)
		{
			synchronized (_eintraege)
			{
				Iterator<ZwischenablageDatensatz> it = _eintraege.iterator();
				while (it.hasNext())
				{
					re.add(it.next());
				}
			}
		} else
		{
			synchronized (_selektionsSpeicherMehrfachZwischenablage)
			{
				Iterator<ZwischenablageDatensatz> it = _selektionsSpeicherMehrfachZwischenablage
						.iterator();
				while (it.hasNext())
				{
					re.add(it.next());
				}
			}
		}

		// return _selektionsSpeicherMehrfachZwischenablage;
		return re;
	}

	/**
	 * Fgt einen Zuhrer vom Typ {@link IMehrfachZwischenablageAusschneiden} zur
	 * Liste der Zuhrer hinzu.
	 * 
	 * @param ausschneiden
	 *          Hinzuzufgender {@link IMehrfachZwischenablageAusschneiden}-Zuhrer.
	 */
	public void hinzufuegenZuhoererAusschneiden(
			IMehrfachZwischenablageAusschneiden ausschneiden)
	{
		synchronized (_zuhoererAusschneiden)
		{
			_zuhoererAusschneiden.add(ausschneiden);
		}
	}

	/**
	 * Entfernt einen Zuhrer vom Typ {@link IMehrfachZwischenablageAusschneiden}
	 * von der Liste der Zuhrer.
	 * 
	 * @param ausschneiden
	 *          Zu entfernender {@link IMehrfachZwischenablageAusschneiden}-Zuhrer.
	 */
	public void entferneZuhoererAusschneiden(
			IMehrfachZwischenablageAusschneiden ausschneiden)
	{
		synchronized (_zuhoererAusschneiden)
		{
			_zuhoererAusschneiden.remove(ausschneiden);
		}
	}

	/**
	 * Benachrichtigt die AusschneideZuhrer ber die zu entfernenden Objekte. Nur
	 * Objekte der Datenstze, die auf isAusgeschnitten <code>true</code>
	 * liefern, werden ausgeschnitten.
	 */
	protected void benachrichtigeZuhoererAusschneiden(
			ZwischenablageDatensatz zwischenablageDatensatz)
	{
		synchronized (_zuhoererAusschneiden)
		{
			Iterator<IMehrfachZwischenablageAusschneiden> it = _zuhoererAusschneiden
					.iterator();
			while (it.hasNext())
			{
				IMehrfachZwischenablageAusschneiden ausschneiden = it.next();

				if (ausschneiden.getTeil()
						.equals(zwischenablageDatensatz.getHerkunft()))
				{
					if (zwischenablageDatensatz.isAusgeschnitten())
					{
						ausschneiden.entferneSelektion(zwischenablageDatensatz.getWert());
					}
				}
			}
		}
	}

	/**
	 * Leert die Eintrge in der Mehrfachzwischenablage und informiert alle
	 * Zuhrer darber.
	 */
	protected void leereMehrfachZwischenAblage()
	{
		_eintraege.clear();
		benachrichtigeZuhoerer();
	}

}
