/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert die Sicht der Mehrfachzwischenablage
 * 
 * @author HofmannJ
 * 
 */
public class SichtMehrfachZwischenablage extends ViewPart implements
		IMehrfachZwischenablageEinfuegen
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Eltern-Composite
	 */
	private Composite _elternComposite = null;

	/**
	 * TableViewer zur Anzeige der Inhalte der Zwischenablage
	 */
	private TableViewer _tabelleObjekte = null;

	/**
	 * {@link MehrfachZwischenablageDaten#}
	 */
	private MehrfachZwischenablageDaten _zwischenablageDaten = null;

	/**
	 * Zuhrer auf nderungen in der Datenstruktur vom
	 * {@link MehrfachZwischenablageDaten}.
	 */
	private IMehrfachZwischenablageDatenZuhoerer _datenZuhoerer = null;

	public SichtMehrfachZwischenablage()
	{
	}

	@Override
	public void createPartControl(Composite parent)
	{

		// this._elternComposite = parent;
		this._elternComposite = new Composite(parent, SWT.NONE);

		erzeugeTabelle();
		// parent.setLayout(new FillLayout());
		_elternComposite.setLayout(new FillLayout());

		_zwischenablageDaten = MehrfachZwischenablageDaten.getInstanz();

		_datenZuhoerer = new IMehrfachZwischenablageDatenZuhoerer()
		{

			public void EintraegeListeAktualisiert(
					List<ZwischenablageDatensatz> eintraegeListe)
			{
				_tabelleObjekte.setInput(eintraegeListe);

			}

		};

		_zwischenablageDaten.hinzufuegenZuhoerer(_datenZuhoerer);

		List<ZwischenablageDatensatz> liste = _zwischenablageDaten.getListe();
		if (liste != null)
		{
			_tabelleObjekte.setInput(liste);
		}

		verankereKontextMenu();

		/*
		 * Hilfe Kontext
		 */
		PlatformUI.getWorkbench().getHelpSystem().setHelp(_elternComposite,
				"de.bsvrz.buv.rw.basislib.mehrfachzwischenablage");
	}

	public void erzeugeTabelle()
	{
		_tabelleObjekte = new TableViewer(_elternComposite, SWT.FULL_SELECTION
				| SWT.MULTI);

		// TableColumn tc1 = new TableColumn(_tabelleSystemObjekte.getTable(),
		// SWT.CENTER);
		// tc1.setText("Lfd. Nr.:");
		// tc1.setWidth(50);

		TableColumn tc2 = new TableColumn(_tabelleObjekte.getTable(), SWT.LEFT);
		tc2.setText("ObjektTyp:");
		tc2.setWidth(200);

		TableColumn tc3 = new TableColumn(_tabelleObjekte.getTable(), SWT.LEFT);
		tc3.setText("Weitere Informationen:");
		tc3.setWidth(300);

		_tabelleObjekte.getTable().setHeaderVisible(true);
		_tabelleObjekte.getTable().setLinesVisible(true);

		_tabelleObjekte.setContentProvider(new TabelleInhaltsVersorger());
		_tabelleObjekte.setLabelProvider(new TabelleDarstellungsVersorger());

		_tabelleObjekte.addSelectionChangedListener(new ISelectionChangedListener()
		{

			public void selectionChanged(SelectionChangedEvent event)
			{

				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();

				List<ZwischenablageDatensatz> tempSelektion = new LinkedList<ZwischenablageDatensatz>();
				
				@SuppressWarnings("unchecked")
				Iterator<ZwischenablageDatensatz> it = selection.iterator();
				while (it.hasNext())
				{
					ZwischenablageDatensatz object = it.next();
					if (object != null)
					{
						tempSelektion.add(object);
					}
				}

				_zwischenablageDaten
						.setSelektionMehrfachZwischenablageIntern(tempSelektion);
			}

		});

	}

	/**
	 * Entfernt den Listener aus {@link MehrfachZwischenablageDaten}.
	 */
	public void dispose()
	{
		super.dispose();

		// if (_datenZuhoerer != null)
		// {
		// _zwischenablageDaten.entferneZuhoerer(_datenZuhoerer);
		// }

	}

	/**
	 * Verankert das Kontextmen
	 * 
	 */
	private void verankereKontextMenu()
	{
		_debug.fine(getClass().getName()
				+ "::verankereContextMenu => Verankere Kontextmen");

		MenuManager menuMgr = new MenuManager("#PopupMenuMehrfachZwischenablage");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{

				_debug.fine(getClass().getName()
						+ "::verankereKontextMenu \n\t => menuaboutToShow\n");

				fuelleKontextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_tabelleObjekte.getControl());
		_tabelleObjekte.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _tabelleObjekte);
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *          Der {@link IMenuManager}, der die Struktur hinter dem Kontextmen
	 *          aufnimmt.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{
		/**
		 * Liste der Actions, die im Kontextmen dargestellt werden sollen
		 */
		List<IAction> actions = new LinkedList<IAction>();

		// Action lschen
		Action actionLoeschen = new Action()
		{
			public void run()
			{
				_debug.fine("AKTION BEREIT");
				aktionSelektiertesElementLoeschen();
				// Object firstElement = ((IStructuredSelection)
				// _tabelleObjekte.getSelection()).getFirstElement();
				// if (firstElement instanceof Element)
				// {
				//					
				// }
			}
		};
		actionLoeschen.setText("Lschen");
		actionLoeschen.setToolTipText("Lscht das Element.");
		actionLoeschen.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		actions.add(actionLoeschen);

		// Hinzufgen der Actions zum Menmanager
		Iterator<IAction> it = actions.iterator();
		int i = -1;
		while (it.hasNext())
		{
			i++;
			manager.add(it.next());
			if (i == 2)
			{
				manager.add(new Separator());
			}
		}
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	/**
	 * Lscht ein Element aus der MehrfachZwischenablage
	 */
	private void aktionSelektiertesElementLoeschen()
	{
		// _zwischenablageDaten.entferneEintrag(_zwischenablageDaten.);
		_debug.fine("LSCHE");
		_zwischenablageDaten.entferneSelektierteEintraege();
	}

	@Override
	public void setFocus()
	{
		if (_elternComposite != null)
			_elternComposite.setFocus();
	}

	private class TabelleInhaltsVersorger implements IStructuredContentProvider
	{
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement)
		{
			Object[] listenObjekte = null;

			if (inputElement instanceof List)
			{
				List tempList = (List) inputElement;

				listenObjekte = new Object[tempList.size()];
			
				Iterator it = tempList.iterator();

				int i = -1;
				while (it.hasNext())
				{
					i++;
					Object object = it.next();

					if (object != null)
					{
						_debug.fine(getClass().getName() + "::getElements \n\t=> I: " + i
								+ " Typ: " + object.getClass().getName() + "\n");

						listenObjekte[i] = object;
					} else
					{
						_debug.fine(getClass().getName() + "::getElements \n\t=> I: " + i
								+ " <= NULL Object\n");
						listenObjekte[i] = "NULL Objekt";
					}

				}

			} else
			{
				//Keine Eingabe
			}

			return listenObjekte;
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class TabelleDarstellungsVersorger implements ITableLabelProvider
	{

		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		public String getColumnText(Object object, int columnIndex)
		{
			String returnString = "";
			ZwischenablageDatensatz zwischenablageDatensatz = null;

			// ID
			if (columnIndex == 0)
			{

				if (object instanceof ZwischenablageDatensatz)
				{
					zwischenablageDatensatz = (ZwischenablageDatensatz) object;
				}

				String tempString = "";
				if (zwischenablageDatensatz != null)
				{
					tempString = zwischenablageDatensatz.getWert().getClass().getName();
				} else
				{
					tempString = object.getClass().getName();
				}

				if (tempString.length() > 20)
				{
					int first = tempString.indexOf(".");
					returnString = returnString + tempString.substring(0, first) + "..";
					int last = tempString.lastIndexOf(".");
					returnString = returnString + tempString.substring(last);
				} else
				{
					returnString = zwischenablageDatensatz.getWert().getClass().getName();
				}

			}
			// Bezeichnung
			else if (columnIndex == 1)
			{
				if (object instanceof ZwischenablageDatensatz)
				{
					zwischenablageDatensatz = (ZwischenablageDatensatz) object;

					if (zwischenablageDatensatz != null)
					{
						returnString = "" + zwischenablageDatensatz.getWert();
					} else
					{
						returnString = "null";
					}
				} else
				{
					returnString = "" + object.toString();
				}
			}
			// Sonst
			else
			{
				returnString = "Fr diese Spalte findet keine Versorgung statt.";
			}

			return returnString;
		}

		public void addListener(ILabelProviderListener listener)
		{
		}

		public void dispose()
		{
		}

		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	public void einfuegen(List<Object> objekte)
	{
		LinkedList<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();
		Iterator<Object> it = objekte.iterator();
		while (it.hasNext())
		{
			Object object = it.next();
			tempListe.add(new ZwischenablageDatensatz(object));
			if (object instanceof String)
			{
				_debug.fine("STRING: " + (String) object);
			}
		}

		_zwischenablageDaten.hinzufuegenEintraege(tempListe);
	}

}
