/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.LinkedList;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert den Transfer fr {@link DeskriptorElement}s.
 * 
 * @version $Revision: 1.2 $ / $Date: 2012/08/21 07:44:07 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class TransferDeskriptorElement extends ByteArrayTransfer
{

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private static final String _typName = "Descriptor-Element-Format";

	private static final int _typId = registerType(_typName);

	/**
	 * Singleton-Instanz
	 */
	private static TransferDeskriptorElement _instanz = new TransferDeskriptorElement();

	public static TransferDeskriptorElement getInstanz()
	{
		return _instanz;
	}

	public void javaToNative(Object object, TransferData transferData)
	{
		_debug.fine(getClass().getName() + "::j2Native => Element");

		if (object == null || !(object instanceof DeskriptorElement[])
				&& !(object instanceof DeskriptorElement))
		{
			_debug.fine(getClass().getName() + "::j2Native => Abbruch: "
					+ object.getClass().getName());
			return;
		}

		if (isSupportedType(transferData))
		{
			_debug.fine(getClass().getName()
					+ "::j2Native => untersttzer Transfertyp");

			DeskriptorElement[] elemente = null;

			// Instanz von DescriptorElement?
			if (object instanceof DeskriptorElement)
			{
				// dann in ein DescriptorElement[] mit einem Element umwandeln
				DeskriptorElement element = (DeskriptorElement) object;
				elemente = new DeskriptorElement[] { element };
			} else if (object instanceof DeskriptorElement[])
			{
				// Sonst nur nach DescriptorElement[]-casten
				elemente = (DeskriptorElement[]) object;
			}

			try
			{
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				DataOutputStream writeOut = new DataOutputStream(out);
				for (int i = 0; i < elemente.length; i++)
				{

					// PluginID
					String pluginID = elemente[i].getPluginID();
					if (pluginID == null)
					{
						pluginID = "";
					}
					byte[] buffer = pluginID.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ActionId
					String actionID = elemente[i].getAktionID();
					if (actionID == null)
					{
						actionID = "";
					}
					buffer = actionID.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// LabelText
					String labelText = elemente[i].getText();
					if (labelText == null)
					{
						labelText = "";
					}
					buffer = labelText.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ToolTipText
					String toolTipText = elemente[i].getToolTipText();
					if (toolTipText == null)
					{
						toolTipText = "";
					}
					buffer = toolTipText.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// ActionDescription
					String actionDescription = elemente[i].getAktionBeschreibung();
					if (actionDescription == null)
					{
						actionDescription = "";
					}
					buffer = actionDescription.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// IconPfad
					String iconPfad = elemente[i].getIconPfad();
					if (iconPfad == null)
					{
						iconPfad = "";
					}
					buffer = iconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// HoverIconPfad
					String hoverIconPfad = elemente[i].getHoverIconPfad();
					if (hoverIconPfad == null)
					{
						hoverIconPfad = "";
					}
					buffer = hoverIconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// DisabledIconPfad
					String disabledIconPfad = elemente[i].getDeaktiviertesIconPfad();
					if (disabledIconPfad == null)
					{
						disabledIconPfad = "";
					}
					buffer = disabledIconPfad.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

					// Ausklapp Aktion SWT.DROP_DOWN
					String ausklappAktion = "nein";
					if (elemente[i].istAusklappAktion())
					{
						ausklappAktion = "ja";
					}
					buffer = ausklappAktion.getBytes();
					writeOut.writeInt(buffer.length);
					writeOut.write(buffer);

				}

				byte[] buffer = out.toByteArray();
				writeOut.close();

				super.javaToNative(buffer, transferData);

			} catch (IOException e)
			{
				e.printStackTrace();
			}
		} else
		{
			_debug.fine(getClass().getName()
					+ "::j2Native => Nicht untersttzer Transfertyp");
		}
	}

	public Object nativeToJava(TransferData transferData)
	{
		if (isSupportedType(transferData))
		{

			byte[] buffer = (byte[]) super.nativeToJava(transferData);
			if (buffer == null)
			{
				_debug.fine(getClass().getName() + "::native2J => Buffer == NULL");
				return null;
			}

			LinkedList<DeskriptorElement> lDeskriptorElemente = new LinkedList<DeskriptorElement>();

			String PluginId = null;
			String ActionId = null;
			String LabelText = null;
			String ToolTipText = null;
			String ActionDescripton = null;
			String IconPfad = null;
			String HoverIconPfad = null;
			String DisabledIconPfad = null;
			boolean bAusklappAktion = false;

			try
			{
				ByteArrayInputStream in = new ByteArrayInputStream(buffer);
				DataInputStream readIn = new DataInputStream(in);
				// while (readIn.available() > 20)
				while (readIn.available() > 0)
				{
					// PluginId
					int size = readIn.readInt();
					byte[] pluginid = new byte[size];
					readIn.read(pluginid);
					PluginId = new String(pluginid);

					// ActionId
					size = readIn.readInt();
					byte[] actionid = new byte[size];
					readIn.read(actionid);
					ActionId = new String(actionid);

					// LabelText
					size = readIn.readInt();
					byte[] labeltext = new byte[size];
					readIn.read(labeltext);
					LabelText = new String(labeltext);

					// ToolTipText
					size = readIn.readInt();
					byte[] tooltiptext = new byte[size];
					readIn.read(tooltiptext);
					ToolTipText = new String(tooltiptext);

					// ActionDescription
					size = readIn.readInt();
					byte[] actiondescription = new byte[size];
					readIn.read(actiondescription);
					ActionDescripton = new String(actiondescription);

					// IconPfad
					size = readIn.readInt();
					byte[] iconpfad = new byte[size];
					readIn.read(iconpfad);
					IconPfad = new String(iconpfad);

					// HoverIconPfad
					size = readIn.readInt();
					byte[] hovericonpfad = new byte[size];
					readIn.read(hovericonpfad);
					HoverIconPfad = new String(hovericonpfad);

					// DisabledIconPfad
					size = readIn.readInt();
					byte[] disablediconpfad = new byte[size];
					readIn.read(disablediconpfad);
					DisabledIconPfad = new String(disablediconpfad);

					// AusklappAktion
					size = readIn.readInt();
					byte[] ausklappAktion = new byte[size];
					readIn.read(ausklappAktion);
					String auswertung = new String(ausklappAktion);
					if (auswertung.equals("ja"))
					{
						bAusklappAktion = true;
					}

					DeskriptorElement tempElement = new DeskriptorElement(PluginId,
							ActionId, LabelText, ToolTipText, ActionDescripton, IconPfad,
							HoverIconPfad, DisabledIconPfad, bAusklappAktion);

					/**
					 * Hier prfen wir, ob nich noch in der Registry das passende
					 * DescriptorElement vorhanden ist.
					 */
					DeskriptorElement descriptorElement = ElementRegistrierung
							.getInstanz().getDeskriptorElement(ActionId);
					if (descriptorElement != null)
					{
						lDeskriptorElemente.add(descriptorElement);
					} else
					{
						lDeskriptorElemente.add(tempElement);
					}

				}
				readIn.close();

			} catch (IOException e)
			{
				e.printStackTrace();
				return null;
			}

			DeskriptorElement[] aDeskriptorElemente = lDeskriptorElemente
					.toArray(new DeskriptorElement[lDeskriptorElemente.size()]);

			return aDeskriptorElemente;
		}
		return null;
	}

	@Override
	protected int[] getTypeIds()
	{
		return new int[] { _typId };
	}

	@Override
	protected String[] getTypeNames()
	{
		return new String[] { _typName };
	}

}
