/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.menu.deskriptoren.zugriff;

import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Bezieht sich auf den Import der verfgbaren Elemente aus der Tabelle.
 * 
 * Importiert aus einer XML-Datei die Elemente, die zur Zusammenstellung des Mens zur Verfgung stehen sollen und
 * registriert diese in der {@link ElementRegistrierung}. <br>
 * Auf diese Weise ist es auch mglich Mens/Symbolleisten zusammenzustellen, wenn die entsprechenden Plug-Ins noch
 * nicht existieren.
 *  *
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:32 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class XmlDeskriptorElementImport
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * JDOM Dokument
   */
  Document _doc = null;

  /**
   * Wurzelelement
   */
  Element _wurzelElement = null;

  private static final String _attPluginId = "pluginId";

  private static final String _attActionId = "actionId";

  private static final String _attLabelText = "labelText";

  private static final String _attAusklappbar = "ausklappbar";

  /**
   * Liest eine XML-Struktur ein.
   * 
   * @param xmlString
   */
  public void einlesenXml(String xmlString)
  {
    SAXBuilder builder = new SAXBuilder();
    builder.setValidation(false);
    try
    {
      StringReader reader = new StringReader(xmlString);
      _doc = builder.build(reader);
      _wurzelElement = _doc.getRootElement();
    }
    catch (JDOMException e)
    {
      _debug.fine(getClass().getName() + " => JDOMException");
      e.printStackTrace();
    }
    catch (IOException e)
    {
      _debug.fine(getClass().getName() + " => IOException");
      e.printStackTrace();
    }
  }

  public Element getWurzel()
  {
    return this._wurzelElement;
  }

  /**
   * Sorgt fr den Import der eingelesenen XML-Datei
   * 
   */
  public void importieren()
  {
    if (_wurzelElement != null)
    {
      @SuppressWarnings("unchecked")
      List<Element> lElement = _wurzelElement.getChildren("element");
      _debug.fine("SIZE: " + lElement.size());
      Iterator<Element> it = lElement.iterator();
      while (it.hasNext())
      {
        Element em = it.next();
        String pluginId = em.getAttributeValue(_attPluginId);
        String actionId = em.getAttributeValue(_attActionId);
        String labelText = em.getAttributeValue(_attLabelText);
        String ausklappbar = em.getAttributeValue(_attAusklappbar);
        boolean bAusklappbar = false;
        if (ausklappbar.equals("ja"))
        {
          bAusklappbar = true;
        }

        DeskriptorElement descriptorElement = new DeskriptorElement(pluginId, actionId, labelText, null, null, null,
            null, null, bAusklappbar);
        try
        {
          ElementRegistrierung.getInstanz().registriereDeskriptorElement(descriptorElement);
        }
        catch (IllegalArgumentException e)
        {
          e.printStackTrace();
        }

      }
    }
  }

}
