/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.LinkedList;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.BereichsInfoAsbKnoten;
import de.bsvrz.dav.daf.main.ClientDavConnection;

public class FilterAsbOrtsangabe implements Cloneable
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _connection = null;

	/**
	 * PID der selektierten Strae
	 */
	private String _pidStrasse = null;

	/**
	 * Anfangsknoten der ASB-Stationierung
	 */
	private String _anfangsKnoten = null;

	/**
	 * Endknoten der ASB-Stationierung
	 */
	private String _endKnoten = null;

	/**
	 * Stationierung
	 */
	private long _stationierung = -1;

	/**
	 * {@link AsbOrtsangabe}
	 */
	private AsbOrtsangabe _asbOrtsangabe = null;

	/**
	 * AsbStationierung
	 */
	private EnumAsbVerkehrsRichtung _asbVerkehrsRichtung = null;

	/**
	 * Konstruktor. bernimmt die Verbindung zum DaV und fhrt die Initialisierung
	 * von {@link AsbOrtsangabe} durch.
	 * 
	 * @param connection
	 *          DaV-Verbindung
	 */
	public FilterAsbOrtsangabe(ClientDavConnection connection)
	{
		this._connection = connection;
		if (_connection != null)
		{
			init();
		}
	}

	/**
	 * Fhrt die Initialisierung durch.
	 */
	private void init()
	{
		_asbOrtsangabe = new AsbOrtsangabe(_connection);
	}

	/**
	 * Liefert die Listen der auswhlbaren {@link Strasse}n zurck.
	 * 
	 * @return Liste der auswhlbaren {@link Strasse}n
	 */
	public LinkedList<Strasse> getStrassen()
	{
		LinkedList<Strasse> strassen = _asbOrtsangabe.getStrassen();
		return strassen;
	}

	/**
	 * Selektiert die gewhlte {@link Strasse} an Hand ihrerer PID. Fhrt die
	 * Initialisierung der {@link AsbOrtsangabe} an Hand der Pid der Strae durch.
	 * 
	 * @param pid
	 *          PID der zu selektierenden {@link Strasse}
	 */
	public void setSelektierteStrasse(String pid)
	{
		this._pidStrasse = pid;
		if (_asbOrtsangabe != null)
		{
			_asbOrtsangabe.initialisiereStrukturNachStrasse(_pidStrasse);
		}
	}

	/**
	 * Liefert die {@link AsbOrtsangabe}, mit der sich die bentigten Knoten
	 * bestimmen lassen und die Stationierung berprfen lt.
	 * 
	 * @return {@link AsbOrtsangabe}
	 */
	public AsbOrtsangabe getAsbOrtsangabe()
	{
		return _asbOrtsangabe;
	}

	/**
	 * Liefert die PID der selektierten {@link Strasse}.
	 * 
	 * @return PID der selektierten {@link Strasse}
	 */
	public String getPidSelektierteStrasse()
	{
		return _pidStrasse;
	}

	/**
	 * Liefert den AngangsKnoten.
	 * 
	 * @return Anfangsknoten.
	 */
	public String getAnfangsKnoten()
	{
		return this._anfangsKnoten;
	}

	/**
	 * Setzt den Anfangsknoten.
	 * 
	 * @param anfangsKnoten
	 *          Anfangsknoten.
	 */
	public void setAnfangsKnoten(String anfangsKnoten)
	{
		this._anfangsKnoten = anfangsKnoten;
	}

	// /**
	// * Liefert die Tmc-Richtung des AnfangsKnotens.
	// *
	// * @return Tmc-Richtung des AnfangsKnotens
	// */
	// public EnumTmcRichtung getAnfangsKnotenTmcRichtung()
	// {
	// return _tmcRichtungAnfangsKnoten;
	// }
	//
	// /**
	// * Setzt die Tmc-Richtung des AnfangsKnotens.
	// *
	// * @param richtung
	// * Tmc-Richtung des AnfangsKnotens
	// */
	// public void setAnfangsKnotenTmcRichtung(EnumTmcRichtung richtung)
	// {
	// this._tmcRichtungAnfangsKnoten = richtung;
	// }

	/**
	 * Liefert den Endknoten.
	 * 
	 * @return Endknoten
	 */
	public String getEndKnoten()
	{
		return _endKnoten;
	}

	/**
	 * Setzt den Endknoten.
	 * 
	 * @param endKnoten
	 */
	public void setEndKnoten(String endKnoten)
	{
		this._endKnoten = endKnoten;
	}

	// /**
	// * Liefert die Tmc-Richtung des EndKnotens.
	// *
	// * @return Tmc-Richtung des EndKnotens
	// */
	// public EnumTmcRichtung getEndKnotenTmcRichtung()
	// {
	// return _tmcRichtungEndKnoten;
	// }
	//
	// /**
	// * Setzt die Tmc-Richtung des EndKnotens.
	// *
	// * @param richtung
	// * Tmc-Richtung des EndKnotens
	// */
	// public void setEndKnotenTmcRichtung(EnumTmcRichtung richtung)
	// {
	// this._tmcRichtungEndKnoten = richtung;
	// }

	/**
	 * Setzt die Stationierung
	 * 
	 * @param stationierungswert
	 */
	public void setStationierung(long stationierungswert)
	{
		this._stationierung = stationierungswert;
	}

	/**
	 * Liefert die Stationierung zurck.
	 * 
	 * @return Stationierung
	 */
	public long getStationierung()
	{
		return _stationierung;
	}

	/**
	 * Liefert die Verkehrsrichtung.
	 * 
	 * @return {@link EnumAsbVerkehrsRichtung}
	 */
	public EnumAsbVerkehrsRichtung getAsbVerkehrsRichtung()
	{
		return _asbVerkehrsRichtung;
	}

	/**
	 * Setzt die Verkehrsrichtung.
	 * 
	 * @param asbVerkehrsRichtung
	 *          {@link EnumAsbVerkehrsRichtung}
	 */
	public void setAsbVerkehrsRichtung(EnumAsbVerkehrsRichtung asbVerkehrsRichtung)
	{
		this._asbVerkehrsRichtung = asbVerkehrsRichtung;
	}

	/**
	 * Ermittelt fr zwei {@link BereichsInfoAsbKnoten} der positiven und
	 * negativen TMC-Richtung den unter Bercksichtigung der
	 * {@link EnumAsbVerkehrsRichtung} den Gltigkeitsbereich.
	 * 
	 * @param bereichA
	 *          {@link BereichsInfoAsbKnoten}
	 * @param bereichB
	 *          {@link BereichsInfoAsbKnoten}
	 * @param richtung
	 *          {@link EnumAsbVerkehrsRichtung}, fr welche die Prfung erfolgen
	 *          soll.
	 * @return Liefert ein long-Array mit dem Von-Wert bei Index==0 und dem
	 *         Bis-Wert bei Index==1 oder <code>null</code>, falls kein Bereich
	 *         ermittelt werden konnte.
	 */
	protected static long[] ermittleMinMax(BereichsInfoAsbKnoten bereichA,
			BereichsInfoAsbKnoten bereichB, EnumAsbVerkehrsRichtung richtung)
	{
		long posMax = -1;
		long posMin = -1;
		long negMin = -1;
		long negMax = -1;

		long[] re = null;

		// A und B != null
		if ((bereichA != null) && (bereichB != null))
		{
			// A == B
			if (bereichA == bereichB)
			{
				if (bereichA.getAsbKnotenVon().getAsbVerkehrsRichtung()
						.equals(richtung))
				{
					re = new long[] { bereichA.getBereichMin(), bereichA.getBereichMax() };
				}
			} else
			{
				// Richtungen von A und B gleich?
				if (bereichA.getAsbKnotenVon().getAsbVerkehrsRichtung().equals(
						bereichB.getAsbKnotenVon().getAsbVerkehrsRichtung()))
				{
					// Beide Bereiche haben die richtige Richtung?
					if (bereichA.getAsbKnotenVon().getAsbVerkehrsRichtung().equals(
							richtung))
					{						
						posMin = bereichA.getBereichMin();
						posMax = bereichA.getBereichMax();
						negMin = bereichB.getBereichMin();
						negMax = bereichB.getBereichMax();

						long min = -1;
						long max = -1;

						min = negMin;
						if (posMin < min)
						{
							min = posMin;
						}

						max = negMax;
						if (posMax > max)
						{
							max = posMax;
						}

						re = new long[] { min, max };
					} else if ((richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
							|| (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)))
					{						
						posMin = bereichA.getBereichMin();
						posMax = bereichA.getBereichMax();
						negMin = bereichB.getBereichMin();
						negMax = bereichB.getBereichMax();

						long min = -1;
						long max = -1;

						min = negMin;
						if (posMin < min)
						{
							min = posMin;
						}

						max = negMax;
						if (posMax > max)
						{
							max = posMax;
						}
						re = new long[] { min, max };
					}
				} else if ((bereichA.getAsbKnotenVon().getAsbVerkehrsRichtung()
						.equals(richtung))
						|| (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
						|| (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)))
				{
					re = new long[] { bereichA.getBereichMin(), bereichA.getBereichMax() };
				} else if ((bereichB.getAsbKnotenVon().getAsbVerkehrsRichtung()
						.equals(richtung))
						|| (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
						|| (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)))
				{
					re = new long[] { bereichB.getBereichMin(), bereichB.getBereichMax() };
				}

			}
		}
		// A != null
		else if ((bereichA != null) && (bereichB == null))
		{
			System.out.println("A != null");
			if ((bereichA.getAsbKnotenVon().getAsbVerkehrsRichtung().equals(richtung))
					|| (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
					|| (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)))
			{
				re = new long[] { bereichA.getBereichMin(), bereichA.getBereichMax() };
			}
		}
		// B != null
		else if ((bereichA == null) && (bereichB != null))
		{
			System.out.println("B != null");
			if ((bereichB.getAsbKnotenVon().getAsbVerkehrsRichtung().equals(richtung))
					|| (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
					|| (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT)))
			{
				re = new long[] { bereichB.getBereichMin(), bereichB.getBereichMax() };
			}
		} else if ((bereichA == null) && (bereichB == null))
		{
			System.out.println("A + B == NULL!!!");
		}

		return re;
	}

	public String toString()
	{
		String re = "===" + getClass().getName() + "===\n";
		re = re + "Strasse (PID):     " + _pidStrasse + "\n";
		re = re + "VON (PID):         " + _anfangsKnoten + "\n";
		re = re + "NACH (PID):        " + _endKnoten + "\n";
		re = re + "Stationierung:     " + _stationierung + "\n";
		re = re + "Verkehrs-Richtung: " + _asbVerkehrsRichtung + "\n";

		for (int i = 0; i < (6 + getClass().getName().length()); i++)
		{
			re = re + "=";
		}

		re = re + "\n";
		return re;
	}

	public Object clone()
	{
		FilterAsbOrtsangabe re = new FilterAsbOrtsangabe(_connection);
		re._pidStrasse = _pidStrasse;
		re._anfangsKnoten = _anfangsKnoten;
		re._endKnoten = _endKnoten;
		re._stationierung = _stationierung;
		re._asbOrtsangabe = _asbOrtsangabe;
		re._asbVerkehrsRichtung = _asbVerkehrsRichtung;
		return re;
	}

}
