/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort.hilfsklassen;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.sys.funclib.debug.Debug;

public class TestBetriebskilometer
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verbindung zum DaV
   */
  private ClientDavConnection _verbindung = null;

  /**
   * IP
   */
  private static final String _ip = "192.0.1.117";

  /**
   * Port
   */
  private static final int _port = 8083;

  /**
   * Login
   */
  private static final String _benutzer = "Tester";

  /**
   * Passwort
   */
  private static final String _passwort = "geheim";

  /**
   * @param args
   */
  public static void main(String[] args)
  {
    TestBetriebskilometer test = new TestBetriebskilometer();
    test.baueDaVVerbindungAuf();

    if (test._verbindung != null)
    {
      test.test();
    }

    test.schliesseDaVVerbindung();
  }

  private void test()
  {
    Betriebskilometer bk = new Betriebskilometer(_verbindung);

    String pidStrasse = "strasse.A.3.tmclc.07077"; // Kln-Frankfurt
    // String pidStrasse = "strasse.A.45.tmclc.07115"; //Gieen-Hagen

    // A3
    // Pruefung(listePos, 24400); //102
    // Pruefung(listePos, 78461); //-1
    // Pruefung(listePos, 138120); //101
    // Pruefung(listeNeg, 76460); //-1
    // Pruefung(listeNeg, 135547); //101
    // Pruefung(listeNeg, 97973); //-1

    // A45
    // long bkWert = 119702; //101

    // A3
    // long bkWert = 24400; //102
    // long bkWert = 78461; //-1
    // long bkWert = 138120; //101
    // long bkWert = 76460; //-1
    // long bkWert = 135547; //101
    long bkWert = 97973; // -1

    LinkedList<String> liste1 = bk.pruefeBetriebsKilometerVorkommen(pidStrasse, bkWert);
    LinkedList<String> liste2 = bk.pruefeBetriebsKilometerVorkommen(pidStrasse, bkWert + 1);

    _debug.fine("Liste 1 Anzahl Elemente: " + liste1.size());
    Iterator<String> itL1 = liste1.iterator();
    while (itL1.hasNext())
    {
      String wert = itL1.next();
      _debug.fine("WERT L1: " + wert);
    }

    _debug.fine("Liste 2 Anzahl Elemente: " + liste2.size());
    Iterator<String> itL2 = liste2.iterator();
    while (itL2.hasNext())
    {
      String wert = itL2.next();
      _debug.fine("WERT L2: " + wert);
    }

    // /*
    // * Sortierte Ausgabe aller Straen
    // */
    // Iterator<Strasse> it = bk.getStrassen().iterator();
    // while(it.hasNext())
    // {
    // System.out.println("STR: " + it.next().toString());
    // }

  }

  /**
   * Baut die Verbindung zum DaV auf.
   */
  private void baueDaVVerbindungAuf()
  {
    try
    {
      _verbindung = new ClientDavConnection();

      // IP setzen
      _verbindung.getClientDavParameters().setDavCommunicationAddress(_ip);

      // Port setzen
      _verbindung.getClientDavParameters().setDavCommunicationSubAddress(_port);

      // Pfad zum Zwischenspeichern setzen
      _verbindung.getClientDavParameters().setConfigurationPath("C:\\temp");

      try
      {
        _verbindung.connect();
        System.out.println("Verbindungsaufbau: Verbindung erfolgreich aufgebaut.");

        try
        {
          _verbindung.login(_benutzer, _passwort);
          System.out.println("Verbindungsaufbau: Benutzer erfolgreich eingeloggt.");
        }
        catch (InconsistentLoginException e)
        {
          // e.printStackTrace();
          System.err.println("Verbindungsaufbau Fehler: Konnte Benutzer nicht einloggen.");
          _verbindung = null;
        }

      }
      catch (CommunicationError e)
      {
        // e.printStackTrace();
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }
      catch (ConnectionException e)
      {
        // e.printStackTrace();
        System.err.println("Verbindungsaufbau Fehler: " + e.toString());
        _verbindung = null;
      }

    }
    catch (MissingParameterException e)
    {
      // e.printStackTrace();
      System.err.println("Verbindungsaufbau Fehler: " + e.toString());
      _verbindung = null;
    }
  }

  /**
   * Schliet die Verbindung zum DaV
   */
  private void schliesseDaVVerbindung()
  {
    if (_verbindung != null)
    {
      _verbindung.disconnect(false, "Kein Fehler.");
      _debug.fine("\nVerbindungsabbau: Verbindung wurde abgebaut.");
    }
  }

}
