/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.seiteneinstellungen;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;

/**
 * Liefert die {@link PreferencePage} zum Vornehmen der Seiteneinstellungen.
 * 
 * @author HofmannJ
 * 
 */
public class Seiteneinstellungen extends PreferencePage implements
		IWorkbenchPreferencePage
{

	/**
	 * ID der {@link PreferencePage}
	 */
	public static String ID = "de.bsvrz.buv.rw.basislib.pageSeiteneinstellungen";

	/**
	 * Composite zum Vornehmen der Seiteneinstellungen.
	 */
	private Zusammelstellung _zusammenstellung = null;

	/**
	 * Konstruktor, parameterlos.
	 */
	public Seiteneinstellungen()
	{
	}

	/**
	 * Konstruktor, setzt den Titel der Seiteneinstellungen.
	 * 
	 * @param title
	 *          Titel der Seiteneinstellungen.
	 */
	public Seiteneinstellungen(String title)
	{
		super(title);
	}

	/**
	 * Konstruktor, setzt den Titel und das Bild der Seiteneinstellugnen.
	 * 
	 * @param title
	 *          Titel der Seiteneinstellungen.
	 * @param image
	 *          Bild der Seiteneinstellungen
	 */
	public Seiteneinstellungen(String title, ImageDescriptor image)
	{
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent)
	{
		_zusammenstellung = new Zusammelstellung(parent, SWT.NONE);
		GridData gridDataBasisZusammenstellung = new GridData();
		gridDataBasisZusammenstellung.horizontalAlignment = GridData.FILL;
		gridDataBasisZusammenstellung.grabExcessHorizontalSpace = true;
		gridDataBasisZusammenstellung.grabExcessVerticalSpace = true;
		gridDataBasisZusammenstellung.verticalAlignment = GridData.FILL;
		_zusammenstellung.setLayoutData(gridDataBasisZusammenstellung);

		laden();

		return null;
	}

	/**
	 * Speichert die Einstellungen
	 * 
	 */
	private void sichern()
	{
		String[] kopfFussZeilen = _zusammenstellung.getKopfFussZeilen();
		getPreferenceStore().setValue(DefsSeiteneinstellungen.kopfzeileLinks,
				kopfFussZeilen[0]);
		getPreferenceStore().setValue(DefsSeiteneinstellungen.kopfzeileMitte,
				kopfFussZeilen[1]);
		getPreferenceStore().setValue(DefsSeiteneinstellungen.kopfzeileRechts,
				kopfFussZeilen[2]);
		getPreferenceStore().setValue(DefsSeiteneinstellungen.fusszeileLinks,
				kopfFussZeilen[3]);
		getPreferenceStore().setValue(DefsSeiteneinstellungen.fusszeileMitte,
				kopfFussZeilen[4]);
		getPreferenceStore().setValue(DefsSeiteneinstellungen.fusszeileRechts,
				kopfFussZeilen[5]);

	}

	/**
	 * Ldt die Einstellungen
	 */
	private void laden()
	{
		String fusszeileLinks = getPreferenceStore().getString(
				DefsSeiteneinstellungen.fusszeileLinks);

		String fusszeileMitte = getPreferenceStore().getString(
				DefsSeiteneinstellungen.fusszeileMitte);

		String fusszeileRechts = getPreferenceStore().getString(
				DefsSeiteneinstellungen.fusszeileRechts);

		String kopfzeileLinks = getPreferenceStore().getString(
				DefsSeiteneinstellungen.kopfzeileLinks);

		String kopfzeileMitte = getPreferenceStore().getString(
				DefsSeiteneinstellungen.kopfzeileMitte);

		String kopfzeileRechts = getPreferenceStore().getString(
				DefsSeiteneinstellungen.kopfzeileRechts);

		_zusammenstellung.setKopfFussZeilen(new String[] { kopfzeileLinks,
				kopfzeileMitte, kopfzeileRechts, fusszeileLinks, fusszeileMitte,
				fusszeileRechts });

	}

	@Override
	protected void performApply()
	{
		sichern();
	}

	@Override
	public boolean performOk()
	{
		performApply();
		return super.performOk();
	}

	@Override
	public void performHelp()
	{
		String helpContextId = Activator.PLUGIN_ID + ".seiteneinstellungen";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	@Override
	public void init(IWorkbench workbench)
	{
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		setPreferenceStore(store);
	}

	@Override
	protected void performDefaults()
	{
		_zusammenstellung.setKopfFussZeilen(new String[] { null, null, null, null,
				null, null });
		super.performDefaults();
	}

}
