/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.simulationsvariante;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

public class SichtSimulationsvarianteDarstellenEinstellen extends ViewPart
{

	public static final String ID = "de.bsvrz.buv.rw.basislib.simulationsvariante.SichtSimulationsvarianteDarstellenEinstellen";

	private Composite _basis = null;

	private Group _gruppeAktuelleSimulationsvariante = null;

	private Group _gruppeSimulationsvarianteWaehlen = null;

	private Group _gruppeKontrolle = null;

	private Label _labelAktuelleSimulationsvariante = null;

	/**
	 * Textfeld zur Anzeige der aktuellen Simulationsvariante
	 */
	private Text _textAktuelleSimulationsvariante = null;

	private Label _labelNeueSimulationsvariante = null;

	/**
	 * {@link Spinner} zum Einstellen der aktuellen Simulationsvariante
	 */
	private Spinner _spinnerSimulationsvariante = null;

	private Button _buttonUebernehmen = null;

	private Button _buttonUebernehmenSchliessen = null;

	private Button _buttonSchliessen = null;

	private Simulationsvariante _simulationsvariante = Simulationsvariante
			.getInstanz();

	private IZuhoererSimulationsvariante _zuhoerer = null;

	@Override
	public void createPartControl(Composite parent)
	{
		_basis = new Composite(parent, SWT.NONE);
		_basis.setLayout(new GridLayout());
		erzeugeGruppeAktuelleSimulationsvariante();
		erzeugeGruppeSimulationsvarianteWaehlen();
		erzeugeGruppeKontrolle();

		init();

		PlatformUI.getWorkbench().getHelpSystem().setHelp(_basis,
				"de.bsvrz.buv.rw.basislib.simulationsvariante");
	}

	private void init()
	{
		_textAktuelleSimulationsvariante.setText(""
				+ _simulationsvariante.getSimulationsvariante());

		if (_zuhoerer == null)
		{
			_zuhoerer = new IZuhoererSimulationsvariante()
			{
				public void aktuelleSimulationsvariante(int simulationsvariante)
				{
					_textAktuelleSimulationsvariante.setText("" + simulationsvariante);
					_textAktuelleSimulationsvariante.pack();
				}
			};
			_simulationsvariante.hinzufuegenZuhoererSimulationsvariante(_zuhoerer);
		}

	}

	/**
	 * Schliet diese View
	 * 
	 */
	private void schliessen()
	{
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.hideView(this);

		if ((_simulationsvariante != null) && (_zuhoerer != null))
		{
			_simulationsvariante.entferneZuhoererSimulationsvariante(_zuhoerer);
		}
	}

	@Override
	public void setFocus()
	{
		if (_basis != null)
			_basis.setFocus();
	}

	/**
	 * This method initializes gruppeAktuelleSimulationsvariante
	 * 
	 */
	private void erzeugeGruppeAktuelleSimulationsvariante()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_gruppeAktuelleSimulationsvariante = new Group(_basis, SWT.NONE);
		_gruppeAktuelleSimulationsvariante.setLayout(new RowLayout());
		_gruppeAktuelleSimulationsvariante.setLayoutData(gridData);
		_gruppeAktuelleSimulationsvariante.setText("Aktuelle Simulationsvariante");
		_labelAktuelleSimulationsvariante = new Label(
				_gruppeAktuelleSimulationsvariante, SWT.NONE);
		_labelAktuelleSimulationsvariante.setText("Aktuelle Simulationsvariante:");
		_textAktuelleSimulationsvariante = new Text(
				_gruppeAktuelleSimulationsvariante, SWT.BORDER);
		_textAktuelleSimulationsvariante.setEditable(false);
	}

	/**
	 * This method initializes gruppeSimulationsvarianteWaehlen
	 * 
	 */
	private void erzeugeGruppeSimulationsvarianteWaehlen()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		_gruppeSimulationsvarianteWaehlen = new Group(_basis, SWT.NONE);
		_gruppeSimulationsvarianteWaehlen.setLayoutData(gridData1);
		_gruppeSimulationsvarianteWaehlen.setLayout(gridLayout);
		_gruppeSimulationsvarianteWaehlen.setText("Simulationsvariante whlen");
		_labelNeueSimulationsvariante = new Label(
				_gruppeSimulationsvarianteWaehlen, SWT.NONE);
		_labelNeueSimulationsvariante
				.setText("Neue Simulationsvariante einstellen:");
		_spinnerSimulationsvariante = new Spinner(
				_gruppeSimulationsvarianteWaehlen, SWT.NONE);
		_spinnerSimulationsvariante.setDigits(0);
		_spinnerSimulationsvariante.setMaximum(999);

		if (_simulationsvariante.getSimulationsvariante() > 0)
		{
			_spinnerSimulationsvariante.setSelection(_simulationsvariante
					.getSimulationsvariante());
		}
	}

	/**
	 * This method initializes gruppeKontrolle
	 * 
	 */
	private void erzeugeGruppeKontrolle()
	{
		GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 3;
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		_gruppeKontrolle = new Group(_basis, SWT.NONE);
		_gruppeKontrolle.setLayoutData(gridData2);
		_gruppeKontrolle.setLayout(gridLayout1);
		_gruppeKontrolle.setText("Einstellungen bernehmen");

		/*
		 * bernehmen
		 */
		_buttonUebernehmen = new Button(_gruppeKontrolle, SWT.NONE);
		_buttonUebernehmen.setText("bernehmen");
		_buttonUebernehmen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_simulationsvariante.setSimulationsvariante(_spinnerSimulationsvariante
						.getSelection());
			}
		});

		/*
		 * bernehmen und Schlieen
		 */
		_buttonUebernehmenSchliessen = new Button(_gruppeKontrolle, SWT.NONE);
		_buttonUebernehmenSchliessen.setText("bernehmen + Schlieen");
		_buttonUebernehmenSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				_simulationsvariante.setSimulationsvariante(_spinnerSimulationsvariante
						.getSelection());
				schliessen();
			}
		});

		/*
		 * Schlieen, ohne die nderungen zu bernehmen.
		 */
		_buttonSchliessen = new Button(_gruppeKontrolle, SWT.NONE);
		_buttonSchliessen.setText("Schlieen");
		_buttonSchliessen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				schliessen();
			}
		});

	}

} // @jve:decl-index=0:visual-constraint="10,10,593,306"
