/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.statusleiste.editor;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.ViewPart;
import org.jdom.Element;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.basislib.einstellungen.dialog.DialogBenutzerWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.hilfsklassen.DefinitionenEinstellungen;
import de.bsvrz.buv.rw.basislib.statusleiste.generator.StatusLeisteGenerator;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.statusleiste.StatusLeisteZugriff;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.IZuhoererUndoStackStatusLeiste;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.UndoStackStatusLeiste;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.XmlBezeichnerStatus;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert die Sicht, mit der sich die Struktur der Statusleiste
 * bearbeiten lsst.
 * 
 * @author HofmannJ
 * 
 */
public class SichtEditorStatusleistenElemente extends ViewPart
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	public static final String ID = "de.bsvrz.buv.rw.basislib.viewEditorStatusLeistenelemente";

	private TableViewer _tableViewer = null;

	private Composite _top = null;

	private StatusLeisteVerwaltung _statusLeisteVerwaltung = null;

	private ZugZuhoererStatusLeistenElementXML _zugZuhoererStatusLeistenElementXML;

	private TabellenFallAdapterElementStatusleiste _tabellenFallAdapterElementStatusleiste;

	/**
	 * {@link Action} zum Speichern in eine Datei.
	 */
	private IAction _aktionSpeichern = null;

	/**
	 * {@link Action} zum Laden aus einer Datei.
	 */
	private IAction _aktionLaden = null;

	/**
	 * {@link Action} zum Aktualisieren der Darstellung.
	 */
	private IAction _aktionAktualisieren = null;

	private Element _elementWurzel = null;

	private IAction _aktionLadenBenutzerspezifischDav = null;

	private IAction _aktionSpeichernBenutzerspezifischDav = null;

	private IAction _aktionLadenAngemeldeterBenutzerDav = null;

	private IAction _aktionSpeichernAngemeldeterBenutzerDav = null;

	private IAction _aktionLadenAllgemeinDav = null;

	private IAction _aktionSpeichernAllgemeinDav = null;

	private IAction _aktionLadenBenutzerspezifischLokal = null;

	private IAction _aktionSpeichernBenutzerspezifischLokal = null;

	private IAction _aktionLadenAngemeldeterBenutzerLokal = null;

	private IAction _aktionSpeichernAngemeldeterBenutzerLokal = null;

	private IAction _aktionLadenAllgemeinLokal = null;

	private IAction _aktionSpeichernAllgemeinLokal = null;

	/**
	 * Handler fr die globale Aktion <code>UNDO</code>.
	 */
	private Action _aktionHandlerUndo = null;

	/**
	 * Handler fr die globale Aktion <code>REDO</code>.
	 */
	private Action _aktionHandlerRedo = null;

	/**
	 * Listener fr den UNDO-Stack
	 */
	private IZuhoererUndoStackStatusLeiste _zuhoererUndoStack;

	@Override
	public void createPartControl(Composite parent)
	{
		parent.setLayout(new FillLayout());
		_top = new Composite(parent, SWT.NONE);
		_top.setLayout(new FillLayout());

		// this._parent = parent;

		init();

		erzeugeTabelle();
		initialisiereTabelle();
		erzeugeBasisAktionen();
		erzeugeSichtSymbolleiste();
		erzeugeSichtMenu();
		verankereKontextMenu();

		registriereUndoListener();
		registriereUndoRedoHandler();

		PlatformUI.getWorkbench().getHelpSystem().setHelp(_top,
				"de.bsvrz.buv.rw.basislib.statusleisteneditor");
	}

	/**
	 * Erzeugt die Basis {@link Action}s dieser View
	 */
	private void erzeugeBasisAktionen()
	{
		/*
		 * Laden aus Datei
		 */
		_aktionLaden = new Action()
		{
			public void run()
			{
				aktionLaden();
			}
		};
		_aktionLaden.setText("Laden");
		_aktionLaden.setToolTipText("Ldt eine XML-Datei");
		_aktionLaden.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Speichern in Datei
		 */
		_aktionSpeichern = new Action()
		{
			public void run()
			{
				aktionSpeichern();
			}
		};
		_aktionSpeichern.setText("Speichern");
		_aktionSpeichern.setToolTipText("Speichern in XML-Datei");
		_aktionSpeichern.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Aktualisieren
		 */
		_aktionAktualisieren = new Action()
		{
			public void run()
			{
				aktionAktualisieren();
			}
		};
		_aktionAktualisieren.setText("Aktualisieren");
		_aktionAktualisieren.setToolTipText("Aktualisieren");
		_aktionAktualisieren.setImageDescriptor(Activator
				.getImageDescriptor("icons/allgemein/refresh.gif"));

		/*
		 * Benutzerspezifische Statusleiste laden Netzweit
		 */
		_aktionLadenBenutzerspezifischDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionLadenBenutzerspezifischDav");
				aktionLadenBenutzerspezifischNetzweit();
			}
		};
		_aktionLadenBenutzerspezifischDav
				.setText("Laden einer benutzerspezifischen Statusleiste aus dem DaV");
		_aktionLadenBenutzerspezifischDav
				.setToolTipText("Ldt eine benutzerspezifische Statusleistenkonfiguration aus dem DaV in den Editor.");
		_aktionLadenBenutzerspezifischDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Benutzerspezifische Statusleiste speichern Netzweit
		 */
		_aktionSpeichernBenutzerspezifischDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionSpeichernBenutzerspezifischDav");
				aktionSpeichernBenutzerspezifischNetzweit();
			}
		};
		_aktionSpeichernBenutzerspezifischDav
				.setText("Speichern einer benutzerspezifischen Statusleiste im DaV");
		_aktionSpeichernBenutzerspezifischDav
				.setToolTipText("Speichert die benutzerspezifische Statusleistenkonfiguration des Editors im DaV.");
		_aktionSpeichernBenutzerspezifischDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Benutzerspezifische Statusleiste laden Lokal
		 */
		_aktionLadenBenutzerspezifischLokal = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionLadenBenutzerspezifischDav");
				aktionLadenBenutzerspezifischLokal();
			}
		};
		_aktionLadenBenutzerspezifischLokal
				.setText("Laden einer benutzerspezifischen Statusleiste aus den Lokalen Einstellungen");
		_aktionLadenBenutzerspezifischLokal
				.setToolTipText("Ldt eine benutzerspezifische Statusleistenkonfiguration aus den lokalen Einstellungen in den Editor.");
		_aktionLadenBenutzerspezifischLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Benutzerspezifische Statusleiste speichern Netzweit
		 */
		_aktionSpeichernBenutzerspezifischLokal = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionSpeichernBenutzerspezifischDav");
				aktionSpeichernBenutzerspezifischLokal();
			}
		};
		_aktionSpeichernBenutzerspezifischLokal
				.setText("Speichern einer benutzerspezifischen Statusleiste in den lokalen Einstellungen");
		_aktionSpeichernBenutzerspezifischLokal
				.setToolTipText("Speichert die benutzerspezifische Statusleistenkonfiguration des Editors in den lokalen Einstellungen.");
		_aktionSpeichernBenutzerspezifischLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Angemeldeter Benutzer Statusleiste laden Netzweit
		 */
		_aktionLadenAngemeldeterBenutzerDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionLadenAngemeldeterBenutzerDav");
				aktionLadenAngemeldeterBenutzerNetzweit();
			}
		};
		_aktionLadenAngemeldeterBenutzerDav
				.setText("Laden der Statusleiste des angemeldeten Benutzers aus dem DaV");
		_aktionLadenAngemeldeterBenutzerDav
				.setToolTipText("Ldt die Statusleistenkonfiguration des angemeldeten Benutzers aus dem DaV in den Editor.");
		_aktionLadenAngemeldeterBenutzerDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Angemeldeter Benutzer Statusleiste speichern Netzweit
		 */
		_aktionSpeichernAngemeldeterBenutzerDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionSpeichernAngemeldeterBenutzerDav");
				aktionSpeichernAngemeldeterBenutzerNetzweit();
			}
		};
		_aktionSpeichernAngemeldeterBenutzerDav
				.setText("Speichern der Statusleiste des angemeldeten Benutzers im DaV");
		_aktionSpeichernAngemeldeterBenutzerDav
				.setToolTipText("Speichert die Statusleistenkonfiguration des Editors des angemeldeten Benutzers im DaV.");
		_aktionSpeichernAngemeldeterBenutzerDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Angemeldeter Benutzer Statusleiste laden Lokal
		 */
		_aktionLadenAngemeldeterBenutzerLokal = new Action()
		{
			public void run()
			{
				aktionLadenAngemeldeterBenutzerLokal();
			}
		};
		_aktionLadenAngemeldeterBenutzerLokal
				.setText("Laden der Statusleiste des angemeldeten Benutzers aus den lokalen Einstellungen.");
		_aktionLadenAngemeldeterBenutzerLokal
				.setToolTipText("Ldt die Statusleistenkonfiguration des angemeldeten Benutzers aus den lokalen Einstellungen in den Editor.");
		_aktionLadenAngemeldeterBenutzerLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Angemeldeter Benutzer Statusleiste speichern Lokal
		 */
		_aktionSpeichernAngemeldeterBenutzerLokal = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionSpeichernAngemeldeterBenutzerDav");
				aktionSpeichernAngemeldeterBenutzerLokal();
			}
		};
		_aktionSpeichernAngemeldeterBenutzerLokal
				.setText("Speichern der Statusleiste des angemeldeten Benutzers in den lokalen Einstellungen");
		_aktionSpeichernAngemeldeterBenutzerLokal
				.setToolTipText("Speichert die Statusleistenkonfiguration des Editors des angemeldeten Benutzers in den lokalen Einstellungen.");
		_aktionSpeichernAngemeldeterBenutzerLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Laden der netzweiten, allgemeinen Statusleiste
		 */
		_aktionLadenAllgemeinDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionLadenAllgemeinDav");
				aktionLadenAllgemeinNetzweit();
			}
		};
		_aktionLadenAllgemeinDav
				.setText("Laden der Statusleiste aus den allgemeinen, netzweiten Einstellungen des DaV");
		_aktionLadenAllgemeinDav
				.setToolTipText("Ldt die Statusleiste aus den allgemeinen, netzweiten Einstellungen des DaV.");
		_aktionLadenAllgemeinDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Speichern der netzweiten, allgemeinen Statusleiste
		 */
		_aktionSpeichernAllgemeinDav = new Action()
		{
			public void run()
			{
				// System.out.println("_aktionSpeichernAllgemeinDav");
				aktionSpeichernAllgemeinNetzweit();
			}
		};
		_aktionSpeichernAllgemeinDav
				.setText("Speichern der Statusleiste, allgemein, netzweit im DaV");
		_aktionSpeichernAllgemeinDav
				.setToolTipText("Speichert die im Editor dargstellte Statusleiste in den allgemeinen, netzweiten Einstellungen im DaV.");
		_aktionSpeichernAllgemeinDav.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

		/*
		 * Laden der lokalen, allgemeinen Statusleiste
		 */
		_aktionLadenAllgemeinLokal = new Action()
		{
			public void run()
			{
				aktionLadenAllgemeinLokal();
			}
		};
		_aktionLadenAllgemeinLokal
				.setText("Laden der Statusleiste aus den allgemeinen, lokalen Einstellungen");
		_aktionLadenAllgemeinLokal
				.setToolTipText("Ldt die Statusleiste aus den allgemeinen, lokalen Einstellungen.");
		_aktionLadenAllgemeinLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/laden_16x16.png"));

		/*
		 * Speichern der lokalen, allgemeinen Statusleiste
		 */
		_aktionSpeichernAllgemeinLokal = new Action()
		{
			public void run()
			{
				aktionSpeichernAllgemeinLokal();
			}
		};
		_aktionSpeichernAllgemeinLokal
				.setText("Speichern der Statusleiste, allgemein, lokal.");
		_aktionSpeichernAllgemeinLokal
				.setToolTipText("Speichert die im Editor dargstellte Statusleiste in den allgemeinen, lokalen Einstellungen.");
		_aktionSpeichernAllgemeinLokal.setImageDescriptor(Activator
				.getImageDescriptor("icons/menubar/speichern_unter_16x16.png"));

	}

	/**
	 * Erzeugt das Men der View
	 */
	private void erzeugeSichtMenu()
	{
		IMenuManager manager = getViewSite().getActionBars().getMenuManager();
		manager.add(_aktionLaden);
		manager.add(_aktionSpeichern);

		manager.add(new Separator());

		MenuManager mmLadenDav = new MenuManager("Laden aus DaV");
		mmLadenDav.add(_aktionLadenBenutzerspezifischDav);
		mmLadenDav.add(_aktionLadenAngemeldeterBenutzerDav);
		mmLadenDav.add(_aktionLadenAllgemeinDav);
		manager.add(mmLadenDav);

		MenuManager mmSpeichernDav = new MenuManager("Speichern in DaV");
		mmSpeichernDav.add(_aktionSpeichernBenutzerspezifischDav);
		mmSpeichernDav.add(_aktionSpeichernAngemeldeterBenutzerDav);
		mmSpeichernDav.add(_aktionSpeichernAllgemeinDav);
		manager.add(mmSpeichernDav);

		MenuManager mmLadenLokal = new MenuManager(
				"Laden aus lokalen Einstellungen");
		mmLadenLokal.add(_aktionLadenBenutzerspezifischLokal);
		mmLadenLokal.add(_aktionLadenAngemeldeterBenutzerLokal);
		mmLadenLokal.add(_aktionLadenAllgemeinLokal);
		manager.add(mmLadenLokal);

		MenuManager mmSpeichernLokal = new MenuManager(
				"Speichern in lokalen Einstellungen");
		mmSpeichernLokal.add(_aktionSpeichernBenutzerspezifischLokal);
		mmSpeichernLokal.add(_aktionSpeichernAngemeldeterBenutzerLokal);
		mmSpeichernLokal.add(_aktionSpeichernAllgemeinLokal);
		manager.add(mmSpeichernLokal);

		manager.add(new Separator());
		manager.add(_aktionAktualisieren);
	}

	/**
	 * Erzeugt die Symbolleiste der View
	 */
	private void erzeugeSichtSymbolleiste()
	{
		IToolBarManager toolBarManager = getViewSite().getActionBars()
				.getToolBarManager();

		toolBarManager.add(_aktionLaden);
		toolBarManager.add(_aktionSpeichern);
		toolBarManager.add(_aktionAktualisieren);

		IAction testAction = new Action()
		{
			public void run()
			{
				ICoolBarManager manager = ElementRegistrierung.getInstanz()
						.getCoolBarManager();
				manager.removeAll();
				manager.update(true);

				IMenuManager menuManager = ElementRegistrierung.getInstanz()
						.getMenuManager();
				menuManager.removeAll();
				menuManager.update(true);

			}
		};

		testAction.setText("Leeren");
		testAction.setToolTipText("Leeren");
		testAction.setImageDescriptor(Activator
				.getImageDescriptor("icons/actions/editdelete16x16.png"));
		// toolBarManager.add(testAction);

	}

	// Initialisierung
	private void init()
	{
		_statusLeisteVerwaltung = StatusLeisteVerwaltung.getInstanz();
	}

	/**
	 * Initialisiert die Tabelle
	 * 
	 */
	private void initialisiereTabelle()
	{

		// _tableViewer.setInput(StatusLeisteVerwaltung.getInstanz());
		// _tableViewer.setInput(_statusLeisteVerwaltung
		// .getListeXmlStatusleisteElemente());

		_elementWurzel = _statusLeisteVerwaltung.getWurzel();
		_tableViewer.setInput(_elementWurzel);
	}

	/**
	 * Erzeugt die Tabelle
	 * 
	 */
	private void erzeugeTabelle()
	{
		_tableViewer = new TableViewer(_top, SWT.FULL_SELECTION | SWT.MULTI);

		TableColumn tc1 = new TableColumn(_tableViewer.getTable(), SWT.CENTER);
		tc1.setText("ID:");
		tc1.setWidth(200);

		TableColumn tc2 = new TableColumn(_tableViewer.getTable(), SWT.LEFT);
		tc2.setText("Bezeichnung:");
		tc2.setWidth(300);

		_tableViewer.getTable().setHeaderVisible(true);
		_tableViewer.getTable().setLinesVisible(true);

		_tableViewer
				.setContentProvider(new StatusLeistenXmlElementeInhaltsVersorger());
		_tableViewer
				.setLabelProvider(new StatusleistenXmlElementeDarstellungsVersorger());

		int ops = DND.DROP_MOVE;
		_zugZuhoererStatusLeistenElementXML = new ZugZuhoererStatusLeistenElementXML(
				_tableViewer);
		Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
		_tableViewer.addDragSupport(ops, dragTransfers,
				_zugZuhoererStatusLeistenElementXML);
		_tabellenFallAdapterElementStatusleiste = new TabellenFallAdapterElementStatusleiste(
				_tableViewer);
		_tableViewer.addDropSupport(ops, dragTransfers,
				_tabellenFallAdapterElementStatusleiste);

	}

	/**
	 * ffnet einen Dateidialog zum Laden einer Definitionsdatei
	 * 
	 */
	private void aktionLaden()
	{
		FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
				SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.OPEN);
		dialog.setFilterNames(new String[] { "XML-Dateien" });
		dialog.setFilterExtensions(new String[] { "*.xml" });
		String dateiPfad = dialog.open();
		if (dateiPfad != null)
		{
			_statusLeisteVerwaltung.leseSortierungAusXmlDatei(dateiPfad);
			initialisiereTabelle();
		}
	}

	/**
	 * Speicher die Definitionsdatei
	 * 
	 */
	private void aktionSpeichern()
	{
		FileDialog dialog = new FileDialog(Display.getCurrent().getActiveShell(),
				SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM | SWT.RESIZE | SWT.SAVE);
		dialog.setFilterNames(new String[] { "XML-Dateien" });
		dialog.setFilterExtensions(new String[] { "*.xml" });
		String dateiPfad = dialog.open();
		if (dateiPfad != null)
		{
			StatusLeisteVerwaltung.getInstanz().schreibeSortierungInXmlString();
			StatusLeisteVerwaltung.getInstanz().schreibeSortierungInXmlDatei(
					dateiPfad, true);
		}
	}

	/**
	 * Ldt die benutzerspezifische Statusleistenstruktur fr einen auszuwhlenden
	 * Benutzer aus den netzweiten Einstellungen.
	 */
	private void aktionLadenBenutzerspezifischNetzweit()
	{
		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());

		String benutzer = bw.oeffne();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);
			if (einstellungen != null)
			{
				String xmlStruktur = einstellungen
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
				initialisiereTabelle();
			}

		}
	}

	/**
	 * Sichert die benutzerspezifische Statusleistenstruktur fr einen
	 * auszuwhlenden Benutzer in den netzweiten Eisntellungen.
	 */
	private void aktionSpeichernBenutzerspezifischNetzweit()
	{
		boolean gespeichert = false;

		String grund = "";

		DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
				.getWorkbench().getDisplay().getActiveShell());

		String benutzer = bw.oeffne();

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungenSpeicherBenzutzerBestimmtNetzweit(benutzer);

			if (einstellungen != null)
			{

				String xmlStruktur = _statusLeisteVerwaltung
						.schreibeSortierungInXmlString();

				if (einstellungen
						.editiereParameter(
								DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
								xmlStruktur))
				{
				} else
				{
					einstellungen.hinzufuegenParameter(
							DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
							xmlStruktur);
				}

				gespeichert = esw.sichereEinstellungenSpeicherBenutzerBestimmtNetzweit(
						benutzer, einstellungen);
			} else
			{
				grund = grund + "Einstellungenspeicher nicht Verfgbar ";
			}
		} else
		{
			grund = grund + "Kein Benutzer angegeben, ";
		}

		if (!gespeichert)
		{
			String msg = "Statusleistenstruktur konnte nicht gespeichert werden. Grund: "
					+ grund;
			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg));

		}
	}

	/**
	 * Ldt die benutzerspezifische Statusleistenstruktur fr einen auszuwhlenden
	 * Benutzer aus den lokalen Einstellungen.
	 */
	protected void aktionLadenBenutzerspezifischLokal()
	{
		String benutzer = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			InputDialog inputDialog = new InputDialog(PlatformUI.getWorkbench()
					.getDisplay().getActiveShell(), "Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte geben Sie\n"
							+ "den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);

			if (einstellungen != null)
			{
				String xmlStruktur = einstellungen
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
				initialisiereTabelle();
			}

		}
	}

	/**
	 * Sichert die benutzerspezifische Statusleistenstruktur fr einen
	 * auszuwhlenden Benutzer in den netzweiten Eisntellungen.
	 */
	protected void aktionSpeichernBenutzerspezifischLokal()
	{
		boolean gespeichert = false;
		String benutzer = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogBenutzerWaehler bw = new DialogBenutzerWaehler(PlatformUI
					.getWorkbench().getDisplay().getActiveShell());
			benutzer = bw.oeffne();
		} else
		{
			InputDialog inputDialog = new InputDialog(
					PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					"Benutzernamen eingeben",
					"Es existiert keine Verbindung zum DaV, bitte geben Sie den Namen des Benutzers manuell ein:",
					"Hier den Benutzernamen eingeben", null);
			if (inputDialog.open() == Dialog.OK)
			{
				if (inputDialog.getValue() != null)
				{
					benutzer = inputDialog.getValue();
				}
			}
		}

		String grund = "";

		if (benutzer != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungenSpeicherBenutzerBestimmtLokal(benutzer);

			if (einstellungen != null)
			{

				String xmlStruktur = _statusLeisteVerwaltung
						.schreibeSortierungInXmlString();

				if (einstellungen
						.editiereParameter(
								DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
								xmlStruktur))
				{
				} else
				{
					einstellungen.hinzufuegenParameter(
							DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
							xmlStruktur);
				}

				gespeichert = esw.sichereEinstellungenSpeicherBenutzerBestimmtLokal(
						benutzer, einstellungen);
			} else
			{
				grund = grund + " kein Einstellungenspeicher Verfgbar.";
			}
		} else
		{
			grund = grund + "Kein Benutzername angegeben, ";
		}

		if (!gespeichert)
		{
			String msg = getClass().getName()
					+ "::aktionSpeichernBenutzerspezifischLokal "
					+ "Die benutzerspezifischen, lokalen Einstellungen "
					+ "konnten nicht gespeichert werde. Grund: " + grund;

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
		}
	}

	/**
	 * Ldt die benutzerspezifische Statusleistenstruktur fr den angemeldeten
	 * Benutzer aus den netzweiten Einstellungen.
	 */
	private void aktionLadenAngemeldeterBenutzerNetzweit()
	{
		String name = BenutzerAngemeldet.getInstanz().getName();
		if (name != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);

			if (einstellungen != null)
			{
				String xmlStruktur = einstellungen
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
				initialisiereTabelle();
			}
		}
	}

	/**
	 * Sichert die benutzerspezifische Statusleistenstruktur fr den angemeldeten
	 * Benutzer in den netzweiten Eisntellungen.
	 */
	private void aktionSpeichernAngemeldeterBenutzerNetzweit()
	{
		boolean gespeichert = false;

		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.benutzerNetzweit);

		if (einstellungen != null)
		{
			String xmlStruktur = _statusLeisteVerwaltung
					.schreibeSortierungInXmlString();

			if (einstellungen.editiereParameter(
					DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur))
			{
			} else
			{
				einstellungen
						.hinzufuegenParameter(
								DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
								xmlStruktur);
			}

			gespeichert = esw.sichereSpeicherBenutzerNetzweit();

		}

		if (gespeichert)
		{
		} else
		{
			_debug.fine("Nicht gespeichert");
		}
	}

	/**
	 * Ldt die benutzerspezifische Statusleistenstruktur fr den angemeldeten
	 * Benutzer aus den lokalen Einstellungen.
	 */
	protected void aktionLadenAngemeldeterBenutzerLokal()
	{
		String name = BenutzerAngemeldet.getInstanz().getName();
		if (name != null)
		{
			EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			EinstellungenSpeicher einstellungen = esw
					.getEinstellungen(EinstellungsArt.benutzerNetzweit);

			if (einstellungen != null)
			{
				String xmlStruktur = einstellungen
						.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

				_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
				initialisiereTabelle();
			}
		}
	}

	/**
	 * Sichert die benutzerspezifische Statusleistenstruktur fr den angemeldeten
	 * Benutzer in den lokalen Einstellungen.
	 */
	protected void aktionSpeichernAngemeldeterBenutzerLokal()
	{
		boolean gespeichert = false;

		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.benutzerNetzweit);

		if (einstellungen != null)
		{
			String xmlStruktur = _statusLeisteVerwaltung
					.schreibeSortierungInXmlString();

			if (einstellungen.editiereParameter(
					DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur))
			{
			} else
			{
				einstellungen
						.hinzufuegenParameter(
								DefinitionenEinstellungen.paramXmlStrukturStatusleiste,
								xmlStruktur);
			}

			gespeichert = esw.sichereSpeicherBenutzerNetzweit();

		}

		if (gespeichert)
		{
		} else
		{
			_debug.fine("Nicht gespeichert");
		}
	}

	/**
	 * Ldt die allgemeine Struktur der Statusleiste aus den netzweiten
	 * Einstellungen.
	 */
	private void aktionLadenAllgemeinNetzweit()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen != null)
		{
			String xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

			_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
			initialisiereTabelle();
		}
	}

	/**
	 * Sichert die allgemeine Struktur der Statusleiste in den netzweiten
	 * Einstellungen.
	 */
	private void aktionSpeichernAllgemeinNetzweit()
	{
		String xmlStruktur = _statusLeisteVerwaltung
				.schreibeSortierungInXmlString();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinNetzweit);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur);
		}

		esw.sichereSpeicherAllgemeinNetzweit();
	}

	/**
	 * Ldt die allgemeine Struktur der Statusleiste aus den lokalen
	 * Einstellungen.
	 */
	protected void aktionLadenAllgemeinLokal()
	{
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen != null)
		{
			String xmlStruktur = einstellungen
					.getParameter(DefinitionenEinstellungen.paramXmlStrukturStatusleiste);

			_statusLeisteVerwaltung.leseSortierungAusXmlString(xmlStruktur);
			initialisiereTabelle();
		}
	}

	/**
	 * Sichert die allgemeine Struktur der Statusleiste in den lokalen
	 * Einstellungen.
	 */
	protected void aktionSpeichernAllgemeinLokal()
	{
		String xmlStruktur = _statusLeisteVerwaltung
				.schreibeSortierungInXmlString();
		EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		EinstellungenSpeicher einstellungen = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);

		if (einstellungen.editiereParameter(
				DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur))
		{
		} else
		{
			einstellungen.hinzufuegenParameter(
					DefinitionenEinstellungen.paramXmlStrukturStatusleiste, xmlStruktur);
		}

		esw.sichereSpeicherAllgemeinLokal();
	}

	/**
	 * Aktualisieren
	 * 
	 */
	private void aktionAktualisieren()
	{
		initialisiereTabelle();
		StatusLeisteZugriff.getInstanz().entferneAlleStatusZeilenElemente();
		StatusLeisteGenerator.getInstanz().generiereStatusLeiste();
		_statusLeisteVerwaltung.aktualisieren();
	}

	/**
	 * Lscht das in der Tabelle selektierte Element.
	 * 
	 */
	private void aktionSelektiertesElementLoeschen()
	{
		IStructuredSelection is = (IStructuredSelection) _tableViewer
				.getSelection();
		Object erstesElement = is.getFirstElement();
		if (erstesElement instanceof Element)
		{
			Element loeschElement = (Element) erstesElement;

			// loeschElement.detach();
			_statusLeisteVerwaltung.loescheXmlElement(loeschElement);

			_tableViewer.refresh();
		}

	}

	/**
	 * Verankert das Kontextmen
	 * 
	 */
	private void verankereKontextMenu()
	{
		_debug.fine(getClass().getName()
				+ "::verankereContextMenu => Verankere Kontextmen");

		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{

				_debug.fine(getClass().getName()
						+ "::verankereKontextMenu \n\t => menuaboutToShow\n");

				fuelleKontextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_tableViewer.getControl());
		_tableViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _tableViewer);
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *          Der {@link IMenuManager}, der die Struktur hinter dem Kontextmen
	 *          aufnimmt.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{
		/**
		 * Liste der Actions, die im Kontextmen dargestellt werden sollen
		 */
		List<IAction> actions = new LinkedList<IAction>();

		// Action lschen
		Action actionLoeschen = new Action()
		{
			public void run()
			{
				Object firstElement = ((IStructuredSelection) _tableViewer
						.getSelection()).getFirstElement();
				if (firstElement instanceof Element)
				{
					aktionSelektiertesElementLoeschen();
				}
			}
		};
		actionLoeschen.setText("Lschen");
		actionLoeschen.setToolTipText("Lscht das Element.");
		actionLoeschen.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		actions.add(actionLoeschen);

		// Hinzufgen der Actions zum Menmanager
		Iterator<IAction> it = actions.iterator();
		int i = -1;
		while (it.hasNext())
		{
			i++;
			manager.add(it.next());
			if (i == 2)
			{
				manager.add(new Separator());
			}
		}
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	/**
	 * Registriert die {@link IZuhoererUndoStackStatusLeiste} vie der Klasse
	 * {@link UndoStackStatusLeiste} der hier verwendeten Instanz der Klasse
	 * {@link StatusLeisteVerwaltung}.<br>
	 * <br>
	 * Sorgt fr das aktivieren bzw. deaktivieren der globalen Actions Rckgngig
	 * machen und Wiederholen.
	 */
	private void registriereUndoListener()
	{
		if (_zuhoererUndoStack == null)
		{
			_zuhoererUndoStack = new IZuhoererUndoStackStatusLeiste()
			{

				@Override
				public void rueckgaengigmachenMoeglich(boolean moeglich)
				{
					if (_aktionHandlerUndo != null)
					{
						if (moeglich)
						{
							_aktionHandlerUndo.setEnabled(true);
						} else
						{
							_aktionHandlerUndo.setEnabled(false);
						}
					}

				}

				@Override
				public void wiederholenMoeglich(boolean moeglich)
				{
					if (_aktionHandlerRedo != null)
					{
						if (moeglich)
						{
							_aktionHandlerRedo.setEnabled(true);
						} else
						{
							_aktionHandlerRedo.setEnabled(false);
						}
					}
				}

			};

			_statusLeisteVerwaltung.getUndoStack().addZuhoererUndoStack(
					_zuhoererUndoStack);
		}
	}

	/**
	 * Registriert die {@link Action}-Handler fr die globalen Aktionen
	 * Rckgngig machen und Wiederholen.
	 */
	private void registriereUndoRedoHandler()
	{
		IActionBars bars = getViewSite().getActionBars();

		/*
		 * UNDO
		 */
		_aktionHandlerUndo = new Action()
		{
			public void run()
			{
				aktionUndo();
			}
		};
		bars.setGlobalActionHandler(ActionFactory.UNDO.getId(), _aktionHandlerUndo);

		/*
		 * REDO
		 */
		_aktionHandlerRedo = new Action()
		{
			public void run()
			{
				aktionRedo();
			}
		};
		bars.setGlobalActionHandler(ActionFactory.REDO.getId(), _aktionHandlerRedo);
	}

	/**
	 * Fhrt die notwendigen Aktivitten hinter der Funktion UNDO aus.
	 */
	private void aktionUndo()
	{
		if (_statusLeisteVerwaltung != null)
		{
			if (_statusLeisteVerwaltung.aktionUndo())
			{
				initialisiereTabelle();
				_tableViewer.refresh();
			}
		}
	}

	/**
	 * Fhrt die notwendigen Aktivitten hinter der Funktion REDO aus.
	 */
	private void aktionRedo()
	{
		if (_statusLeisteVerwaltung != null)
		{
			if (_statusLeisteVerwaltung.aktionRedo())
			{
				initialisiereTabelle();
				_tableViewer.refresh();
			}
		}
	}

	@Override
	public void setFocus()
	{
		if (_top != null)
			_top.setFocus();

		initialisiereTabelle();

	}

	private class StatusLeistenXmlElementeInhaltsVersorger implements
			IStructuredContentProvider
	{
		public Object[] getElements(Object inputElement)
		{
			Object[] returnElemente = null;

			if (inputElement instanceof Element)
			{
				Element eingabeElement = (Element) inputElement;
				if (eingabeElement.getName().equals(
						XmlBezeichnerStatus.elementStatusleiste))
				{
					@SuppressWarnings("unchecked")
					List elementElemente = eingabeElement.getChildren();
					returnElemente = new Object[elementElemente.size()];
					int i = -1;

					@SuppressWarnings("unchecked")
					Iterator it = elementElemente.iterator();
					while (it.hasNext())
					{
						i++;
						Object tempElement = it.next();

						if (tempElement instanceof Element)
						{
							Element elementElement = (Element) tempElement;
							returnElemente[i] = elementElement;
						} else
						{
							returnElemente[i] = null;
						}
					}
				}
			} else
			{
				_debug.fine(getClass().getName() + "::getElements => KEINE Eingabe!");
				returnElemente = new Object[0];
			}

			return returnElemente;
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class StatusleistenXmlElementeDarstellungsVersorger implements
			ITableLabelProvider
	{

		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		public String getColumnText(Object element, int columnIndex)
		{
			String returnString;
			if (element instanceof Element && element != null)
			{
				Element elementElement = (Element) element;
				if (!elementElement.getName()
						.equals(XmlBezeichnerStatus.elementElement))
				{
					returnString = "Der XML-Elementtyp " + elementElement.getName()
							+ " wird nicht untersttzt.";
				} else
				{
					// ID
					if (columnIndex == 0)
					{
						returnString = elementElement
								.getAttributeValue(XmlBezeichnerStatus.attributId);
					}
					// Bezeichnung
					else if (columnIndex == 1)
					{
						returnString = elementElement
								.getAttributeValue(XmlBezeichnerStatus.attributBezeichnung);
					}
					// Sonst
					else
					{
						returnString = "Fr diese Spalte findet keine Versorgung statt.";
					}

				}

			} else
			{
				returnString = "Nicht untersttztes Element vom Typ "
						+ element.getClass().getName() + ".";
			}

			return returnString;
		}

		public void addListener(ILabelProviderListener listener)
		{
		}

		public void dispose()
		{
		}

		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

}
