/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;

public class DialogFilterEditor extends Dialog
{

	private static final String _pIdx = "Idx:";
	private static final String _pTitel = "Titel der Spalte:";
	private static final String _pRegEx = "Regulrer Ausdruck:";

	private Shell _shell = null;
	private Group _gruppeFilter = null;
	private Composite _bereichUnten = null;
	private Button _schaltflaecheUebernehmen = null;
	private Button _schaltflaecheBearbeiten = null;
	private Button _schaltflaecheAbbrechen = null;
	private Button _schaltflaecheHilfe = null;
	private Table _tabelleUebersicht = null;
	private TableViewer _tableViewer = null;

	private TabellenFilter _tabellenFilter = null;
	private boolean _bearbeitet = false;

	/**
	 * Konstruktor dieser Klasse.
	 * 
	 * @param parent
	 *          Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *          soll.
	 */
	public DialogFilterEditor(Shell parent)
	{
		super(parent, 0);
	}

	/**
	 * ffnet diesen Dialog zur Bearbeitung eines {@link TabellenFilter}s.
	 * 
	 * @param tabellenFilter
	 *          {@link TabellenFilter} zur Bearbeitung. Dieser wird gecloned, so
	 *          dass das Original nicht verndert wird.
	 * @return Liefert den bearbeiteten {@link TabellenFilter} zurck oder
	 *         <code>null</code>, wenn keine Bearbeitung statt gefunden hat.
	 */
	public TabellenFilter oeffne(TabellenFilter tabellenFilter)
	{
		/*
		 * Tiefe Kopie des TabellenFilters erzeugen
		 */
		if (tabellenFilter != null)
		{
			this._tabellenFilter = (TabellenFilter) tabellenFilter.clone();
		}

		_shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		_shell.setText("Filter Editor");

		_shell.setImage(Activator.getImageDescriptor(
				"icons/sonstiges/text_editor_16x16.png").createImage());

		erzeugeDarstellung();

		if (_tabellenFilter != null)
		{
			_tableViewer.setInput(_tabellenFilter);

			CellEditor[] editors = new CellEditor[3];
			editors[0] = null;
			editors[1] = null;
			editors[2] = new TextCellEditor(_tabelleUebersicht);
			_tableViewer.setCellEditors(editors);
			_tableViewer.setCellModifier(new CellMod());

		}

		_shell.setSize(new Point(400, 300));
		_shell.open();

		Display display = getParent().getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		if (_bearbeitet)
		{
			return _tabellenFilter;
		} else
		{
			return null;
		}
	}

	/**
	 * Erzeugung der Darstellung
	 */
	private void erzeugeDarstellung()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;

		erzeugeGruppeFilter();
		_shell.setLayout(gridLayout);
		createBereichUnten();

	}

	/**
	 * Erzeugung von gruppeFilter
	 * 
	 */
	private void erzeugeGruppeFilter()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_gruppeFilter = new Group(_shell, SWT.NONE);
		_gruppeFilter.setLayout(new GridLayout());
		_gruppeFilter.setLayoutData(gridData);
		_gruppeFilter.setText("Filter");
		_tabelleUebersicht = new Table(_gruppeFilter, SWT.FULL_SELECTION
				| SWT.BORDER);
		_tabelleUebersicht.setHeaderVisible(true);
		_tabelleUebersicht.setLayoutData(gridData2);
		_tabelleUebersicht.setLinesVisible(true);
		_tableViewer = new TableViewer(_tabelleUebersicht);

		/*
		 * Spalten erzeugen
		 */
		TableColumn tcIndex = new TableColumn(_tabelleUebersicht, SWT.CENTER);
		tcIndex.setText(_pIdx);
		tcIndex.setWidth(35);

		TableColumn tcSpalte = new TableColumn(_tabelleUebersicht, SWT.CENTER);
		tcSpalte.setText(_pTitel);
		tcSpalte.setWidth(120);

		TableColumn tcRegEx = new TableColumn(_tabelleUebersicht, SWT.CENTER);
		tcRegEx.setText(_pRegEx);
		tcRegEx.setWidth(200);

		/*
		 * ColumnProperties setzen
		 */
		_tableViewer.setColumnProperties(new String[] { _pIdx, _pTitel, _pRegEx });

		ContentProvider cp = new ContentProvider();
		_tableViewer.setContentProvider(cp);

		LabelProvider lp = new LabelProvider();
		_tableViewer.setLabelProvider(lp);

		_tableViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{
			public void selectionChanged(SelectionChangedEvent event)
			{
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Object firstElement = selection.getFirstElement();
				if (firstElement instanceof FilterAusdruck)
				{
					// System.out.println("YEHA");
				}
			}

		});

	}

	/**
	 * Aktion hinter der Schaltflche bernehmen. Das Flag
	 * {@link DialogFilterEditor#_bearbeitet} wird auf <code>true</code>
	 * gesetzt.
	 */
	private void aktionUebernehmen()
	{
		_bearbeitet = true;
		aktionSchliessen();
	}

	/**
	 * Aktion hinter der Schaltflche Schlieen. Das
	 * {@link DialogFilterEditor#_bearbeitet} wird auf <code>false</code>, so
	 * dass von der {@link DialogFilterEditor#oeffne(TabellenFilter)}-Methode nur
	 * ein <code>null</code> zurckgeliefert wird.
	 */
	private void aktionSchliessen()
	{
		_shell.setVisible(false);
		_shell.dispose();
	}

	/**
	 * Aktion hinter der Schaltflche Bearbeiten. Bearbeitet den Regulren
	 * Ausdruck der aktuellen Selektion in der Tabelle.
	 */
	private void aktionBearbeiten()
	{
		ISelection selection = _tableViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if (firstElement instanceof FilterAusdruck)
			{
				FilterAusdruck filterAusdruck = (FilterAusdruck) firstElement;
				EditorRegulaererAusdruck era = new EditorRegulaererAusdruck(_shell);
				String oeffne = era.oeffne(filterAusdruck.getFilter());
				if (oeffne != null)
				{
					filterAusdruck.setFilter(oeffne);
					_bearbeitet = true;
					_tableViewer.setInput(_tabellenFilter);
				}
			}
		}
	}

	/**
	 * ffnet die Hilfe zu diesem Dialog.
	 */
	private void aktionHilfe()
	{
		String helpContextId = Activator.PLUGIN_ID + ".filterDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * Initialisierung des unteren Bereichs.
	 * 
	 */
	private void createBereichUnten()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		_bereichUnten = new Composite(_shell, SWT.NONE);
		_bereichUnten.setLayout(rowLayout);
		_bereichUnten.setLayoutData(gridData1);
		_schaltflaecheUebernehmen = new Button(_bereichUnten, SWT.NONE);
		_schaltflaecheUebernehmen.setText("bernehmen");
		_schaltflaecheUebernehmen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUebernehmen();
			}			
		});
		
		_schaltflaecheBearbeiten = new Button(_bereichUnten, SWT.NONE);
		_schaltflaecheBearbeiten.setText("Bearbeiten");
		_schaltflaecheBearbeiten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionBearbeiten();
			}			
		});
		
		_schaltflaecheAbbrechen = new Button(_bereichUnten, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");		
		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}			
		});
		
		_schaltflaecheHilfe = new Button(_bereichUnten, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}			
		});
	}

	/**
	 * Liefert einen {@link IStructuredContentProvider} fr die Tabelle zur
	 * Darstellung der {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ContentProvider implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof TabellenFilter)
			{
				TabellenFilter tabellenFilter = (TabellenFilter) inputElement;
				LinkedList<FilterAusdruck> filterAusdruecke = tabellenFilter
						.getFilterAusdruecke();
				re = new Object[filterAusdruecke.size()];
				int i = 0;
				Iterator<FilterAusdruck> it = filterAusdruecke.iterator();
				while (it.hasNext())
				{
					re[i] = it.next();
					i++;
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * Liefert einen {@link ITableLabelProvider} fr die Darstellung der Tabelle
	 * der {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "";

			if (element instanceof FilterAusdruck)
			{
				FilterAusdruck filterAusdruck = (FilterAusdruck) element;
				if (columnIndex == 0)
				{
					re = "" + filterAusdruck.getSpalte();
				} else if (columnIndex == 1)
				{
					re = "" + filterAusdruck.getBezeichnung();
				} else if (columnIndex == 2)
				{
					re = "" + filterAusdruck.getFilter();
				}
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Liefert einen {@link ICellModifier} zur Bearbeitung von Regulren
	 * Ausdrcken in der Tabelle zur Darstellung von {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CellMod implements ICellModifier
	{

		@Override
		public boolean canModify(Object element, String property)
		{
			boolean re = false;

			if (property.equals(_pRegEx))
			{
				re = true;
			}

			return re;
		}

		@Override
		public Object getValue(Object element, String property)
		{
			Object re = null;

			if (property.equals(_pRegEx))
			{
				if (element instanceof FilterAusdruck)
				{
					FilterAusdruck filterAusdruck = (FilterAusdruck) element;
					re = filterAusdruck.getFilter();
				}
			}

			return re;
		}

		@Override
		public void modify(Object element, String property, Object value)
		{
			if (property.equals(_pRegEx))
			{
				if (element instanceof TableItem)
				{
					TableItem ti = (TableItem) element;
					FilterAusdruck tfi = (FilterAusdruck) ti.getData();

					if (value instanceof String)
					{
						String wert = (String) value;

						if (!tfi.getFilter().equals(wert))
						{
							tfi.setFilter(wert);
							_bearbeitet = true;
							_tableViewer.setInput(_tabellenFilter);
						}
					}

				}
			}
		}

	}

}
