/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;

public class DialogFilterVerwaltung extends Dialog
{
	private Shell _shell = null;
	private Group _gruppeFilterUebersicht = null;
	private Composite _bereichKontrolle = null;
	private Button _schaltflaecheUebernehmen = null;
	private Button _schaltflaecheAnlegen = null;
	private Button _schaltflaecheDuplizieren = null;
	private Button _schaltflaecheBearbeiten = null;
	private Button _schaltflaecheLoeschen = null;
	private Button _schaltflaecheUmbenennen = null;
	private Button _schaltflaecheSchliessen = null;
	private Table _tabelleFilter = null;
	private TableViewer _tableViewer = null;

	/**
	 * Speichert den {@link XmlFilterVerwaltung}
	 */
	private XmlFilterVerwaltung _xmlFilterVerwaltung = null;

	/**
	 * Zeigt an, ob die XmlFilterVerwaltung bearbeitet wurde oder nicht.
	 */
	private boolean _bearbeitet = false;

	/**
	 * Speichert die {@link TabellenDatenStruktur}, auf die sich die Filter der
	 * {@link XmlFilterVerwaltung} beziehen sollen.
	 */
	private TabellenDatenStruktur _tds = null;

	public DialogFilterVerwaltung(Shell parent)
	{
		super(parent, 0);

	}

	public XmlFilterVerwaltung open(XmlFilterVerwaltung xmlFilterVerwaltung,
			TabellenDatenStruktur tds)
	{
		Shell parent = getParent();

		if (xmlFilterVerwaltung != null)
		{
			this._xmlFilterVerwaltung = xmlFilterVerwaltung.clone();
		} else
		{
			_xmlFilterVerwaltung = new XmlFilterVerwaltung();
		}

		this._tds = tds;

		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		_shell.setText("Filter Verwaltung");
		_shell.setImage(Activator.getImageDescriptor(
				"icons/sonstiges/text_editor_16x16.png").createImage());

		erzeugeDarstellung();

		if (_xmlFilterVerwaltung != null)
		{
			_tableViewer.setInput(_xmlFilterVerwaltung);
		}

		_shell.open();
		Display display = parent.getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();
		Rectangle newBounds = new Rectangle(cursorLocation.x, cursorLocation.y,
				bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		if (_bearbeitet)
		{
			return _xmlFilterVerwaltung;
		} else
		{
			return null;
		}
	}

	/**
	 * This method initializes sShell
	 */
	private void erzeugeDarstellung()
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;

		erzeugeGruppeFilterUebersicht();
		_shell.setLayout(gridLayout);
		erzeugeBereichKontrolle();
		_shell.setSize(new Point(400, 250));
	}

	/**
	 * This method initializes _gruppeFilterUebersicht
	 * 
	 */
	private void erzeugeGruppeFilterUebersicht()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_gruppeFilterUebersicht = new Group(_shell, SWT.NONE);
		_gruppeFilterUebersicht.setLayout(new GridLayout());
		_gruppeFilterUebersicht.setLayoutData(gridData);
		_gruppeFilterUebersicht.setText("Filter bersicht:");
		_tabelleFilter = new Table(_gruppeFilterUebersicht, SWT.FULL_SELECTION
				| SWT.BORDER);
		_tabelleFilter.setHeaderVisible(true);
		_tabelleFilter.setLayoutData(gridData2);
		_tabelleFilter.setLinesVisible(true);
		_tableViewer = new TableViewer(_tabelleFilter);

		/*
		 * Spalten erzeugen
		 */
		TableColumn tcIndex = new TableColumn(_tabelleFilter, SWT.CENTER);
		tcIndex.setText("Id:");
		tcIndex.setWidth(35);

		TableColumn tcBeschreibung = new TableColumn(_tabelleFilter, SWT.CENTER);
		tcBeschreibung.setText("Beschreibung:");
		tcBeschreibung.setWidth(200);

		/*
		 * ContentProvider
		 */
		ContentProvider cp = new ContentProvider();
		_tableViewer.setContentProvider(cp);

		/*
		 * LabelProvider
		 */
		LabelProvider lp = new LabelProvider();
		_tableViewer.setLabelProvider(lp);
	}

	/**
	 * Legt einen neuen Filter an und fgt ihn in die Darstellung ein
	 */
	private void aktionAnlegen()
	{
		DialogFilterEditor dialogFilterEditor = new DialogFilterEditor(_shell);
		TabellenFilter tabellenFilter = _tds.new TabellenFilter(_tds
				.getAnzahlAttribute(), _tds);
		dialogFilterEditor.oeffne(tabellenFilter);

		/*
		 * Namen fr den Filter vergeben
		 */
		InputDialog eingabeFilterName = new InputDialog(PlatformUI.getWorkbench()
				.getDisplay().getActiveShell(), "Filternamen eingeben:",
				"Bitte geben Sie einen Namen fr den Filter ein!", "leer", null);

		if (eingabeFilterName.open() == org.eclipse.jface.dialogs.Dialog.OK)
		{
			String filterName = eingabeFilterName.getValue();
			if ((filterName != null) && (filterName.length() > 0))
			{
				/*
				 * ElementFilter erzeugen
				 */
				ElementFilter filter = new ElementFilter(_xmlFilterVerwaltung
						.bestimmeNaechsteFreieIdElementFilter(), filterName);

				/*
				 * Regulre Ausdrcke erzeugen und zum ElementFilte hinzufgen
				 */
				Iterator<FilterAusdruck> it = tabellenFilter.getFilterAusdruecke()
						.iterator();
				while (it.hasNext())
				{
					FilterAusdruck filterAusdruck = it.next();
					int spalte = filterAusdruck.getSpalte();
					String bezeichnung = filterAusdruck.getBezeichnung();
					String regExString = filterAusdruck.getFilter();

					ElementRegEx elementRegEx = new ElementRegEx(spalte, bezeichnung,
							regExString);
					filter.addRegEx(elementRegEx);
				}

				/*
				 * Filter zur Filterverwaltung hinzufgen
				 */
				_xmlFilterVerwaltung.addElementFilter(filter);

				/*
				 * Darstellung updaten
				 */
				_tableViewer.setInput(_xmlFilterVerwaltung);
			}
		}
	}

	private void aktionDuplizieren()
	{
		ISelection selection = _tableViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();

			if (firstElement != null)
			{
				if (firstElement instanceof ElementFilter)
				{
					ElementFilter elementFilter = (ElementFilter) firstElement;

					ElementFilter clone = elementFilter.clone();

					clone.setBezeichnung("Kopie von " + clone.getBezeichnung());
					clone.setId(_xmlFilterVerwaltung
							.bestimmeNaechsteFreieIdElementFilter());

					_xmlFilterVerwaltung.addElementFilter(clone);

					_tableViewer.setInput(_xmlFilterVerwaltung);

				}
			}
		}
	}

	private void aktionBearbeiten()
	{
		ISelection selection = _tableViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if ((firstElement != null) && (firstElement instanceof ElementFilter))
			{
				ElementFilter elementFilterBearbeiten = (ElementFilter) firstElement;

				DialogFilterEditor dialogFilterEditor = new DialogFilterEditor(_shell);
				TabellenFilter tabellenFilter = _tds.new TabellenFilter(_tds
						.getAnzahlAttribute(), _tds);

				Iterator<ElementRegEx> it = elementFilterBearbeiten.getRegex()
						.iterator();
				while (it.hasNext())
				{
					ElementRegEx elementRegEx = it.next();
					int id = elementRegEx.getId();
					String regEx = elementRegEx.getRegEx();

					tabellenFilter.setFilterAusdruck(id, _tds.new FilterAusdruck(id,
							regEx, _tds));
				}

				/*
				 * Dialog zum Filterbearbeiten ffnen
				 */

				TabellenFilter tabellenFilterBearbeitet = dialogFilterEditor
						.oeffne(tabellenFilter);
				if (tabellenFilterBearbeitet != null)
				{
					/*
					 * Regulre Ausdrcke erzeugen und zum ElementFilte hinzufgen
					 */
					Iterator<FilterAusdruck> itRegEx = tabellenFilterBearbeitet
							.getFilterAusdruecke().iterator();
					while (itRegEx.hasNext())
					{
						FilterAusdruck filterAusdruck = itRegEx.next();
						int spalte = filterAusdruck.getSpalte();
						String bezeichnung = filterAusdruck.getBezeichnung();
						String regExString = filterAusdruck.getFilter();

						ElementRegEx elementRegEx = new ElementRegEx(spalte, bezeichnung,
								regExString);

						elementFilterBearbeiten.addRegEx(elementRegEx);
					}

					/*
					 * Filter zur Filterverwaltung hinzufgen
					 */
					_xmlFilterVerwaltung.addElementFilter(elementFilterBearbeiten);

					/*
					 * Darstellung updaten
					 */
					_tableViewer.setInput(_xmlFilterVerwaltung);
				}

			}
		}
	}

	private void aktionLoeschen()
	{
		ISelection selection = _tableViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if ((firstElement != null) && (firstElement instanceof ElementFilter))
			{
				ElementFilter elementFilterLoeschen = (ElementFilter) firstElement;
				_xmlFilterVerwaltung.removeElementFilter(elementFilterLoeschen);
				_tableViewer.setInput(_xmlFilterVerwaltung);
			}
		}
	}

	private void aktionUmbenennen()
	{
		System.out.println("=========>umbenennen:");
		ISelection selection = _tableViewer.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection iss = (IStructuredSelection) selection;
			Object firstElement = iss.getFirstElement();
			if ((firstElement != null) && (firstElement instanceof ElementFilter))
			{
				ElementFilter elementUmbenennen = (ElementFilter) firstElement;

				/*
				 * Namen fr den Filter vergeben
				 */
				InputDialog eingabeFilterName = new InputDialog(PlatformUI
						.getWorkbench().getDisplay().getActiveShell(),
						"Neuen Filternamen eingeben:",
						"Bitte geben Sie einen neuen Namen fr den Filter ein!",
						elementUmbenennen.getBezeichnung(), null);

				if (eingabeFilterName.open() == org.eclipse.jface.dialogs.Dialog.OK)
				{
					String value = eingabeFilterName.getValue();
					if (!elementUmbenennen.getBezeichnung().equals(value))
					{
						elementUmbenennen.setBezeichnung(value);
						_xmlFilterVerwaltung.addElementFilter(elementUmbenennen);
						_tableViewer.setInput(_xmlFilterVerwaltung);
					}
				}
			}
		}
	}

	private void aktionUebernehmen()
	{
		_bearbeitet = true;
		aktionSchliessen();
	}

	/**
	 * Schliet den Dialog
	 */
	private void aktionSchliessen()
	{
		_shell.dispose();
	}

	/**
	 * This method initializes _bereichKontrolle
	 * 
	 */
	private void erzeugeBereichKontrolle()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout.pack = false;
		_bereichKontrolle = new Composite(_shell, SWT.NONE);
		_bereichKontrolle.setLayout(rowLayout);
		_bereichKontrolle.setLayoutData(gridData1);

		_schaltflaecheUebernehmen = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheUebernehmen.setText("bernehmen");
		_schaltflaecheUebernehmen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUebernehmen();
			}
		});

		_schaltflaecheAnlegen = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheAnlegen.setText("Anlegen");
		_schaltflaecheAnlegen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionAnlegen();
			}
		});

		_schaltflaecheDuplizieren = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheDuplizieren.setText("Duplizieren");
		_schaltflaecheDuplizieren.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionDuplizieren();
			}
		});

		_schaltflaecheBearbeiten = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheBearbeiten.setText("Bearbeiten");
		_schaltflaecheBearbeiten.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionBearbeiten();
			}
		});

		_schaltflaecheLoeschen = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheLoeschen.setText("Lschen");
		_schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionLoeschen();
			}
		});

		_schaltflaecheUmbenennen = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheUmbenennen.setText("Umbenennen");
		_schaltflaecheUmbenennen.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionUmbenennen();
			}
		});

		_schaltflaecheSchliessen = new Button(_bereichKontrolle, SWT.NONE);
		_schaltflaecheSchliessen.setText("Schlieen");
		_schaltflaecheSchliessen.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionSchliessen();
			}

		});
	}

	private class ContentProvider implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof XmlFilterVerwaltung)
			{
				XmlFilterVerwaltung filterVerwaltung = (XmlFilterVerwaltung) inputElement;
				LinkedList<ElementFilter> listeElementFilter = filterVerwaltung
						.getElementFilter();
				re = new Object[listeElementFilter.size()];

				int i = 0;
				Iterator<ElementFilter> it = listeElementFilter.iterator();
				while (it.hasNext())
				{
					ElementFilter elementFilter = it.next();
					re[i] = elementFilter;
					i++;
				}

			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "";
			if (element instanceof ElementFilter)
			{
				ElementFilter elementFilter = (ElementFilter) element;
				if (columnIndex == 0)
				{
					re = "" + elementFilter.getId();
				} else if (columnIndex == 1)
				{
					re = elementFilter.getBezeichnung();
				}
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@SuppressWarnings("unused")
	private void zeigeInfoBox()
	{
		MessageBox mb = new MessageBox(_shell, SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Funktion noch nicht implementiert!");
		mb
				.setMessage("Die von Ihnen gewhlte Funktion ist zur Zeit noch nicht implementiert!");
		int open = mb.open();
	}

}
