/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.jdom.Attribute;
import org.jdom.Element;


/**
 * Klasse zur Reprsentation eins XML-Filterelements
 * 
 * @author HofmannJ
 * 
 */
public class ElementFilter implements Comparable<ElementFilter>
{
	/**
	 * ID des Elementes
	 */
	private int _id = -1;

	/**
	 * Bezeichnung
	 */
	private String _bezeichnung = null;

	/**
	 * Das Element selbst
	 */
	private Element _xmlElement = null;

	/**
	 * Liste der regulren Ausdrcke des Elementes
	 */
	private LinkedList<ElementRegEx> _regEx = new LinkedList<ElementRegEx>();

	/**
	 * Konstuktor, initialisiert einen {@link ElementFilter} mit ID und
	 * Bezeichnung
	 * 
	 * @param id
	 *          ID des Elementes
	 * @param bezeichnung
	 *          Bezeichnung
	 */
	public ElementFilter(int id, String bezeichnung)
	{
		initialisiere(id, bezeichnung);
	}

	/**
	 * Konstruktor, initialisiert einen {@link ElementFilter} an Hand des JDOM-{@link Element}es
	 * 
	 * @param element
	 */
	public ElementFilter(Element element)
	{
		initialisiere(element);
	}

	/**
	 * Fhrt die Initialisierung durch.
	 * 
	 * @param id
	 *          ID zur Initialisierung
	 * @param bezeichnung
	 *          Bezeichnung zur Initialisierung
	 */
	public void initialisiere(int id, String bezeichnung)
	{
		this._id = id;
		this._bezeichnung = bezeichnung;
		_xmlElement = new Element(XmlFilterVerwaltung._xmlElementFilter);
		Attribute attId = new Attribute(XmlFilterVerwaltung._xmlAttId, "" + id);
		_xmlElement.setAttribute(attId);
		Attribute attBez = new Attribute(XmlFilterVerwaltung._xmlAttBez, bezeichnung);
		_xmlElement.setAttribute(attBez);
	}

	/**
	 * Initialisiert das Element an Hand eines JDOM-{@link Element}es
	 * 
	 * @param element
	 *          JDOM-{@link Element}
	 */
	public void initialisiere(Element element)
	{
		if (element.getName().equals(XmlFilterVerwaltung._xmlElementFilter))
		{
			_xmlElement = element;

			/*
			 * ID Holen
			 */
			try
			{
				this._id = Integer.valueOf(_xmlElement.getAttributeValue(XmlFilterVerwaltung._xmlAttId));
			} catch (NumberFormatException e)
			{
				e.printStackTrace();
			}

			/*
			 * Bezeichnung Holen
			 */
			_bezeichnung = _xmlElement.getAttributeValue(XmlFilterVerwaltung._xmlAttBez);

			/*
			 * RegEx holen
			 */
			// @SuppressWarnings("unchecked")
			// List<Element> children =
			// _xmlElement.getChildren(_xmlElementAusdruck);
			// Iterator<Element> it = children.iterator();
			// _regEx.clear();
			// while (it.hasNext())
			// {
			// Element child = it.next();
			// _regEx.add(child.getText());
			// }
			@SuppressWarnings("unchecked")
			Iterator<Element> it = _xmlElement.getChildren(XmlFilterVerwaltung._xmlElementAusdruck)
					.iterator();
			while (it.hasNext())
			{				
				Element ele = it.next();				
				ElementRegEx eleRegEx = new ElementRegEx(ele);				
				_regEx.add(eleRegEx);
			}

		}
	}

	/**
	 * Liefert diesen {@link ElementFilter} als JDOM-{@link Element} zurck.
	 * 
	 * @return {@link ElementFilter} als JDOM-{@link Element}
	 */
	public Element getElement()
	{
		/*
		 * Initialisieren
		 */
		initialisiere(_id, _bezeichnung);

		/*
		 * Liste von RegEx
		 */
		if (_regEx != null)
		{
			_xmlElement.removeChildren(XmlFilterVerwaltung._xmlElementAusdruck);

			if ((_regEx != null) && (_regEx.size() > 0))
			{
				Iterator<ElementRegEx> it = _regEx.iterator();
				while (it.hasNext())
				{
					ElementRegEx elementRegEx = it.next();					
					_xmlElement.addContent(elementRegEx.getElement());
				}
			}

		}

		return _xmlElement;
	}

	/**
	 * Liefert die ID dieses Elementes
	 * 
	 * @return ID
	 */
	public int getId()
	{
		return _id;
	}

	/**
	 * Setzt die ID dieses Elementes
	 * 
	 * @param id
	 *          ID
	 */
	public void setId(int id)
	{
		this._id = id;
	}

	/**
	 * Liefert die Bezeichnung dieses Elementes.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung()
	{
		return _bezeichnung;
	}

	/**
	 * Setzt die Bezeichnung dieses Elementes.
	 * 
	 * @param bezeichnung
	 *          Zu setzende Bezeichnung
	 */
	public void setBezeichnung(String bezeichnung)
	{
		this._bezeichnung = bezeichnung;
	}

	/**
	 * Fgt einen regulren Ausdruck zur Liste der regulren Ausdrcke hinzu.
	 * Existiert ein regulrer Ausdruck mit der gleichen ID, so wird dieser
	 * gelscht und durch den neuen ersetzt.
	 * 
	 * @param regEx
	 *          Hinzuzufgender regulrer Ausdruck
	 */
	public void addRegEx(ElementRegEx regEx)
	{

		// Liste zu lschender Elemente
		LinkedList<ElementRegEx> loeschListe = new LinkedList<ElementRegEx>();

		/**
		 * Prfen, welche Elemente gelscht werden
		 */
		Iterator<ElementRegEx> itLoeschElementSuchen = _regEx.iterator();
		while (itLoeschElementSuchen.hasNext())
		{
			ElementRegEx pruefElement = itLoeschElementSuchen.next();
			if (pruefElement.getId() == regEx.getId())
			{
				loeschListe.add(pruefElement);
			}
		}

		/*
		 * Lschen
		 */
		Iterator<ElementRegEx> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext())
		{
			ElementRegEx loeschElement = itLoeschen.next();
			_regEx.remove(loeschElement);
		}

		/*
		 * Hinzufgen
		 */
		_regEx.add(regEx);

	}

	/**
	 * Liefert die Liste der regulren Ausdrcke dieses Elementes.
	 * 
	 * @return Liste der regulren Ausdrcke.
	 */
	public LinkedList<ElementRegEx> getRegex()
	{
		return _regEx;
	}

	/**
	 * Ersetzt die Liste von regulren Ausdrcken mit einer neuen Liste von
	 * regulren Ausdrcken.
	 * 
	 * @param regExes
	 *          Ersatzliste von regulren Ausdrcken.
	 */
	public void setRegEx(LinkedList<ElementRegEx> regExes)
	{
		this._regEx = regExes;
	}

	/**
	 * Bestimmt die nchste, freie ID fr Elemente vom Typ {@link ElementRegEx}.
	 * 
	 * @return ID
	 */
	public int bestimmeNaechsteFreieIdRegEx()
	{
		int re = -1;

		Iterator<ElementRegEx> it = _regEx.iterator();
		while (it.hasNext())
		{
			ElementRegEx regEx = it.next();
			if (regEx.getId() > re)
			{
				re = regEx.getId();
			}
		}

		re = re + 1;

		return re;
	}

	public ElementFilter clone()
	{
		ElementFilter elementFilterClone = new ElementFilter(_id, _bezeichnung);

		Iterator<ElementRegEx> it = _regEx.iterator();
		while (it.hasNext())
		{		
			ElementRegEx regExClone = it.next().clone();
			elementFilterClone.addRegEx(regExClone);
		}		

		return elementFilterClone;
	}

	public String toString()
	{
//		String re = "";
//
//		re = re + "===ElementFilter===\n";
//		re = re + "ID: " + _id + " Bezeichnung: " + _bezeichnung
//				+ " Anzahl RegEx: " + _regEx.size() + "\n";
//		
//		Iterator<ElementRegEx> it = _regEx.iterator();
//		while(it.hasNext())
//		{
//			re = re + it.next().toString() + "\n";
//		}
//
//		re = re + "===================";
		
		String re = "Filter " + _id + ": " + _bezeichnung;

		return re;
	}

	@Override
	public int compareTo(ElementFilter o)
	{
		int re = 0;
		
		if(this.getId() < o.getId())
		{
			re = -1;
		}
		else if(this.getId() == o.getId())
		{
			re = 0;
		}
		else if(this.getId() > o.getId())
		{
			re = 1;
		}
		
		return re;
	}



}