/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * In dieser Klasse wird getestet, ob eine Verbindung mit dem Datenverteiler
 * Ordnungsgem aufgebaut werden kann. Hierzu ist es notwendig, dass lokal ein
 * Datenverteiler luft auf Port 8083, der unter dem Benutzer "Tester" unter dem
 * Passwort "geheim" erreichbar ist.
 * <br>
 * <b>Wichtig:</b> Dieser Test kann nur als PDE JUnit-Test erfolgen!
 * 
 * @author HofmannJ
 * 
 */
public class DaVVerbindungTest
{
	private static final String _benutzer = "Tester";
	private static final String _passwort = "geheim";
	private static final String _ipAdresse = "localhost";
	private static final int _port = 8083;

	/**
	 * Verbindung zum DaV
	 */
	private static DaVVerbindung _verbindung;

	/**
	 * Notwendig fr den Singleton
	 */
	private DaVVerbindung _singleton = null;

	@Before
	public void setUp() throws Exception
	{
	}

	@After
	public void tearDown() throws Exception
	{
	}

	@BeforeClass
	public static void runBeforeClass()
	{
		_verbindung = DaVVerbindung.getInstanz();
	}

	@AfterClass
	public static void runAfterClass()
	{

	}
	
	/**
	 * Testmethode fr {@link DaVVerbindung#getInstanz()}
	 */
	@Test
	public void testGetInstanz()
	{
		
	}
	

	/**
	 * Testet den Verbindungsaufbau mit dem Datenverteiler. Gleichzeitig wird die
	 * Benachrichtigung ber den Verbindungsaufbau mit dem
	 * {@link IDaVVerbindungZuhoerer} geprft.
	 */
	@Test
	public void testBaueVerbindungMitDaVAuf()
	{
		if (_verbindung != null)
		{
			/*
			 * Verbindungszuhrer erzeugen und hinzufgen
			 */
			DaVVerbindungsZuhoerer zuhoerer = new DaVVerbindungsZuhoerer();
			_verbindung.hinzufuegenZuhoerer(zuhoerer);

			/*
			 * Daten der Verbindung setzten
			 */
			_verbindung.setDaVBenutzer(_benutzer);
			_verbindung.setDaVPasswort(_passwort);
			_verbindung.setDaVIp(_ipAdresse);
			_verbindung.setDaVPort(_port);

			/*
			 * Direkte Rckmeldung, dass die DaV-Verbindung aufgebaut wurde
			 */
			boolean baueVerbindungMitDaVAuf = _verbindung.baueVerbindungMitDaVAuf();

			/*
			 * Rckmeldung vom IDaVVerbindungsZuhoerer mit der IP des DaV
			 */
			boolean bIp = false;

			/*
			 * Rckmeldung vom IDaVVerbindungsZuhoerer mit dem Port des DaV
			 */
			boolean bPort = false;

			DaVVerbindungEreignis verbindungAufgebautEreignis = zuhoerer
					.getVerbindungAufgebautEreignis();
			if (verbindungAufgebautEreignis != null)
			{
				String ipAdresseVerbunden = verbindungAufgebautEreignis
						.getDaVIpAdresse();
				int portVerbunden = verbindungAufgebautEreignis.getDaVPort();

				if (ipAdresseVerbunden.equals(_ipAdresse))
				{
					bIp = true;
					if (portVerbunden == _port)
					{
						bPort = true;
					}
				}
			}

			_verbindung.entferneZuhoerer(zuhoerer);

			assertEquals(true, (baueVerbindungMitDaVAuf && bIp && bPort));
		} else
		{
			fail("Es ist keine Instanz der Klasse DaVVerbindung verfgbar");
		}
	}

	/**
	 * Trennt die im vorhergehenden Test aufgebaute Verbindung zum DaV und prft
	 * damit erneut den {@link IDaVVerbindungZuhoerer}.
	 */
	@Test
	public void testTrenneVerbindungMitDaV()
	{
		MockDaVVerbindungsZuhoerer daVVerbindungsZuhoerer = new MockDaVVerbindungsZuhoerer();
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(daVVerbindungsZuhoerer);

		DaVVerbindung.getInstanz().trenneDaVVerbindung();

		synchronized (daVVerbindungsZuhoerer)
		{
			try
			{
				daVVerbindungsZuhoerer.wait(10000);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}

		/*
		 * Dieses Ereignis wird nur dann geworfen, wenn die Verbindung zum DaV
		 * erfolgreich getrennt werden konnte.
		 */
		DaVVerbindungEreignis verbindungAbgebautEreignis = daVVerbindungsZuhoerer
				.getVerbindungAbgebautEreignis();

		assertEquals(true, verbindungAbgebautEreignis != null);
	}

	/**
	 * Mit Hilfe dieser Klasse kann innerhalb des Tests darauf gewartet werden,
	 * das sich die Verbindung ordnungsgem schliet.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class MockDaVVerbindungsZuhoerer implements IDaVVerbindungZuhoerer
	{
		private DaVVerbindungEreignis _verbindungAufgebautEreignis = null;
		private DaVVerbindungEreignis _verbindungAbgebautEreignis = null;

		@Override
		public void verbindungAbgebaut(DaVVerbindungEreignis e)
		{
			_verbindungAbgebautEreignis = e;
			synchronized (this)
			{
				notifyAll();
			}
		}

		@Override
		public void verbindungAufgebaut(DaVVerbindungEreignis e)
		{
			_verbindungAufgebautEreignis = e;
		}

		public DaVVerbindungEreignis getVerbindungAufgerbautEreignis()
		{
			return _verbindungAufgebautEreignis;
		}

		public DaVVerbindungEreignis getVerbindungAbgebautEreignis()
		{
			return _verbindungAbgebautEreignis;
		}

	}

	public class DaVVerbindungsZuhoerer implements IDaVVerbindungZuhoerer
	{
		/**
		 * Speichert das Ereignis fr die aufgebaute Verbindung
		 */
		private DaVVerbindungEreignis _verbindungAufgebautEreignis = null;
		
		/**
		 * Speichert das Ereignis fr die abgebaute Verbindung
		 */
		private DaVVerbindungEreignis _verbindungAbgebautEreignis = null;

		@Override
		public void verbindungAbgebaut(DaVVerbindungEreignis e)
		{
			this._verbindungAbgebautEreignis = e;
		}

		@Override
		public void verbindungAufgebaut(DaVVerbindungEreignis e)
		{
			this._verbindungAufgebautEreignis = e;
		}

		public DaVVerbindungEreignis getVerbindungAufgebautEreignis()
		{
			return _verbindungAufgebautEreignis;
		}

		public DaVVerbindungEreignis getVerbindungAbgebautEreignis()
		{		
			return _verbindungAbgebautEreignis;
		}

	}
	
	/**
	 * Testklasse fr den Singleton-Test
	 * @author HofmannJ
	 *
	 */
	@SuppressWarnings("unused")
	private class SingletonRun implements Runnable
	{

		@Override
		public void run()
		{
			DaVVerbindung s = DaVVerbindung.getInstanz();
			synchronized (DaVVerbindung.class)
			{
				if (_singleton == null)
				{
					_singleton = s;
				}
				assertSame(s, _singleton);
			}
		}
	}

}
