/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import de.bsvrz.buv.rw.rw.Activator;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dient der Registrierung von Elementen/Actions, die zum Aufbau von Mens und
 * Symbolleisten genutzt werden. <br>
 * Diese Klasse ist Threadsicher.
 * 
 * @version $Revision: 1.9 $ / $Date: 2010/02/08 08:58:41 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
@SuppressWarnings("restriction")
public class ElementRegistrierung
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Speichert die Singleton-Instanz dieser Klasse.
	 */
	private static ElementRegistrierung _instanz = null;

	/**
	 * {@link IMenuManager} der Menleiste.
	 */
	private IMenuManager _menuManager = null;

	/**
	 * {@link ICoolBarManager} der Symbolleiste.
	 */
	private ICoolBarManager _symbolLeisteManager = null;

	/**
	 * HashMap, speichert alle Actions. Muster: ActionID, {@link DeskriptorAktion}
	 * .
	 */
	private TreeMap<String, DeskriptorAktion> _hmAktionen = new TreeMap<String, DeskriptorAktion>();

	/**
	 */
	private TreeMap<String, DeskriptorAktion> _hmAktionenGlobal = new TreeMap<String, DeskriptorAktion>();

	/**
	 * HashMap, speichert alle Elemente. Muster: ActionID,
	 * {@link DeskriptorElement}.
	 */
	private TreeMap<String, DeskriptorElement> _hmElemente = new TreeMap<String, DeskriptorElement>();

	/**
	 * Liefert die Sinleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static ElementRegistrierung getInstanz()
	{
		if (_instanz == null)
		{
			synchronized (ElementRegistrierung.class)
			{
				if (_instanz == null)
				{
					_instanz = new ElementRegistrierung();
				}
			}
		}
		return _instanz;
	}

	/**
	 * Privater Konstruktor.
	 */
	private ElementRegistrierung()
	{
		// leseActionSets();
	}

	/**
	 * Setzt den {@link IMenuManager} MenuManager, der fr die Darstellung des
	 * Mens zustndig ist.
	 * 
	 * @param manager
	 *          Zu setzender {@link IMenuManager}.
	 */
	public void setMenuManager(IMenuManager manager)
	{
		this._menuManager = manager;

		_debug.fine(getClass().getName() + " => MenuManager wurde gesetzt!");

	}

	/**
	 * Liefert den {@link IMenuManager}, der fr die Darstellung des Mens
	 * zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebnen Objekt nicht
	 * vergessen!
	 * 
	 * @return Liefert den {@link IMenuManager}.
	 */
	public IMenuManager getMenuManager()
	{
		IMenuManager re = null;
		re = this._menuManager;
		return re;
	}

	/**
	 * Setzt den {@link ICoolBarManager}, der fr die Darstellung der Symbolleiste
	 * zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebenen Objekt nicht
	 * vergessen!
	 * 
	 * @param manager
	 *          Zu setzender {@link ICoolBarManager}.
	 */
	public void setStatusLeisteManager(ICoolBarManager manager)
	{
		this._symbolLeisteManager = manager;
	}

	/**
	 * Liefert den {@link ICoolBarManager}, der fr die Darstellung der
	 * Symbolleiste zustndig ist.<br>
	 * <b>Achtung:</b> Synchronisation auf dem zurckgegebenen Objekt nicht
	 * vergessen!
	 * 
	 * @return Liefert den {@link ICoolBarManager}.
	 */
	public ICoolBarManager getCoolBarManager()
	{
		return this._symbolLeisteManager;
	}

	/**
	 * Registriert ein {@link DeskriptorElement}.
	 * 
	 * @param descriptorElement
	 *          Zu registrierendes {@link DeskriptorElement}.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, falls bereits ein {@link DeskriptorElement} mit
	 *           der gleichen ID registriert wurden.
	 */
	public void registriereDeskriptorElement(DeskriptorElement descriptorElement)
			throws IllegalArgumentException
	{
		synchronized (_hmElemente)
		{
			if (!_hmElemente.containsKey(descriptorElement.getAktionID()))
			{
				_hmElemente.put(descriptorElement.getAktionID(), descriptorElement);
			} else
			{
				throw new IllegalArgumentException(
						"Das DescriptorElement mit der ActionID "
								+ descriptorElement.getAktionID()
								+ " wurde schon einmal registriert. Eine erneute Registrierung ist nicht mglich.");
			}
		}
	}

	/**
	 * Registriert eine {@link DeskriptorAktion}.
	 * 
	 * @param descriptorAction
	 *          Zu registrierende {@link DeskriptorAktion}.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, falls bereits eine {@link DeskriptorAktion}
	 *           registriert wurde.
	 */
	public void registriereDeskriptorAktion(DeskriptorAktion descriptorAction)
			throws IllegalArgumentException
	{
		synchronized (_hmAktionen)
		{
			if (!_hmAktionen.containsKey(descriptorAction.getAktionID()))
			{
				_hmAktionen.put(descriptorAction.getAktionID(), descriptorAction);
				_debug.fine(getClass().getName()
						+ "::registriereDeskriptorAktion \n\t=> Registrierung ok -> "
						+ descriptorAction.getAktionID());
			} else
			{
				throw new IllegalArgumentException(
						"Die DescriptorAction mit der ActionID "
								+ descriptorAction.getAktionID()
								+ " wurde schon einmal registriert. Eine erneute Registrierung ist nicht mglich.");
			}
		}
	}

	/**
	 * Registriert eine globale {@link DeskriptorAktion}. Das Flag fr globale
	 * Aktion muss gesetzt sein.
	 * 
	 * @param descriptorActionGlobal
	 *          Globale {@link DeskriptorAktion}, die registriert werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, falls bereits eine globale
	 *           {@link DeskriptorAktion} mit der gleichen ID registriert wurde
	 *           oder das Flag fr globale Aktion nicht gesetzt wurde.
	 */
	public void registriereDeskriptorAktionGlobal(
			DeskriptorAktion descriptorActionGlobal) throws IllegalArgumentException
	{
		if (descriptorActionGlobal.istGlobaleAktion())
		{
			synchronized (_hmAktionenGlobal)
			{
				if (!_hmAktionenGlobal
						.containsKey(descriptorActionGlobal.getAktionID()))
				{
					_hmAktionenGlobal.put(descriptorActionGlobal.getAktionID(),
							descriptorActionGlobal);
				} else
				{
					String meldung = "Die globale DescriptorAction mit der ActionID "
							+ descriptorActionGlobal.getAktionID()
							+ "wurde schon einmal registriert. Eine erneute Registrierung ist nicht mglich.";
					System.out.println("=> " + meldung);
					throw new IllegalArgumentException(meldung);
				}
			}

		} else
		{
			throw new IllegalArgumentException(
					"Sie haben versucht, eine nicht globale Aktion in der HashMap der globalen Aktionen einzufgen. Im DescriptorAction muss das Flag fr globale Aktion gesetzt sein.");
		}

	}

	/**
	 * Liefert ein {@link DeskriptorElement} oder <code>null</code>, falls dieses
	 * nicht existiert.
	 * 
	 * @param actionID
	 *          ID des {@link DeskriptorElement}, das geliefert werden soll.
	 * @return {@link DeskriptorElement} oder <code>null</code>.
	 */
	public DeskriptorElement getDeskriptorElement(String actionID)
	{
		synchronized (_hmElemente)
		{
			if (_hmElemente.containsKey(actionID))
			{
				return _hmElemente.get(actionID);
			} else
			{
				return null;
			}
		}
	}

	/**
	 * Liefert eine {@link DeskriptorAktion} oder <code>null</code>, falls diese
	 * nicht existiert.
	 * 
	 * @param actionID
	 *          ID der {@link DeskriptorAktion}, die geliefert werden soll.
	 * @return {@link DeskriptorAktion} oder <code>null</code>.
	 */
	public DeskriptorAktion getDeskriptorAktion(String actionID)
	{
		synchronized (_hmAktionen)
		{
			if (_hmAktionen.containsKey(actionID))
			{
				return _hmAktionen.get(actionID);
			} else
			{
				return null;
			}
		}
	}

	/**
	 * Liefert eine globale {@link DeskriptorAktion} oder <code>null</code>, falls
	 * diese nicht existiert.
	 * 
	 * @param actionID
	 *          ID der globalen {@link DeskriptorAktion}, die geliefert werden
	 *          soll.
	 * @return Globale {@link DeskriptorAktion} oder <code>null</code>.
	 */
	public DeskriptorAktion getDeskriptorAktionGlobal(String actionID)
	{
		synchronized (_hmAktionenGlobal)
		{
			if (_hmAktionenGlobal.containsKey(actionID))
			{
				return _hmAktionenGlobal.get(actionID);
			} else
			{
				return null;
			}
		}
	}

	/**
	 * Liefert eine Liste aller {@link DeskriptorElement}e eines Plugins.
	 * 
	 * @param pluginID
	 *          ID des Plug-Ins.
	 * @return Liste aller {@link DeskriptorElement}e eines Plug-Ins.
	 */
	public List<DeskriptorElement> getPluginElementDeskriptoren(String pluginID)
	{
		List<DeskriptorElement> elemente = new LinkedList<DeskriptorElement>();

		synchronized (_hmElemente)
		{
			Iterator<Entry<String, DeskriptorElement>> it = _hmElemente.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, DeskriptorElement> entry = it.next();
				if (entry.getValue().getPluginID().equals(pluginID))
				{
					elemente.add(entry.getValue());
				}
			}
		}

		return elemente;
	}

	/**
	 * Liefert eine Liste aller {@link DeskriptorAktion}s eines Plugins.
	 * 
	 * @param pluginID
	 *          ID des Plug-Ins.
	 * @return Liste aller {@link DeskriptorAktion}s eines Plug-Ins.
	 */
	public List<DeskriptorAktion> getPluginAktionDeskriptoren(String pluginID)
	{
		List<DeskriptorAktion> actions = new LinkedList<DeskriptorAktion>();

		synchronized (_hmAktionen)
		{
			Iterator<Entry<String, DeskriptorAktion>> it = _hmAktionen.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, DeskriptorAktion> entry = it.next();
				if (entry.getValue().getPluginID().equals(pluginID))
				{
					actions.add(entry.getValue());
				}
			}
		}

		return actions;
	}

	/**
	 * Liefert die Descriptoren in kombinierter Form, wobei Deskriptoren vom Typ
	 * {@link DeskriptorAktion} die Deskriptoren vom Typ {@link DeskriptorElement}
	 * berschreiben.
	 * 
	 * @param pluginID
	 *          ID des Plug-Ins, zu dem die kombinierten Deskriptoren gebildet
	 *          werden sollen.
	 * @return Eine Liste von Objekten, die aus {@link DeskriptorAktion} und
	 *         {@link DeskriptorElement} Elementen besteht. Unterscheidung nach
	 *         Typ kann z.B. mittels <code>instanceof</code> erfolgen.
	 */
	public List<Object> getPluginDeskriptorenKombiniert(String pluginID)
	{
		_debug.fine("\n==================================Plugin: " + pluginID
				+ "==================================");

		// TreeMap fr die Sortierung
		TreeMap<String, Object> tm = new TreeMap<String, Object>();

		// Elemente
		_debug.fine(getClass().getName() + " -> Anzahl Elemente Insgesamt: "
				+ _hmElemente.size());
		_debug.fine(getClass().getName() + " -> Anzahl Actions Insgesamt: "
				+ _hmAktionen.size());

		synchronized (_hmElemente)
		{
			Iterator<Entry<String, DeskriptorElement>> it = _hmElemente.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, DeskriptorElement> entry = it.next();
				if (entry.getValue().getPluginID().equals(pluginID))
				{
					tm.put(entry.getKey(), entry.getValue());
				}
			}
		}

		// Actions
		synchronized (_hmAktionen)
		{
			Iterator<Entry<String, DeskriptorAktion>> itActions = _hmAktionen
					.entrySet().iterator();
			while (itActions.hasNext())
			{
				Entry<String, DeskriptorAktion> entry = itActions.next();
				if (entry.getValue().getPluginID().equals(pluginID))
				{
					tm.put(entry.getKey(), entry.getValue());
				}
			}
		}

		// Liste erzeugen
		List<Object> rueckListe = new LinkedList<Object>();
		Iterator<Entry<String, Object>> itListenBildung = tm.entrySet().iterator();
		while (itListenBildung.hasNext())
		{
			Entry<String, Object> entry = itListenBildung.next();
			rueckListe.add(entry.getValue());
		}

		_debug.fine(getClass().getName() + " -> Anzahl Kombiniert fr Plugin ("
				+ pluginID + "): " + tm.size());
		_debug
				.fine("====================================================================================\n");

		return rueckListe;
	}

	/**
	 * Liefert eine Liste aller globalen {@link DeskriptorAktion}s eines Plugins.
	 * 
	 * @param pluginID
	 *          ID des Plug-Ins.
	 * @return Liste aller globalen {@link DeskriptorAktion}s eines Plug-Ins.
	 */
	public List<DeskriptorAktion> getPluginAktionDeskriptorenGlobal(
			String pluginID)
	{
		List<DeskriptorAktion> globaleActions = new LinkedList<DeskriptorAktion>();

		synchronized (_hmAktionenGlobal)
		{
			Iterator<Entry<String, DeskriptorAktion>> it = _hmAktionenGlobal
					.entrySet().iterator();
			while (it.hasNext())
			{
				Entry<String, DeskriptorAktion> entry = it.next();
				if (entry.getValue().getPluginID().equals(pluginID))
				{
					globaleActions.add(entry.getValue());
				}
			}
		}

		return globaleActions;
	}

	/**
	 * Liefert eine Liste mit den IDs aller Plug-Ins.
	 * 
	 * @return Liste aller Plug-In IDs.
	 */
	public List<String> getPluginIDs()
	{
		// Speichert sortiert die PluginIDs
		TreeMap<String, String> tm = new TreeMap<String, String>();

		// Iterator ber hmActions
		synchronized (_hmAktionen)
		{
			Iterator<Entry<String, DeskriptorAktion>> itActions = _hmAktionen
					.entrySet().iterator();
			while (itActions.hasNext())
			{
				Entry<String, DeskriptorAktion> entry = itActions.next();
				if (!tm.containsKey(entry.getValue().getPluginID()))
				{
					tm
							.put(entry.getValue().getPluginID(), entry.getValue()
									.getPluginID());
				}
			}
		}

		// Iterator ber hmActionsGlobal
		synchronized (_hmAktionenGlobal)
		{
			Iterator<Entry<String, DeskriptorAktion>> itActionsGlobal = _hmAktionenGlobal
					.entrySet().iterator();
			while (itActionsGlobal.hasNext())
			{
				Entry<String, DeskriptorAktion> entry = itActionsGlobal.next();
				if (!tm.containsKey(entry.getValue().getPluginID()))
				{
					tm
							.put(entry.getValue().getPluginID(), entry.getValue()
									.getPluginID());
				}
			}
		}

		// Iterator ber hmElemente
		synchronized (_hmElemente)
		{
			Iterator<Entry<String, DeskriptorElement>> itElemente = _hmElemente
					.entrySet().iterator();
			while (itElemente.hasNext())
			{
				Entry<String, DeskriptorElement> entry = itElemente.next();
				if (!tm.containsKey(entry.getValue().getPluginID()))
				{
					tm
							.put(entry.getValue().getPluginID(), entry.getValue()
									.getPluginID());
				}
			}
		}

		// Iterator ber TM
		List<String> returnList = new LinkedList<String>();
		Iterator<Entry<String, String>> itTm = tm.entrySet().iterator();
		while (itTm.hasNext())
		{
			returnList.add(itTm.next().getKey());
		}

		return returnList;
	}

	@SuppressWarnings( { "unused" })
	private void leseActionSetsOld()
	{
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IConfigurationElement[] aConfigurationElementsFor = reg
				.getConfigurationElementsFor("org.eclipse.ui.actionSets");
		for (IConfigurationElement confEle : aConfigurationElementsFor)
		{
			String contributor = confEle.getContributor().getName();

			IConfigurationElement[] iceAction = confEle.getChildren("action");
			for (IConfigurationElement ceAction : iceAction)
			{
				String attID = ceAction.getAttribute("id");
				String attLabel = ceAction.getAttribute("label");
				String attIcon = ceAction.getAttribute("icon");
				String attTooltip = ceAction.getAttribute("tooltip");

				boolean global = false;
				String attRetarget = ceAction.getAttribute("retarget");
				if ((attRetarget != null) && (attRetarget.length() > 0))
				{
					if (attRetarget.toLowerCase().equals("true"))
					{
						global = true;
					}
				}

				String attClass = ceAction.getAttribute("class");
				String attStyle = ceAction.getAttribute("style");

				int style = IAction.AS_PUSH_BUTTON;

				if (attStyle != null)
				{
					if (attStyle.equals("radio"))
					{
						style = IAction.AS_RADIO_BUTTON;
					} else if (attStyle.equals("toogle"))
					{
						style = IAction.AS_CHECK_BOX;
					} else if (attStyle.equals("pulldown"))
					{
						style = IAction.AS_DROP_DOWN_MENU;
					}
				}

				PluginAction action = new PluginAction(ceAction, attID, style)
				{
				};
				action.setText(attLabel);
				action.setToolTipText(attTooltip);

				if ((attIcon != null) && (attIcon.length() > 0))
				{
					ImageDescriptor imageDescriptor = Activator.getImageDescriptor(
							contributor, attIcon);
					if (imageDescriptor != null)
					{
						action.setImageDescriptor(imageDescriptor);
					}
				}

				/*
				 * DeskriptorAktion erzeugen und registrieren.
				 */
				DeskriptorAktion deskriptorAktion = new DeskriptorAktion(contributor,
						action, global);
				if (global)
				{
					registriereDeskriptorAktionGlobal(deskriptorAktion);
				} else
				{
					registriereDeskriptorAktion(deskriptorAktion);
				}
			}
		}
	}

	/**
	 * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow
	 * ein.<br>
	 * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des
	 * aktuellen EclipseWindows hngen. <br>
	 * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen
	 * bercksichtigt werden. Dies ist fr ein Window aber NICHT ntig.
	 */
	public void leseActionSets()
	{
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IConfigurationElement[] aConfigurationElementsFor = reg
				.getConfigurationElementsFor("org.eclipse.ui.actionSets");
		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();
		for (IConfigurationElement confEle : aConfigurationElementsFor)
		{
			String contributor = confEle.getContributor().getName();

			IConfigurationElement[] iceAction = confEle.getChildren("action");
			for (IConfigurationElement ceAction : iceAction)
			{
				boolean global = false;
				String attRetarget = ceAction.getAttribute("retarget");
				if ((attRetarget != null) && (attRetarget.length() > 0))
				{
					if (attRetarget.toLowerCase().equals("true"))
					{
						global = true;
					}
				}

				ActionDescriptor actionDescriptor = createActionDescriptor(ceAction,
						window);

				PluginAction action = actionDescriptor.getAction();
				DeskriptorAktion deskriptorAktion = new DeskriptorAktion(contributor,
						action, global);
				if (global)
				{
					try
					{
						registriereDeskriptorAktionGlobal(deskriptorAktion);
					} catch (IllegalArgumentException iae)
					{
					}
				} else
				{					
					try
					{
						registriereDeskriptorAktion(deskriptorAktion);
					} catch (IllegalArgumentException iae)
					{
					}
				}
			}
		}
	}

	// Vgl. PluginActionSetBuilder.creatActionDescriptor
	protected ActionDescriptor createActionDescriptor(
			IConfigurationElement element, IWorkbenchWindow window)
	{
		ActionDescriptor desc = null;

		/*
		 * "pulldown" ist z.Zt. als Deprecated markiert. Es wird das
		 * "Style"-Attribut verwendet.
		 */
		boolean pullDownStyle = false;
		String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
		if (style != null)
		{
			pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
		} else
		{
			String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
			pullDownStyle = pulldown != null && pulldown.equals("true");
		}

		if (pullDownStyle)
		{
			desc = new ActionDescriptor(element,
					ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
		} else
		{
			desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
		}

		// WWinPluginAction action = (WWinPluginAction) desc.getAction();
		// action.setActionSetId(actionSet.getDesc().getId());
		// actionSet.addPluginAction(action);
		return desc;
	}

}
