/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.menu.xml.datenstruktur;

import org.jdom.Attribute;
import org.jdom.Element;

import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Reprsentation eines "Items". Die Definition eines Items richtet sich nach <code>menubar.dtd</code> bzw.
 * <code>toolbar.dtd</code>.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:05 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class XmlElementItem
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private String _id = null;

  private String _name = null;

  private String _actionId = null;

  private Element _elItem = null;

  private boolean _ausklappbar = false;

  private String _elItemName = XmlBezeichnerMenuSymbol.elementItem;

  private String _elItemAttributId = XmlBezeichnerMenuSymbol.attId;

  private String _elItemAttributName = XmlBezeichnerMenuSymbol.attName;

  private String _elItemAttributActionId = XmlBezeichnerMenuSymbol.attActionId;

  private String _elItemAttributAusklappbar = XmlBezeichnerMenuSymbol.attItemAusklappbar;

  /**
   * Parameterloser Konstruktor
   * 
   */
  public XmlElementItem()
  {
  }

  /**
   * Konstruktor, erzeugt direkt ein Item mit id und name
   * 
   * @param id
   *          Id des zu erzeugenden Items
   * @param actionId
   *          Id der Action in der {@link ElementRegistrierung}.
   * @param name
   *          Name des zu erzeugenden Items
   */
  public XmlElementItem(String id, String actionId, String name)
  {
    this._id = id;
    this._actionId = actionId;
    this._name = name;
    generiereElement();
  }

  /**
   * Konstruktor, erzeugt direk ein Item mit id und name und setzt das Flag ausklappbar.
   * 
   * @param id
   *          Id des zu erzeugenden Items
   * @param actionId
   *          Id der Action in der {@link ElementRegistrierung}.
   * @param name
   *          Name des zu erzeugenden Items
   * @param ausklappbar
   *          Ausklappbares Item ja oder nein. Ist nur fr CoolBars interessant.
   */
  public XmlElementItem(String id, String actionId, String name, boolean ausklappbar)
  {
    this._id = id;
    this._actionId = actionId;
    this._name = name;
    this._ausklappbar = ausklappbar;
    generiereElement();
  }

  /**
   * Generiert das <code>Element</code> fr den <code>JDOM</code>-Baum
   * 
   */
  private void generiereElement()
  {
    _elItem = new Element(_elItemName);
    Attribute elItemId = new Attribute(_elItemAttributId, _id);
    Attribute elActionId = new Attribute(_elItemAttributActionId, _actionId);
    Attribute elItemName = new Attribute(_elItemAttributName, _name);
    _elItem.setAttribute(elItemId);
    _elItem.setAttribute(elActionId);
    _elItem.setAttribute(elItemName);

    // Ausklappbar, nur bei Bedarf.
    if (_ausklappbar)
    {
      Attribute elItemAusklappbar = new Attribute(_elItemAttributAusklappbar, "ja");
      _elItem.setAttribute(elItemAusklappbar);
    }
  }

  /**
   * Generiert die Objektreprsentation eines Items aus einem <code>Element</code>. Es wird hierzu kein Clone
   * erstellt, folglich bleiben Eltern und Kinder des Elements erhalten.
   * 
   * @param el
   *          <code>Element</code>, zu dem die Objektreprsentation erstellt wird.
   */
  public void generiereElement(Element el)
  {
    if (el.getName().equalsIgnoreCase(_elItemName) && el.getAttributes().size() == 2)
    {
      _elItem = el;
      _id = el.getAttributeValue(_elItemAttributId);
      _actionId = el.getAttributeValue(_elItemAttributActionId);
      _name = el.getAttributeValue(_elItemAttributName);

      // Ausklappbar
      String ausklappbar = el.getAttributeValue(_elItemAttributAusklappbar);
      if ((ausklappbar != null) && (!ausklappbar.equals("")))
      {
        _debug.fine(getClass().getName() + "::generiereElement \n\t=> Attribut Ausklappbar gefunden.\n");
        if (ausklappbar.equals("ja"))
        {
          _ausklappbar = true;
        }
        else if (ausklappbar.equals("nein"))
        {
          _ausklappbar = false;
        }
      }

    }
  }

  /**
   * Generiert die Objektreprsentation eines Items aus einem <code>Element</code>. Es handelt sich hierbei um eine
   * flache Kopie, Eltern- und Kindelemente gehen verloren.
   * 
   * @param el
   *          <code>Element</code>, von dem eine flache Kopie erstellt werden soll.
   */
  public void generiereAusElement(Element el)
  {
    // if(el.getName().equalsIgnoreCase(_elItemName) &&
    // el.getAttributes().size() == 2)
    if (el.getName().equalsIgnoreCase(_elItemName))
    {
      if (el.getAttributeValue(_elItemAttributId) != null)
      {
        _id = el.getAttributeValue(_elItemAttributId);
      }
      else
      {
        _id = "[leer]";
      }

      if (el.getAttributeValue(_elItemAttributActionId) != null)
      {
        _actionId = el.getAttributeValue(_elItemAttributActionId);
      }
      else
      {
        _actionId = "[leer]";
      }

      if (el.getAttributeValue(_elItemAttributName) != null)
      {
        _name = el.getAttributeValue(_elItemAttributName);
      }
      else
      {
        _name = "[leer]";
      }

      // Ausklappbar
      if ((el.getAttributeValue(_elItemAttributAusklappbar) != null)
          && (!el.getAttributeValue(_elItemAttributAusklappbar).equals("")))
      {
        if (el.getAttributeValue(_elItemAttributAusklappbar).equals("ja"))
        {
          _ausklappbar = true;
        }
        else
        {
          _ausklappbar = false;
        }
      }

      generiereElement();
    }
  }

  /**
   * Liefert das <code>Element</code> des Items
   * 
   * @return Das <code>Element</code> des Items
   */
  public Element getElementItem()
  {
    return _elItem;
  }

  /**
   * Liefert eine String-Reprsentation des Items
   * 
   * @return Stringreprsentation des Items.
   */
  public String toString()
  {
    String returnString = "Element: " + _elItemName + " ActionId: " + _actionId + " Id: " + _id + " Name: " + _name;
    if (_ausklappbar)
    {
      returnString = returnString + " Ausklappbar: ja";
    }
    return returnString;
  }

}