/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.parameter;

import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationTaskException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.management.UserAdministration;

public class AdminBestimmung
{
	/**
	 * Flag Adminstrator
	 */
	private boolean _isAdminstrator = false;

	/**
	 * Privater Konstruktor
	 */
	private AdminBestimmung()
	{
		init();
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link AdminBestimmung}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen
	{
		private static AdminBestimmung INSTANZ = new AdminBestimmung();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link AdminBestimmung}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link AdminBestimmung}
	 */
	public static AdminBestimmung getInstanz()
	{
		return Innen.INSTANZ;
	}

	/**
	 * Initialisierung
	 */
	private void init()
	{
		bestimmeObAdminRechteVorhandenAnhandParameter();
	}

	private void bestimmeObAdminRechteVorhandenAnhandParameter()
	{
		KmdParameter kmdParameter = Kommandozeilenparameter.getInstanz()
				.getKommandoZeilenParameter("-Admin");
		if (kmdParameter != null)
		{
			String wert = kmdParameter.getWert();
			if (wert.equalsIgnoreCase("true"))
			{
				_isAdminstrator = true;
			}
		}
		else
		{
			_isAdminstrator = false;
		}
	}

	/**
	 * Prft, ob man ber administrative Rechte verfgt.<br>
	 * Besteht keine Verbindung zum DaV, wird mit Hilfe des Parameters
	 * -Admin=true||false festgelegt, ob der Benutzer ber administrative Rechte
	 * verfgt.<br>
	 * Besteht eine Verbindung zum DaV, wird die Benutzerverwaltung des DaV
	 * verwendet um zu Bestimmen, ob administrative Rechte fr den aktuell
	 * angemeldeten Benutzer vorhanden sind.
	 * 
	 * @return Liefert <code>true</code>, wenn man ber administrative Rechte
	 *         verfgt, sonst <code>false</code>.
	 */
	public boolean istAdministrator()
	{
		/*
		 * Existiert eine DaV-Verbindung? Dann an Hand der DaV Benutzerverwaltung
		 * prfen, ob administrative Rechte vorhanden sind oder nicht.
		 */
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			ClientDavInterface connection = DaVVerbindung.getInstanz()
					.getDaVVerbindung();
			DataModel model = connection.getDataModel();
			UserAdministration userAdministration = model.getUserAdministration();

			String benutzerName = BenutzerAngemeldet.getInstanz().getName();
			String passwort = BenutzerAngemeldet.getInstanz().getPasswort();

			boolean adminRights = false;
			try
			{
				userAdministration.changeUserRights(benutzerName, passwort,
						benutzerName, true);
				adminRights = true;
			}
			catch (ConfigurationTaskException e)
			{
				// e.printStackTrace();
			}

			_isAdminstrator = adminRights;
		}
		else
		// Ansonsten findet die Bestimmung an Hand des Parameter
		// -admin=<true>||<false> statt.
		{
			bestimmeObAdminRechteVorhandenAnhandParameter();
		}

		return _isAdminstrator;
	}

}
