/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.transfers;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

/**
 * Transfer Objekt zum bertragen von Konfigurationsobjekten ber die 
 * Zwischenablage. Das Datum dieser Klasse ist die PID als String. <br>
 * 
 * Siehe Verwendung von TextTransfer.
 * @author Wolf
 *
 */
/**
 * <code>Transfer</code> zum Speichern und Lesen von {@link KonfigurationsObjekt}en aus und in die Zwischenablage.
 * 
 * @author mkempka
 */
public class KonfigurationsObjektPidTransfer extends ByteArrayTransfer
{
  private static final String KONFIGURATIONS_NAME = KonfigurationsObjektPidTransfer.class.getName();

  private static final int KONFIGURATIONS_ID = registerType(KONFIGURATIONS_NAME);

  private static KonfigurationsObjektPidTransfer instance = null;

  private KonfigurationsObjektPidTransfer()
  {
    super();
  }

  /**
   * Gibt die Einzelstck-Instanz der Klasse zurck.
   * 
   * @return
   */
  public static KonfigurationsObjektPidTransfer getInstanz()
  {
    if (instance == null)
    {
      instance = new KonfigurationsObjektPidTransfer();
    }

    return instance;
  }

  /*
   * (Kein Javadoc)
   * 
   * @see org.eclipse.swt.dnd.ByteArrayTransfer#javaToNative(java.lang.Object, org.eclipse.swt.dnd.TransferData)
   */
  @Override
  public void javaToNative(Object object, TransferData transferData)
  {
    if ((object == null) || ((!(object instanceof String)) && (!(object instanceof String[]))))
    {
      return;
    }

    if (isSupportedType(transferData))
    {
      String[] data = null;

      if (object instanceof String)
      {
        data = new String[] { (String)object };
      }
      else
      {
        data = (String[])object;
      }

      try
      {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream writeOut = new DataOutputStream(out);

        for (int i = 0; i < data.length; i++)
        {
          writeOut.writeChars(data[i]);
          writeOut.writeChar(1); // Trenner fr Pids
        }

        byte[] buffer = out.toByteArray();
        writeOut.close();
        super.javaToNative(buffer, transferData);
      }
      catch (IOException ioe)
      {
        ioe.printStackTrace();
      }
    }
  }

  /*
   * (Kein Javadoc)
   * 
   * @see org.eclipse.swt.dnd.ByteArrayTransfer#nativeToJava(org.eclipse.swt.dnd.TransferData)
   */
  @Override
  protected Object nativeToJava(TransferData transferData)
  {
    List<String> result = new ArrayList<String>();

    if (isSupportedType(transferData))
    {
      byte[] buffer = (byte[])super.nativeToJava(transferData);

      if (buffer == null)
      {
        return null;
      }

      try
      {
        ByteArrayInputStream in = new ByteArrayInputStream(buffer);
        DataInputStream readIn = new DataInputStream(in);

        while (readIn.available() > 0)
        {
          StringBuilder sb = new StringBuilder();
          char idChar;

          do
          {
            idChar = readIn.readChar();

            if (idChar != 1) // Trenner fr Pids
            {
              sb.append(idChar);
            }
          } while (readIn.available() > 0 && idChar != 1);

          result.add(sb.toString());
        }
      }
      catch (IOException ioe)
      {
        ioe.printStackTrace();
      }
    }

    return result.toArray(new String[0]);
  }

  /*
   * (Kein Javadoc)
   * 
   * @see org.eclipse.swt.dnd.Transfer#getTypeIds()
   */
  @Override
  protected int[] getTypeIds()
  {
    return new int[] { KONFIGURATIONS_ID };
  }

  /*
   * (Kein Javadoc)
   * 
   * @see org.eclipse.swt.dnd.Transfer#getTypeNames()
   */
  @Override
  protected String[] getTypeNames()
  {
    return new String[] { KONFIGURATIONS_NAME };
  }
}
