/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.dav.dav.
 * 
 * de.bsvrz.dav.dav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.dav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.dav.dav; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.dav.main;

import de.bsvrz.dav.daf.communication.lowLevel.telegrams.BaseSubscriptionInfo;
import de.bsvrz.dav.daf.communication.lowLevel.telegrams.TransmitterDataSubscription;


/**
 * Ein Eintrag mit Anmeldungsinformationen wird erstellt und in eine Verwaltungstabelle aufgenommen. Die Klasse reprsentiert einen Vermerk in der
 * Verwaltungstabelle fr Anmeldungsinformationen Diese Klasse speichert folgende Informationen:<ul> <li>Die ID der Anmeldeinstanz.</li> <li>Die
 * Basisanmeldeinformation (Objekt-ID, Attributgruppencode, Aspektcode, Simulationsvariante).</li> <li>Information, ob es sich um einen Sender oder einen
 * Empfnger handelt.</li> <li>Information ber den Quittungszustands (Positiv, negativ, ...).</li> <li>Die ID des Zuliefererdatenverteilers.</li> <li>Eine
 * Liste der potentiellen Zentraldatenverteiler.</li> <li>Die Information ber nachfolgende Anmeldungen beim anderen Datenverteiler und deren
 * Quittungszustand.</li> <li>Die Information, ob es sich um eine Applikations- oder eine Datenverteileranmeldung handelt.</li> </ul>
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5068 $
 */
public class InAndOutSubscription {

	class OutSubscription {

		long targetTransmitterId;

		long transmitterToConsider[];

		long mainTransmitterId;

		byte receip;

		OutSubscription() {
		}
	}

	/** Die Id des Initiator-Datenverteilers */
	long sourceTransmitterId;

	/** Die Basisinformationen einer Anmeldung */
	BaseSubscriptionInfo baseSubscriptionInfo;

	/** Die Information, ob die Anmeldung eine Sender- oder Empfngeranmeldung ist 0: Senderanmeldung 1: Empfngeranmeldung */
	byte subscriptionState;

	/**
	 * Quittungszustand <ul> <li>-1: noch nicht quittiert</li> <li> 0: Keiner der angemeldeten Zentraldatenverteiler ist fr die Daten zustndig.</li> <li> 1: Der
	 * spezifizierte Datenverteiler ist der Zustndige fr die Daten.</li> <li> 2: Der spezifizierte Datenverteiler ist der Zustndige fr die Daten, allerdings
	 * sind die notwendigen Rechte nicht vorhanden.</li> <li> 3: Mehr als eine Quelle im System angemeldet.</li> </ul>
	 */
	byte receip;

	/** Die Id des Zulieferer-Datenverteilers, wenn dieser positiv quittiert ist, sonst -1 */
	long mainTransmitterId;

	/** Die Liste der zu bercksichtigenden Datenverteiler, die beim Erzeugen dieses Objektes bergegeben wurde (dient auch zur Identifikation der Anmeldung). */
	long transmitterToConsider[];

	/** Die Liste der aktuellen zu bercksichtigenden Datenverteiler */
	long actualTransmitterToConsider[];

	/** Liste der rausgeschickten Anmeldungen */
	OutSubscription outSubscriptions[];

	/** Information, ob diese Anmeldungsanfrage von einer Applikation oder von einem Datenverteiler gestartet wurde. */
	boolean isTransmitterConnection;

	/** Anzahl der gleichen Anmeldungen */
	int count = 1;

	public String toString() {
		return "InAndOutSubscription[" + "sourceTransmitterId:" + sourceTransmitterId + ", " + "baseSubscriptionInfo:" + baseSubscriptionInfo + ", "
		       + "subscriptionState:" + subscriptionState + ", " + "receip:" + receip + ", " + "mainTransmitterId:" + mainTransmitterId + ", "
		       + "outSubscriptions:" + outSubscriptions + ", " + "isTransmitterConnection:" + isTransmitterConnection + ", " + "count:" + count + "]";
	}

	InAndOutSubscription(long sourceId, TransmitterDataSubscription subscription) {
		if(subscription == null) {
			throw new IllegalArgumentException("Null Argumente");
		}
		sourceTransmitterId = sourceId;
		baseSubscriptionInfo = subscription.getBaseSubscriptionInfo();
		subscriptionState = subscription.getSubscriptionState();
		transmitterToConsider = subscription.getTransmitters();
		actualTransmitterToConsider = transmitterToConsider;
		receip = (byte)-1;
		mainTransmitterId = -1;
		isTransmitterConnection = true;
	}

	/**
	 * Erzeugt ein neues Objekt mit den gegebenen Parametern.
	 *
	 * @param sourceId               Id der Applikation
	 * @param info                   Anmeldeinformationen
	 * @param _subscriptionState     0: Senderanmeldung 1: Empfngeranmeldung
	 * @param _transmitterToConsider Ids von zu bercksichtigenden Datenverteilern
	 */
	InAndOutSubscription(long sourceId, BaseSubscriptionInfo info, byte _subscriptionState, long _transmitterToConsider[]) {
		if(info == null) {
			throw new IllegalArgumentException("Null Argumente");
		}
		sourceTransmitterId = sourceId;
		baseSubscriptionInfo = info;
		subscriptionState = _subscriptionState;
		transmitterToConsider = _transmitterToConsider;
		actualTransmitterToConsider = transmitterToConsider;
		receip = (byte)-1;
		mainTransmitterId = -1;
		isTransmitterConnection = false;
	}

	/**
	 * Erzeugt ein neues Objekt mit den gegebenen Parametern.
	 *
	 * @param origin Objekt dieser Klasse
	 */
	InAndOutSubscription(InAndOutSubscription origin) {
		if(origin == null) {
			throw new IllegalArgumentException("Null Argumente");
		}
		sourceTransmitterId = origin.sourceTransmitterId;
		baseSubscriptionInfo = origin.baseSubscriptionInfo;
		subscriptionState = origin.subscriptionState;
		transmitterToConsider = origin.transmitterToConsider;
		actualTransmitterToConsider = transmitterToConsider;
		isTransmitterConnection = origin.isTransmitterConnection;
		receip = (byte)-1;
		mainTransmitterId = -1;
	}

	/**
	 * Erzeugt eine Instanz der inneren Klasse, die eine ausgehende Anmeldung reprsentiert.
	 *
	 * @param targetTransmitterId   ID des Zieldatenverteilers
	 * @param transmitterToConsider Die Liste der aktuellen zu bercksichtigenden Datenverteiler.
	 * @param receip                Quittungszustand <ul> <li>-1: noch nicht quittiert</li> <li> 0: Keiner der angemeldeten Zentraldatenverteiler ist fr die Daten
	 *                              zustndig.</li> <li> 1: Der spezifizierte Datenverteiler ist der Zustndige fr die Daten.</li> <li> 2: Der spezifizierte
	 *                              Datenverteiler ist der Zustndige fr die Daten, allerdings sind die notwendigen Rechte nicht vorhanden.</li> <li> 3: Mehr als
	 *                              eine Quelle im System angemeldet.</li> </ul>
	 *
	 * @return eine ausgehende Anmeldung
	 */
	OutSubscription newOutSubscription(long targetTransmitterId, long transmitterToConsider[], byte receip) {
		final OutSubscription outSubscription = new OutSubscription();
		outSubscription.targetTransmitterId = targetTransmitterId;
		outSubscription.transmitterToConsider = transmitterToConsider;
		outSubscription.receip = receip;
		return outSubscription;
	}
}
