/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.kex.tls.osi2osi3.
 * 
 * de.bsvrz.kex.tls.osi2osi3 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.kex.tls.osi2osi3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi2osi3; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.kex.tls.osi2osi3.osi2.api;

import java.util.*;

/**
 * Klasse mit Default-Implementierungen einzelner Methoden der Schnittstelle fr Protokolle der Sicherungsschicht (OSI 2).
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5114 $
 */
public abstract class AbstractDataLinkLayer implements DataLinkLayer {

	private final LinkedList _dataLinkLayerListeners = new LinkedList();

	private int _localAddress = 0;

	private Properties _properties = null;

	private final Object _propertiesSync = new Object();

	public AbstractDataLinkLayer() {

		_properties = new Properties();
	}

	abstract protected String getDefaultProperty(String name);

	public String getProperty(String name) {
		synchronized(_propertiesSync) {
			String value = (_properties != null) ? _properties.getProperty(name) : null;
			return (value != null) ? value : getDefaultProperty(name);
		}
	}

	public void setProperties(Properties properties) {
		synchronized(_propertiesSync) {
			_properties = properties;
		}
	}

	public final void addEventListener(DataLinkLayerListener dataLinkLayerListener) {
		synchronized(_dataLinkLayerListeners) {
			_dataLinkLayerListeners.add(dataLinkLayerListener);
		}
	}

	public final void removeEventListener(DataLinkLayerListener dataLinkLayerListener) {
		synchronized(_dataLinkLayerListeners) {
			_dataLinkLayerListeners.remove(dataLinkLayerListener);
		}
	}

	protected final void notifyEvent(DataLinkLayerEvent event) {
		synchronized(_dataLinkLayerListeners) {
			Iterator iterator = _dataLinkLayerListeners.iterator();
			while(iterator.hasNext()) {
				((DataLinkLayerListener)iterator.next()).handleDataLinkLayerEvent(event);
			}
		}
	}

	protected final void notifyEvent(DataLinkLayer.Link link, DataLinkLayerEvent.Type type, Object data) {
		notifyEvent(new DataLinkLayerEvent(this, link, type, data));
	}

	public void setLocalAddress(int port) {
		_localAddress = port;
	}

	public int getLocalAddress() {
		return _localAddress;
	}

	public abstract static class Link implements DataLinkLayer.Link {

		protected final int _remoteAddress;

		protected final Object _linkLock = new Object();

		protected LinkState _linkState;

		public Link(int remoteAddress) {
			_remoteAddress = remoteAddress;
		}

		public String toString() {
			return "OSI2-Verbindung von " + getDataLinkLayer() + " nach Kommunikationspartner(" + getRemoteAddress() + "), Zustand: " + getState();
		}

		public int getRemoteAddress() {
			return _remoteAddress;
		}

		public LinkState getState() {
			synchronized(_linkLock) {
				return _linkState;
			}
		}

		protected final void notifyEvent(DataLinkLayerEvent.Type type, Object data) {
			((AbstractDataLinkLayer)getDataLinkLayer()).notifyEvent(this, type, data);
		}
	}
}
