/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2005 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.pat.sysbed.
 * 
 * de.bsvrz.pat.sysbed is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pat.sysbed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pat.sysbed; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.pat.sysbed.dataview;

import de.bsvrz.dav.daf.main.config.AttributeGroup;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Erstellt ein Fenster mit der {@link DataViewPanel OnlineTabelle}. bergebene Daten werden angezeigt.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5052 $
 * @see #addDataset(DataTableObject)
 */
public class ArchiveDataTableView {

	private final DataViewModel _dataViewModel;

	private boolean _isDisposed = false;


	public ArchiveDataTableView(final AttributeGroup attributeGroup) {
		_dataViewModel = new DataViewModel(attributeGroup);
		final DataViewPanel dataViewPanel = new DataViewPanel(_dataViewModel);
		_dataViewModel.addDataViewListener(dataViewPanel);

		final JFrame frame = new JFrame("Streambasierte Archivanfrage");
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		Container pane = frame.getContentPane();
		pane.setLayout(new BorderLayout());
		pane.add(dataViewPanel, BorderLayout.CENTER);

		frame.addWindowListener(
				new WindowAdapter() {
					public void windowClosed(WindowEvent e) {
						_isDisposed = true;          // damit ein Thread abfragen kann, ob das Fenster geschlossen wurde und keine weiteren Daten mehr braucht!
					}
				}
		);
		frame.setSize(1000, 400);
		frame.setVisible(true);
	}

	public void addDataset(final DataTableObject dataTableObject) {
		_dataViewModel.addDatasetBelow(dataTableObject);
	}

	public boolean isDisposed() {
		return _isDisposed;
	}
}
